/*
	The Universe Development Kit
	Copyright (C) 2000  Sean O'Neil
	soneil@home.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Master.h"
#include "GameApp.h"
#include "GameEngine.h"

CGameEngine::CGameEngine()
{
	CLine l(CVector(0.0f, 0.0f, 0.0f), CVector(0.0f, 1.0f, 1.0f));
	CLine l2(CVector(0.0f, 5.0f, 0.0f), CVector(1.0f, 5.0f, 0.0f));
	CVector p(0.0f, 5.0f, 0.0f);
	CVector v = l.ClosestPoint(p);
	float f = (v-p) | l.m_vDir;
	f = v.Distance(p);
	f = l.Distance(l2);

	m_fMaxError = 0.01f;
	m_nPolygonMode = GL_FILL;
	m_bTexture = true;
	m_bInfo = false;

	// Load and initialize the font object
	m_fFont.Init(GetGameApp()->GetHDC());

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	float fAmbient[4] = {0.5f, 0.5f, 0.5f, 0.5f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, fAmbient);
	float fDiffuse[4] = {1.0, 1.0, 1.0, 1.0};
	float fAmbient2[4] = {0.0, 0.0, 0.0, 0.0};
	glLightfv(GL_LIGHT0, GL_DIFFUSE, fDiffuse);
	glLightfv(GL_LIGHT0, GL_AMBIENT, fAmbient2);

	// Initialize the planet, moon, and camera objects
	m_rsPlanet.Init(354786, EARTH_RADIUS, 10.0f, 0.5f);
	m_rsMoon.Init(2376446, MOON_RADIUS, 2.0f, 0.75f);
	m_rsMoon.SetPosition(CVector(0, 0.0f, MOON_ORBIT));
	m_rsMoon.SetVelocity(CVector(-sqrtf(MOON_MASS*GRAVCONST/MOON_ORBIT), 0.0f, 0.0f));
	m_3DCamera.SetParent(&m_rsPlanet);
	m_3DCamera.SetPosition(CVector(2500.0f, 0.0f, MOON_ORBIT+5000.0f));
	//m_3DCamera.SetPosition(CVector(0.0f, 0.0f, EARTH_RADIUS + 2000.0f));
	
	// Initialize the textures to use for planet and moon
	CPixelBuffer pbTemp;
	BuildTexture(&pbTemp, 10.0f);
	m_tPlanet.Init(&pbTemp, true);
	pbTemp.Init(256, 256, 1, GL_LUMINANCE);
	pbTemp.MakeNoise(98469456);
	m_tMoon.Init(&pbTemp, true);
	m_nShadingMode = GL_SMOOTH;
}

CGameEngine::~CGameEngine()
{
}

/* Use to draw a big square with moon's texture for testing
void CGameEngine::RenderFrame(int nMilliseconds)
{
	// Determine the FPS
	static char szFrameCount[20] = {0};
	static int nTime = 0;
	static int nFrames = 0;
	nTime += nMilliseconds;
	if(nTime >= 1000)
	{
		float fFPS = (float)(nFrames * 1000) / (float)nTime;
		sprintf(szFrameCount, "%2.2f FPS", fFPS);
		nTime = nFrames = 0;
	}
	nFrames++;

	// Clear the display buffer and initialize the MODELVIEW matrix
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	gluLookAt(0, 0, -1.75, 0, 0, 0, 0, 1, 0);
	glDisable(GL_LIGHTING);
	m_tMoon.Enable();
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2d(0.0, 0.0);
	glVertex3d(-1, 0.75, 0);
	glTexCoord2d(1.0, 0.0);
	glVertex3d(1, 0.75, 0);
	glTexCoord2d(0.0, 1.0);
	glVertex3d(-1, -0.75, 0);
	glTexCoord2d(1.0, 1.0);
	glVertex3d(1, -0.75, 0);
	glEnd();
	m_tMoon.Disable();
	glPopMatrix();

	// Draw the FPS in the top-left corner
	m_fFont.Begin();
	glColor3d(1.0, 1.0, 1.0);
	m_fFont.SetPosition(10, 460);
	m_fFont.Print(szFrameCount);
	m_fFont.End();
	glFlush();
}
*/

void CGameEngine::RenderFrame(int nMilliseconds)
{
	// Determine the FPS
	static char szFrameCount[20] = {0};
	static int nTime = 0;
	static int nFrames = 0;
	nTime += nMilliseconds;
	if(nTime >= 1000)
	{
		float fFPS = (float)(nFrames * 1000) / (float)nTime;
		sprintf(szFrameCount, "%2.2f FPS", fFPS);
		nTime = nFrames = 0;
	}
	nFrames++;

	//m_rsPlanet.Rotate(CVector(0.0f, 1.0f, 0.0f), 0.001f * nMilliseconds * 0.001f);
	//m_rsMoon.Rotate(CVector(0.0f, 1.0f, 0.0f), -0.001f * nMilliseconds * 0.001f);
	//m_rsPlanet.SetVelocity(CVector(10.0f, 0.0f, 0.0f));
	//m_rsPlanet.UpdatePosition(nMilliseconds);

	// Clear the display buffer and initialize the MODELVIEW matrix
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	CVector vPosition = m_3DCamera.GetPosition();
	HandleInput(nMilliseconds * 0.001f);
	if(m_rsPlanet.Update(&m_3DCamera, m_fMaxError))
	{
		m_3DCamera.SetVelocity(-m_3DCamera.GetVelocity());
		m_3DCamera.SetPosition(vPosition);
	}
	if(m_rsMoon.Update(&m_3DCamera, m_fMaxError))
	{
		m_3DCamera.SetVelocity(-m_3DCamera.GetVelocity());
		m_3DCamera.SetPosition(vPosition);
	}

	glPushMatrix();
	glLoadMatrixf(m_3DCamera.GetViewMatrix());
	float fPosition[4] = {50000.0f, 25000.0f, 50000.0f, 0.0f};
	glLightfv(GL_LIGHT0, GL_POSITION, fPosition);

	// Draw the planet
	if(m_bTexture)
		m_tPlanet.Enable();
	m_rsPlanet.Draw(&m_3DCamera, m_nShadingMode);
	if(m_bTexture)
		m_tPlanet.Disable();

	// Draw the moon
	if(m_bTexture)
		m_tMoon.Enable();
	m_rsMoon.Draw(&m_3DCamera, m_nShadingMode);
	if(m_bTexture)
		m_tMoon.Disable();

	glPopMatrix();

	// Draw the FPS in the top-left corner
	m_fFont.Begin();
	glColor3d(1.0, 1.0, 1.0);
	m_fFont.SetPosition(10, 460);
	m_fFont.Print(szFrameCount);
	
	if(m_bInfo)
	{
		char szBuffer[_MAX_PATH];
		m_fFont.SetPosition(10, 445);
#ifdef USE_ALLOCATOR
		sprintf(szBuffer, "V: %d, T: %d, D: %d", CROAMVertex::Array.GetUsedElementCount(), CROAMTriangle::m_mbTriangle.GetUsedElementCount(), CROAMDiamond::m_mbDiamond.GetUsedElementCount());
#else
		sprintf(szBuffer, "V: %d, T: %d, D: %d", CROAMVertex::Array.GetUsedElementCount(), m_rsPlanet.GetTriangleCount() + m_rsMoon.GetTriangleCount(), m_rsPlanet.GetDiamondCount() + m_rsMoon.GetDiamondCount());
#endif
		m_fFont.Print(szBuffer);
		m_fFont.SetPosition(10, 430);
		sprintf(szBuffer, "Earth: %2.2f km", (float)m_3DCamera.DistanceTo(m_rsPlanet) - EARTH_RADIUS);
		m_fFont.Print(szBuffer);
		m_fFont.SetPosition(10, 415);
		sprintf(szBuffer, "Moon: %2.2f km", (float)m_3DCamera.DistanceTo(m_rsMoon) - MOON_RADIUS);
		m_fFont.Print(szBuffer);
		m_fFont.SetPosition(10, 400);
		sprintf(szBuffer, "Speed: %2.2f km/sec", m_3DCamera.GetVelocity().Magnitude());
		m_fFont.Print(szBuffer);
	}
	
	m_fFont.End();
	glFlush();
}

void CGameEngine::HandleInput(float fSeconds)
{
	if(GetKeyState(VK_ADD) & 0x8000)
		m_fMaxError *= 0.9f;
	if(GetKeyState(VK_SUBTRACT) & 0x8000)
		m_fMaxError *= 1.1f;

	// Turn left/right means rotate around the up axis
	if(GetKeyState(VK_NUMPAD6) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetUpAxis(), fSeconds * -0.5f);
	if(GetKeyState(VK_NUMPAD4) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetUpAxis(), fSeconds * 0.5f);

	// Turn up/down means rotate around the right axis
	if(GetKeyState(VK_NUMPAD8) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetRightAxis(), fSeconds * -0.5f);
	if(GetKeyState(VK_NUMPAD2) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetRightAxis(), fSeconds * 0.5f);

	// Roll means rotate around the view axis
	if(GetKeyState(VK_NUMPAD7) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetViewAxis(), fSeconds * -0.5f);
	if(GetKeyState(VK_NUMPAD9) & 0x8000)
		m_3DCamera.Rotate(m_3DCamera.GetViewAxis(), fSeconds * 0.5f);

#define ACCEL	100.0f	// Acceleration rate (km/s*s)
#define DAMP	0.0f	// Damping effect on velocity

	// Handle acceleration keys
	if(GetKeyState(VK_SPACE) & 0x8000)
		m_3DCamera.SetVelocity(CVector(0.0f));	// Full stop
	else
	{
		// Handle camera's acceleration due to gravity from planet
		CVector vGravity = m_3DCamera.VectorTo(m_rsPlanet);	// Get direction vector to center of planet (not normalized)
		float fRatio = 1 / vGravity.MagnitudeSquared();		// fRatio = 1 / distance squared (1 / km2)
		vGravity *= sqrtf(fRatio);							// Normalize vGrativy
		float fGravity = (EARTH_MASS * GRAVCONST) * fRatio;	// Determine acceleration due to gravity (km / sec2)
		fGravity *= fSeconds;								// Determine how much gravity affects the camera this frame (km / sec)
		m_3DCamera.AddVelocity(vGravity * fGravity);		// Adjust the camera's velocity vector

		// Handle camera's acceleration due to gravity from moon
		vGravity = m_3DCamera.VectorTo(m_rsMoon);			// Get direction vector to center of planet (not normalized)
		fRatio = 1 / vGravity.MagnitudeSquared();			// fRatio = 1 / distance squared (1 / km2)
		vGravity *= sqrtf(fRatio);							// Normalize vGrativy
		fGravity = (MOON_MASS * GRAVCONST) * fRatio;		// Determine acceleration due to gravity (km / sec2)
		fGravity *= fSeconds;								// Determine how much gravity affects the camera this frame (km / sec)
		m_3DCamera.AddVelocity(vGravity * fGravity);		// Adjust the camera's velocity vector

		float fBoost = 1.0f;
		if(GetKeyState(VK_CONTROL) & 0x8000)
			fBoost = 100.0f;

		// Thrust forward/reverse affects velocity along the view axis
		if(GetKeyState('W') & 0x8000)
			m_3DCamera.AddVelocity(m_3DCamera.GetViewAxis() * fSeconds * ACCEL * fBoost);
		if(GetKeyState('S') & 0x8000)
			m_3DCamera.AddVelocity(m_3DCamera.GetViewAxis() * fSeconds * -ACCEL * fBoost);

		// Thrust left/right affects velocity along the right axis
		if(GetKeyState('D') & 0x8000)
			m_3DCamera.AddVelocity(m_3DCamera.GetRightAxis() * fSeconds * ACCEL * fBoost);
		if(GetKeyState('A') & 0x8000)
			m_3DCamera.AddVelocity(m_3DCamera.GetRightAxis() * fSeconds * -ACCEL * fBoost);

		// Apply damping (i.e. wind resistance) and update the camera's position
		if(DAMP != 0.0f)
			m_3DCamera.SetVelocity(m_3DCamera.GetVelocity() * (1.0f - (DAMP * fSeconds)));
		m_3DCamera.UpdatePosition(fSeconds);
	}

	// Handle moon's acceleration due to gravity
	CVector vGravity = m_rsMoon.VectorTo(m_rsPlanet);	// Get direction vector to center of planet (not normalized)
	float fRatio = 1 / vGravity.MagnitudeSquared();		// fRatio = 1 / distance squared (1 / km2)
	vGravity *= sqrtf(fRatio);							// Normalize vGrativy
	float fGravity = (EARTH_MASS * GRAVCONST) * fRatio;	// Determine acceleration due to gravity (km / sec2)
	fGravity *= fSeconds;								// Determine how much gravity affects the camera this frame (km / sec)
	m_rsMoon.AddVelocity(vGravity * fGravity);			// Adjust the camera's velocity vector
	m_rsMoon.UpdatePosition(fSeconds);
}
