/*
	The Universe Development Kit
	Copyright (C) 2000  Sean O'Neil
	soneil@home.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Master.h"
#include "PixelBuffer.h"

void CPixelBuffer::Init(int nWidth, int nHeight, int nChannels, int nFormat, int nDataType)
{
	// If the buffer is already initialized to the specified settings, then nothing needs to be done
	if(m_pBuffer && m_nWidth == nWidth && m_nHeight == nHeight && m_nChannels == nChannels && m_nFormat == nFormat && m_nDataType == nDataType)
		return;

	Cleanup();
	m_nWidth = nWidth;
	m_nHeight = nHeight;
	m_nChannels = nChannels;
	m_nFormat = nFormat;
	m_nDataType = nDataType;
	switch(m_nDataType)
	{
		case GL_UNSIGNED_BYTE:
			m_pBuffer = new unsigned char[m_nWidth * m_nHeight * m_nChannels];
			break;
		case GL_UNSIGNED_SHORT:
			m_pBuffer = new unsigned short[m_nWidth * m_nHeight * m_nChannels];
			break;
		default:
			// Not supported yet
			break;
	}
}

void CPixelBuffer::MakeNoise(unsigned long nSeed)
{
	CFractal fractal(2, nSeed, 0.5f, 4.0f);
	float f[2];
	int nDest = 0;
	for(int y=0; y<m_nHeight; y++)
	{
		for(int x=0; x<m_nWidth; x++)
		{
			f[0] = 2.0f * (float)x / (float)m_nWidth;
			f[1] = 2.0f * (float)y / (float)m_nHeight;
			float fValue = fractal.fBm(f, 10);
			((unsigned char *)m_pBuffer)[nDest++] = (unsigned char)Max(0.0f, Min(255.0f, ((fValue+1.0f) * 128.0f)));
		}
	}
}

