// DRGMaterial.h - Created with DR&E AppWizard 1.0B8
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// DRGMaterial.h: declaration of the CDRGMaterial class.
// This class is used to contain one or more material properties, so that objects within
// a hierarchy can have knowledge of and set their materials prior to rendering
// There are 3 key member vars, of which one or more can be valid. THese member vars are
// a D3DMaterial7, and DRGTexture, and a state block, which is a collection of 
// renderstates. Also, there is a flags var used to determine which of the 3 key member 
// vars are valid.
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister. 
// 
////////////////////////////////////////////////////////////////////////////////////////

#define DRGMATERIAL_TEXTURE    1
#define DRGMATERIAL_MATERIAL   2
#define DRGMATERIAL_STATEBLOCK 4


class CDRGMaterial {

public:

	CDRGMaterial();
	virtual ~CDRGMaterial();
	
	HRESULT GetTexture(CDRGTexture **ppTex);
	HRESULT SetTexture(CDRGTexture *pTex);
	HRESULT GetMaterial(LPD3DMATERIAL7 *pMat);
	HRESULT SetMaterial(LPD3DMATERIAL7 pMat);
	HRESULT GetStateBlock(DWORD	*pBlock);
	HRESULT SetStateBlock(DWORD	Block);

	HRESULT Activate(DWORD	dwFlags = (DRGMATERIAL_TEXTURE |
									   DRGMATERIAL_MATERIAL |
									   DRGMATERIAL_STATEBLOCK));
	HRESULT TargetChanged(CDRGWrapper *pDRGWrapper);
	
	// member var used to track which of the key member vars are valid	
	DWORD		m_dwFlags;


protected:

	//The 3 key member vars for types of properties the material could affect
	CDRGTexture		*m_pDRGTexture;
	LPD3DMATERIAL7	m_pD3DMaterial;
	DWORD			m_dwStateBlock;

	//a pointer to the wrapper, to be used during state changes and the like
	CDRGWrapper *m_pWrapper; 

};
