//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#ifndef PIII_BEZIER_PATCH_H__
#define PIII_BEZIER_PATCH_H__

#include "Bezier.h"
#include "PIIIBezierTesselation.h"
#include <fvec.h>
#include "scene.h"
#include "bezierspline.h"	// Added by ClassView

struct PIIIVector3 {
	F32vec4 x,y,z;
};

struct PIIIVector4 {
	F32vec4 w,x,y,z;
};

class CPIIIBezierSpline : public CBezierSpline {
public:
	struct PIIILight {
//		union {
			PIIIVector3	pos,norm;
//			PIIIVector3	norm;
//		};
		PIIIVector3 color;
		CScene::Light::LightType type;
		DWORD stub[3];	// to keep alignment
	};

protected:
	PIIILight *m_lights;
	virtual CBezierPatch *CreatePatch(DWORD *patchIndices);
	void PrepearSpline(CScene *scene) ;
public:
	CPIIIBezierSpline(CPIIIBezierTessalation *tesl);
	~CPIIIBezierSpline();

	friend class CPIIIBezierPatch;
};


class CPIIIBezierPatch : public CBezierPatch {
protected:
	virtual bool	EvalSurface(LPDIRECT3DVERTEXBUFFER7 vb,CScene *scene,LPWORD &indices,DWORD &indicesCnt);
public:
	CPIIIBezierPatch(CBezierSpline *spline,DWORD *indices);

};



class CPIIIBezierSpline2 : public CPIIIBezierSpline {
public:

protected:
	PIIIVector4		*m_p3ProjCtrlPts;
	PIIIVector3		*m_p3WorldCtrlPts;
	virtual CBezierPatch *CreatePatch(DWORD *patchIndices);
	void PrepearSpline(CScene *scene) ;
public:
	virtual void	CreateSplineFromFile(const char *filename);
	CPIIIBezierSpline2(CPIIIBezierTessalation2 *tesl);
	~CPIIIBezierSpline2();
	friend class CPIIIBezierPatch2;
};


class CPIIIBezierSpline3 : public CPIIIBezierSpline {
public:

protected:
	PIIIVector4		*m_p3ProjCtrlPts;
	PIIIVector3		*m_p3WorldCtrlPts;
	virtual CBezierPatch *CreatePatch(DWORD *patchIndices);
	void PrepearSpline(CScene *scene) ;
	void EvalNormals();
public:
	virtual void	CreateSplineFromFile(const char *filename);
	CPIIIBezierSpline3(CPIIIBezierTessalation3 *tesl);
	~CPIIIBezierSpline3();
	friend class CPIIIBezierPatch3;
};

class CPIIIBezierSpline4 : public CPIIIBezierSpline {
public:

protected:
	PIIIVector4		*m_p3ProjCtrlPts;
//	PIIIVector3		*m_p3WorldCtrlPts;
	virtual CBezierPatch *CreatePatch(DWORD *patchIndices);
	void PrepearSpline(CScene *scene) ;
	void EvalNormals();
public:
	virtual void	CreateSplineFromFile(const char *filename);
	CPIIIBezierSpline4(CPIIIBezierTessalation3 *tesl);
	~CPIIIBezierSpline4();
	friend class CPIIIBezierPatch4;
};

class CPIIIBezierPatch2 : public CPIIIBezierPatch {
protected:
	DWORD		m_p3ProjCtrlPtsOffs[4][4];
	DWORD		m_p3WorldCtrlPtsOffs[4][4];
	virtual bool	EvalSurface(LPDIRECT3DVERTEXBUFFER7 vb,CScene *scene,LPWORD &indices,DWORD &indicesCnt);
public:
	CPIIIBezierPatch2(CBezierSpline *spline,DWORD *indices);

};

class CPIIIBezierPatch3 : public CPIIIBezierPatch {
private:
		static PIIIVector3		m_SamplePointsNormals[4];
protected:
	DWORD		m_p3ProjCtrlPtsOffs[4][4];
	DWORD		m_p3WorldCtrlPtsOffs[4][4];
	virtual bool	EvalSurface(LPDIRECT3DVERTEXBUFFER7 vb,CScene *scene,LPWORD &indices,DWORD &indicesCnt);
	void EvalNormals();
public:
	CPIIIBezierPatch3(CBezierSpline *spline,DWORD *indices);

};


class CPIIIBezierPatch4 : public CPIIIBezierPatch {
private:
		static PIIIVector3		m_SamplePointsColors[4];
protected:
	DWORD		m_p3ProjCtrlPtsOffs[4][4];
	DWORD		m_p3WorldCtrlPtsOffs[4][4];
	virtual bool	EvalSurface(LPDIRECT3DVERTEXBUFFER7 vb,CScene *scene,LPWORD &indices,DWORD &indicesCnt);
	Vector3 EvalColor(CScene *scene,Vector4& pos, Vector3& norm);
	void EvalColors(CScene *scene);
public:
	CPIIIBezierPatch4(CBezierSpline *spline,DWORD *indices);

};


#endif

