//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#include "pIIIBezierTesselation.h"
#include "util.h"
#include <fvec.h>

void CPIIIBezierTessalation::GenerateTessalationLevel(TessLevel *level) 
{
	DWORD i,j;
	DWORD teslLevel = level->m_level;
	DWORD triCnt = (teslLevel - 1)*(teslLevel - 1)*2;
	level->m_indicesCount = 3*triCnt;
	
	alignedFree(level->m_indices);
	alignedFree(level->m_teslValues);

	LPWORD idx = level->m_indices = (LPWORD)alignedAlloc(sizeof(WORD)*level->m_indicesCount);

	for (i=1;i<teslLevel;i++) {
		for (j=1;j<teslLevel;j++) {
			idx[1] = (i-1)*teslLevel + j;
			idx[0] = idx[1] - 1;
			idx[5] = idx[1] + teslLevel;
			idx[4] = idx[1];
			idx[2] = idx[5] - 1;
			idx[3] = idx[2];
			idx += 6;
		}
	}

	DWORD teslSkip = level->m_teslValuesSkip = (level->m_level + 3) & 0xffffffc;
	level->m_teslValues = (float *)alignedAlloc(sizeof(F32vec4)*teslSkip*8);

	F32vec4 *coeffs = (F32vec4 *)level->m_teslValues;

	coeffs[0] = F32vec4(1.0f);
	coeffs[1] = F32vec4(0.0f);
	coeffs[2] = F32vec4(0.0f);
	coeffs[3] = F32vec4(0.0f);
	coeffs[4] = F32vec4(-3.0f);
	coeffs[5] = F32vec4(3.0f);
	coeffs[6] = F32vec4(0.0f);
	coeffs[7] = F32vec4(0.0f);

	teslLevel--;

	coeffs[0 + teslLevel*8] = F32vec4(0.0f);
	coeffs[1 + teslLevel*8] = F32vec4(0.0f);
	coeffs[2 + teslLevel*8] = F32vec4(0.0f);
	coeffs[3 + teslLevel*8] = F32vec4(1.0f);
	coeffs[4 + teslLevel*8] = F32vec4(0.0f);
	coeffs[5 + teslLevel*8] = F32vec4(0.0f);
	coeffs[6 + teslLevel*8] = F32vec4(-3.0f);
	coeffs[7 + teslLevel*8] = F32vec4(3.0f);

	float skip = 1.0f/(float)(teslLevel);
	float t = skip;

	coeffs += 8;

	for (i=1;i<teslLevel;i++) {
		float one_minus_t = 1.0f - t;
		float factor = t/one_minus_t;
		float last;
		coeffs[0] = F32vec4(last = one_minus_t*one_minus_t*one_minus_t);
		last *= factor;
		coeffs[1] = F32vec4(last * 3.0f);
		last *= factor;
		coeffs[2] = F32vec4(last * 3.0f);
		coeffs[3] = F32vec4(last * factor);


		coeffs[4] = F32vec4(-3*one_minus_t*one_minus_t);
		coeffs[5] = F32vec4(3*one_minus_t*one_minus_t - 6*t*one_minus_t);
		coeffs[6] = F32vec4(6*t*one_minus_t - 3*t*t);
		coeffs[7] = F32vec4(3*t*t);

		coeffs += 8;
		t += skip;
	}		

}




void CPIIIBezierTessalation2::GenerateTessalationLevel(TessLevel *level) 
{
	DWORD i,j;
	DWORD teslLevel = level->m_level;
	DWORD triCnt = (teslLevel - 1)*(teslLevel - 1)*2;
	level->m_indicesCount = 3*triCnt;
	
	alignedFree(level->m_indices);
	alignedFree(level->m_teslValues);

	LPWORD idx = level->m_indices = (LPWORD)alignedAlloc(sizeof(WORD)*level->m_indicesCount);

	for (i=1;i<teslLevel;i++) {
		for (j=1;j<teslLevel;j++) {
			idx[1] = (i-1)*teslLevel + j;
			idx[0] = idx[1] - 1;
			idx[5] = idx[1] + teslLevel;
			idx[4] = idx[1];
			idx[2] = idx[5] - 1;
			idx[3] = idx[2];
			idx += 6;
		}
	}

	DWORD teslSkip = level->m_teslValuesSkip = (level->m_level*level->m_level + 3) & 0xffffffc;
	level->m_teslValues = (float *)alignedAlloc(sizeof(float)*teslSkip*8*2);

	F32vec4 *coeffs = (F32vec4 *)level->m_teslValues;

	DWORD u,v;
	DWORD mod = 0;
	float fu,fv = 0.0f;
	float delta = 1.0f/(teslLevel - 1);

	for (v=0;v<teslLevel;v++) {
		fu = 0.0f;
		float one_minus_fv = 1.0f - fv;
		for (u=0;u<teslLevel;u++) {
			float one_minus_fu = 1.0f - fu;
			coeffs[0][mod] = one_minus_fu*one_minus_fu*one_minus_fu; 
			coeffs[1][mod] = 3*one_minus_fu*one_minus_fu*fu;
			coeffs[2][mod] = 3*one_minus_fu*fu*fu;
			coeffs[3][mod] = fu*fu*fu;
			coeffs[4][mod] = -3*one_minus_fu*one_minus_fu;
			coeffs[5][mod] = 3*one_minus_fu*one_minus_fu - 6*fu*one_minus_fu;
			coeffs[6][mod] = 6*fu*one_minus_fu - 3*fu*fu;
			coeffs[7][mod] = 3*fu*fu;
			fu += delta;
			
			coeffs[ 8][mod] = one_minus_fv*one_minus_fv*one_minus_fv; 
			coeffs[ 9][mod] = 3*one_minus_fv*one_minus_fv*fv;
			coeffs[10][mod] = 3*one_minus_fv*fv*fv;
			coeffs[11][mod] = fv*fv*fv;
			coeffs[12][mod] = -3*one_minus_fv*one_minus_fv;
			coeffs[13][mod] = 3*one_minus_fv*one_minus_fv - 6*fv*one_minus_fv;
			coeffs[14][mod] = 6*fv*one_minus_fv - 3*fv*fv;
			coeffs[15][mod] = 3*fv*fv;

			mod++;
			if (mod == 4) {
				coeffs += 16;
				mod = 0;
			}

		}
		fv += delta;
	}		

}


void CPIIIBezierTessalation3::GenerateTessalationLevel(TessLevel *level) 
{
	DWORD i,j;
	DWORD teslLevel = level->m_level;
	DWORD triCnt = (teslLevel - 1)*(teslLevel - 1)*2;
	level->m_indicesCount = 3*triCnt;
	
	alignedFree(level->m_indices);
	alignedFree(level->m_teslValues);

	LPWORD idx = level->m_indices = (LPWORD)alignedAlloc(sizeof(WORD)*level->m_indicesCount);

	for (i=1;i<teslLevel;i++) {
		for (j=1;j<teslLevel;j++) {
			idx[1] = (i-1)*teslLevel + j;
			idx[0] = idx[1] - 1;
			idx[5] = idx[1] + teslLevel;
			idx[4] = idx[1];
			idx[2] = idx[5] - 1;
			idx[3] = idx[2];
			idx += 6;
		}
	}

	DWORD teslSkip = level->m_teslValuesSkip = (level->m_level*level->m_level + 3) & 0xffffffc;
	level->m_teslValues = (float *)alignedAlloc(sizeof(float)*teslSkip*5*2);

	F32vec4 *coeffs = (F32vec4 *)level->m_teslValues;

	DWORD u,v;
	DWORD mod = 0;
	float fu,fv = 0.0f;
	float delta = 1.0f/(teslLevel - 1);

	for (v=0;v<teslLevel;v++) {
		fu = 0.0f;
		float one_minus_fv = 1.0f - fv;
		for (u=0;u<teslLevel;u++) {
			float one_minus_fu = 1.0f - fu;
			coeffs[0][mod] = one_minus_fu*one_minus_fu*one_minus_fu; 
			coeffs[1][mod] = 3*one_minus_fu*one_minus_fu*fu;
			coeffs[2][mod] = 3*one_minus_fu*fu*fu;
			coeffs[3][mod] = fu*fu*fu;
			coeffs[4][mod] = fu;
			fu += delta;
			
			coeffs[ 5][mod] = one_minus_fv*one_minus_fv*one_minus_fv; 
			coeffs[ 6][mod] = 3*one_minus_fv*one_minus_fv*fv;
			coeffs[ 7][mod] = 3*one_minus_fv*fv*fv;
			coeffs[ 8][mod] = fv*fv*fv;
			coeffs[ 9][mod] = fv;

			mod++;
			if (mod == 4) {
				coeffs += 10;
				mod = 0;
			}

		}
		fv += delta;
	}		

}
