// firetest.cpp - Created with DR&E AppWizard 1.0B8
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// firetest.cpp: implementation the firetest class.
// This class is the "blank" class created by the DRG 3D Appwizard, where app specific
// stuff gets placed
//
// In this case, it's used to do our fire/feedback demo. A series of randomly colored
// points is drawn to the bottom of a texture map. A couple spheres are also rendered
// to the texture map, using a noise texture. This is repeated from frame to frame, but
// in each frame, the previous frame is used as the backdrop. The UV coordinates for 
// the triangles making up the backdrop are offset slightly, to result in the image 
// scrolling upwards. The bilinear filtering and vertex color modulation, smooth and
// fade the scrolling image over time. 
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////

#include "DRGShell.h"


// -----------------------------------------------------------------------
// CFiretest::CFiretest() 
// The Firetest class constructor initializes some of it's member vars (which
// are instances of the DRGMatrix class
// -----------------------------------------------------------------------
CFiretest::CFiretest()
{
	m_MxWorld.SetIdentity();
	//set the camera at 0,0,-5; the target at 0,0,0 and the up vector at 0,1,0
	m_MxView.SetView( DRGVector( 0, 0, -3 ), DRGVector( 0, 0, 0 ), DRGVector( 0, 1, 0 ) );
	//set the projection matrix to 1.57 field of view (radians), 1.33 aspect ratio 
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxProjection.SetProjection( 1.57f, 1.333f, 1.0f, 100.0f );

	
	m_pDefaultTexture = NULL;

	m_pDefaultMaterial = NULL;

	m_pFireTextureArray[0] = NULL;
	m_pFireTextureArray[1] = NULL;
	m_pFireTextureArray[2] = NULL;

	m_pSampleRootObj = NULL;
	m_pSampleSphereObj1 = NULL;
	m_pSampleSphereObj2 = NULL;
	
	m_iSceneTarget = DRAWSCENETOBOTH;

}

// -----------------------------------------------------------------------
// CFiretest::~CFiretest()
// destructor does nothing for now.
// -----------------------------------------------------------------------
CFiretest::~CFiretest()
{

}

// -----------------------------------------------------------------------
// HRESULT CFiretest::InitWorld( CDRGWrapper *pWrapper )
//
// -----------------------------------------------------------------------
HRESULT CFiretest::InitWorld( CDRGWrapper *pWrapper )
{
	// Make local copy of the pointer to the wrapper class
	//
	m_pWrapper = pWrapper;

	// function to setup the verts for two triangles that make a background
	SetupSampleBackgroundVerts();
	

	//init some common elements in the array of verts that will be used for 
	// the random fire bits
	for (int i=0; i<TEXSIZE;i++)
	{
		m_FireVerts[i].sx = (float)i;
		m_FireVerts[i].sy = (float)TEXSIZE - 1.0f;
		m_FireVerts[i].sz = 0.99f;
		m_FireVerts[i].rhw = 0.00001f;
		m_FireVerts[i].dcSpecular = 0;		
		m_FireVerts[i].tu = 0.0f ;	m_FireVerts[i].tv = 0.0f;	
		m_FireVerts[i].dcColor = D3DRGB( 1.0f, 0.0f, 0.0f );	
	}		
	
	// init two render target textures. My texture wrapper class isn't set up to
	// create these as a flipping chain, but that would be the ideal method of doing
	// this

	m_pFireTextureArray[0] = new CDRGTexture;
	if (m_pFireTextureArray[0]->CreateTexture(pWrapper,TEXSIZE,TEXSIZE,16,0,DRGTEXTR_RENDERTARGET | DRGTEXTR_32BITSPERPIXEL,NULL) == S_OK)
		m_pFireTextureArray[0]->Restore(pWrapper->m_pD3DDevice7);

	m_pFireTextureArray[1] = new CDRGTexture;

	if (m_pFireTextureArray[1]->CreateTexture(pWrapper,TEXSIZE,TEXSIZE,16,0,DRGTEXTR_RENDERTARGET | DRGTEXTR_32BITSPERPIXEL,NULL) == S_OK)
		m_pFireTextureArray[1]->Restore(pWrapper->m_pD3DDevice7);

	//create my object hierarchy. It's just two spheres with two dummy frames
	m_pSampleRootObj		= new CDRGObject;
	m_pSampleRootObj->Init(pWrapper, NULL, NULL, 0, 0,						 
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f),  DRGVector(0.0f),  0.0f,
					0x0);

	m_pSampleSphereObj1		= new CDRGObjectSphere;
	m_pSampleSphereObj1->Init(pWrapper, 1.0f, 24,
					DRGVector(0.0f,0.0f,0.0f), DRGVector(0.5f), DRGVector(0.5f,1.5f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 1.5f,
					0x0);
	m_pSampleRootObj->AddChild(m_pSampleSphereObj1);


	m_pSampleDummyObj		= new CDRGObject;
	m_pSampleDummyObj->Init(pWrapper, NULL, NULL, 0, 0,
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.0f), DRGVector(1.0f,0.5f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 2.0f,
					0x0);
	m_pSampleSphereObj1->AddChild(m_pSampleDummyObj);


	m_pSampleSphereObj2		= new CDRGObjectSphere;
	m_pSampleSphereObj2->Init(pWrapper, 1.0f, 12,
					DRGVector(2.0f,2.0f,0.0f), DRGVector(0.3f), DRGVector(1.0f,0.5f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 1.0f,
					0x0);
	m_pSampleDummyObj->AddChild(m_pSampleSphereObj2);


	// this is just a texture to add noise to the smoke effect the sphere generatesw
	m_pDefaultTexture = new CDRGTexture;

	if( m_pDefaultTexture->CreateTexture( CBMPLoader( CFileStream( "Images\\test.bmp" ) ),
						  (DWORD)0, (DWORD)0) == S_OK )
		m_pDefaultTexture->Restore(pWrapper->m_pD3DDevice7);


	//now we call target changed, which sets up device dependant items
	TargetChanged();
	
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CFiretest::DestroyWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CFiretest::DestroyWorld( void )
{
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CFiretest::UpdateWorld( DWORD dwTime, int InputFlag )
//
// -----------------------------------------------------------------------
HRESULT CFiretest::UpdateWorld()
{

	//Call the Update method of the object at the root of our hierarchy,
	// specifying to propogate the update down the hierarchy. This will
	// update all the objects positions/orientations according to their
	// velocities. By specifying the frame time, our animation speed is
	// independant of frame rate
	if(m_pSampleRootObj)
		m_pSampleRootObj->Update(m_pWrapper->m_fFrameTime, true);

	float red, green;
	float divisor = 1.0f/(3.0f*(float)RAND_MAX);
	//update our random fire pixels
	for (int i=0; i<TEXSIZE/4;i++) //unrolled  to share values
	{
		//random between 0.66 and 1
		red = 0.66f + (float)rand()  * divisor;// / (3.0f*(float)RAND_MAX);
		//random between 0 and 0.66
		green = (float)rand() * divisor * 2.0f;// / (1.5f*(float)RAND_MAX);

		DWORD color = D3DRGB( red, green, 0.0f);
						
		m_FireVerts[i*4].dcColor = color;
		m_FireVerts[i*4+1].dcColor = color;
		m_FireVerts[i*4+2].dcColor = color;
		m_FireVerts[i*4+3].dcColor = color;
	}


	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CFiretest::RenderWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CFiretest::RenderWorld( void )
{

	HRESULT hr;

	//We need to call endscene first because our wrapper class called beginscene
	// before calling FireTest's RenderWorld() function. This results in an empty begin/endscene
	// pair, which is a little inefficient, but it's OK for this demo
	m_pWrapper->m_pD3DDevice7->EndScene();

	//////////////
	//save off the old viewport and set up the new one for when we call SRT
	D3DVIEWPORT7	VPTemp, VPOld;

	m_pWrapper->m_pD3DDevice7->GetViewport(&VPOld);
	memcpy(&VPTemp,&VPOld,sizeof(D3DVIEWPORT7));

	VPTemp.dwWidth = TEXSIZE;
	VPTemp.dwHeight = TEXSIZE;
	
	//////////////
	// Draw on texture 1, using texture 2. This will cause the previous frame to be
	// offset slightly and filtered
	//
	// to do this, we need to point the render target to the texture, adjust the 
	// viewport size, etc
	m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pFireTextureArray[0]->ptexTexture, 0);
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPTemp);
	
	D3DRECT rcClear;
	
	rcClear.x1 = 0;
	rcClear.y1 = 0;
	rcClear.x2 = m_pFireTextureArray[0]->m_ddsd.dwWidth;
	rcClear.y2 = m_pFireTextureArray[0]->m_ddsd.dwHeight;
	//clear the buffer
	hr = m_pWrapper->m_pD3DDevice7->Clear( 1, &rcClear, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
							0x0, 1.0f, 0 );


	// do a beginscene, set the texture to the previous frames scene, and draw our primitives
	m_pWrapper->m_pD3DDevice7->BeginScene();
	m_pWrapper->m_pD3DDevice7->SetTexture(m_pFireTextureArray[1]->dwStage,m_pFireTextureArray[1]->ptexTexture);

	hr = m_pWrapper->m_pD3DDevice7->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
			m_TextureBackgroundVerts, NUMSAMPLEBACKGROUNDVERTS, 0);		

	// switch off textures, and draw points to bottom row of the screen to do our 'noise'
	// based fire. 
	m_pWrapper->m_pD3DDevice7->SetTexture(m_pFireTextureArray[1]->dwStage,NULL);
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, FALSE);

	hr = m_pWrapper->m_pD3DDevice7->DrawPrimitive( D3DPT_POINTLIST, D3DFVF_TLVERTEX,
			m_FireVerts, TEXSIZE, 0);		

	// set the texture to the one we are using for our 'smoky sphere'
	hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pDefaultTexture->dwStage, m_pDefaultTexture->ptexTexture);	

	// the if statement here is because we all for drawing it only to either of two buffers, or to both
	if ((m_iSceneTarget == DRAWSCENETOBOTH) || (m_iSceneTarget == DRAWSCENETOTEXTURE))
	{
		if(m_pSampleRootObj)
			hr = m_pSampleRootObj->Render(m_MxWorld,true);
	}

	m_pWrapper->m_pD3DDevice7->EndScene();


	////////////
	// Now that we've ended the scene where we are rendering to the texture,
	// we can swap the texture pointers in order to just repeat the above code on
	// the next frame. However, we have to make sure to render the scene to the
	// back buffer in the meantime, so that we can see it 
	
	//flip the two, by hand
	m_pFireTextureArray[2] = m_pFireTextureArray[0];
	m_pFireTextureArray[0] = m_pFireTextureArray[1];
	m_pFireTextureArray[1] = m_pFireTextureArray[2];
	
	//point back to back buffer, re-init viewport
	m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pWrapper->m_pBackBuffer, 0);
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPOld);

	//clear the back buffer
	m_pWrapper->Clear( D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER );
	//render the scene
	m_pWrapper->m_pD3DDevice7->BeginScene();
	
	// Draw two triangles that make up the background
	hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pFireTextureArray[0]->dwStage, m_pFireTextureArray[0]->ptexTexture);	
	hr = m_pWrapper->m_pD3DDevice7->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
		m_SampleBackgroundVerts, NUMSAMPLEBACKGROUNDVERTS, 0);		
	

	// Now I set the texture to null so that when we draw the spheres to the back buffer, 
	// are just white (but the 'smoky' versions are still leaving trails)
	hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pDefaultTexture->dwStage, NULL);	
	
	if ((m_iSceneTarget == DRAWSCENETOBOTH) || (m_iSceneTarget == DRAWSCENETOBACKBUFFER))
	{
		if(m_pSampleRootObj)
			hr = m_pSampleRootObj->Render(m_MxWorld,true);
	}
	
	return S_OK;
}


// -----------------------------------------------------------------------
// HRESULT CFiretest::TargetChanged( void )
//
// -----------------------------------------------------------------------
HRESULT CFiretest::TargetChanged( void )
{
	HRESULT hr;
	//
	// This indicates that the devices of the DRGWrapper class have changed.
	// If you have anything that depends on the current rendering device, you'll
	// have to re-initialize it here
	//
	m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_DITHERENABLE, TRUE );

	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxProjection.m_Mx );

	//
	// re-setup background for new screen res
	//
	SetupSampleBackgroundVerts();

	SetupTextureBackgroundVerts((float)TEXSIZE,(float)TEXSIZE);

	hr = SetupDependantItems();

	return hr;
}

HRESULT CFiretest::SetupDependantItems(void)
{
	HRESULT hr;


	//restore texture
	if (m_pFireTextureArray[0])
		m_pFireTextureArray[0]->Restore(m_pWrapper->m_pD3DDevice7);
	
	if (m_pFireTextureArray[1])
		m_pFireTextureArray[1]->Restore(m_pWrapper->m_pD3DDevice7);
	
	if (m_pDefaultTexture)
		m_pDefaultTexture->Restore(m_pWrapper->m_pD3DDevice7);

	if(m_pSampleRootObj)
		hr = m_pSampleRootObj->TargetChanged(m_pWrapper, true);

	//create a default material
	D3DMATERIAL7	*ptempmat = new D3DMATERIAL7;
	ZeroMemory( ptempmat, sizeof(D3DMATERIAL7) );
  
	ptempmat->ambient.r = 0.7f;    ptempmat->ambient.g = 0.7f;    ptempmat->ambient.b = 0.7f;
	ptempmat->diffuse.r = 1.0f;    ptempmat->diffuse.g = 1.0f;    ptempmat->diffuse.b = 1.0f;
	ptempmat->specular.r = 1.0f;    ptempmat->specular.g = 1.0f;    ptempmat->specular.b = 1.0f;

	if( FAILED( hr = m_pWrapper->m_pD3DDevice7->SetMaterial(ptempmat) ) )
		return hr;



	m_pDefaultMaterial = new CDRGMaterial;
		
	m_pDefaultMaterial->SetMaterial(ptempmat);
	m_pDefaultMaterial->SetTexture(m_pDefaultTexture);
	
	// Set up a default light
    D3DLIGHT7 light;
    ZeroMemory( &light, sizeof(light) );
    light.dltType       = D3DLIGHT_DIRECTIONAL;
    light.dcvDiffuse.r  = 0.8f;
    light.dcvDiffuse.g  = 0.8f;
    light.dcvDiffuse.b  = 0.8f;
    light.dvRange       = D3DLIGHT_RANGE_MAX;
    
    light.dvDirection    = D3DVECTOR( 0.2f, 0.2f, 0.9f);

    // Set the light
    if( FAILED(m_pWrapper->m_pD3DDevice7->SetLight( 0, &light )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->LightEnable( 0, TRUE )))
		return hr;

    if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_LIGHTING, TRUE )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_AMBIENT, 0xff333333 ) ) )
		return hr;
	
	if (FAILED(hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_COLORVERTEX, TRUE) ))
		return hr;

	return S_OK;
}

// -----------------------------------------------------------------------
// void CFiretest::SetupBackGroundVerts(float scale)
//
//   this is just a function that sets up a pool of member verts as an axis
//   at the center of the world.
// -----------------------------------------------------------------------

void CFiretest::SetupSampleBackgroundVerts()
{
	memset( m_SampleBackgroundVerts, 0, sizeof( D3DTLVERTEX ) * NUMSAMPLEBACKGROUNDVERTS );

	float width = float( m_pWrapper->m_dwRenderWidth);
	float height = float( m_pWrapper->m_dwRenderHeight);

	float margin = 0.01f;

	m_SampleBackgroundVerts[0].sx = 0.0f + margin;	m_SampleBackgroundVerts[0].sy = 0.0f + margin;	
	m_SampleBackgroundVerts[1].sx = width - margin;	m_SampleBackgroundVerts[1].sy = height - margin;
	m_SampleBackgroundVerts[2].sx = 0.0f + margin;	m_SampleBackgroundVerts[2].sy = height - margin;	
	m_SampleBackgroundVerts[3].sx = 0.0f + margin;	m_SampleBackgroundVerts[3].sy = 0.0f + margin;	
	m_SampleBackgroundVerts[4].sx = width - margin;	m_SampleBackgroundVerts[4].sy = 0.0f + margin;	
	m_SampleBackgroundVerts[5].sx = width - margin;	m_SampleBackgroundVerts[5].sy = height - margin;	
		
	// common elements
	for (int i=0;i<NUMSAMPLEBACKGROUNDVERTS;i++)
	{
		m_SampleBackgroundVerts[i].sz = 0.99999f;
		m_SampleBackgroundVerts[i].rhw = 0.00001f;
		m_SampleBackgroundVerts[i].dcSpecular = 0;
		m_SampleBackgroundVerts[i].dcColor = D3DRGB( 1.0f, 1.0f, 1.0f );
	}

	m_SampleBackgroundVerts[0].tu = 0.0f;	m_SampleBackgroundVerts[0].tv = 0.0f;	
	m_SampleBackgroundVerts[1].tu = 1.0f;	m_SampleBackgroundVerts[1].tv = 1.0f;	
	m_SampleBackgroundVerts[2].tu = 0.0f;	m_SampleBackgroundVerts[2].tv = 1.0f;	
	m_SampleBackgroundVerts[3].tu = 0.0f;	m_SampleBackgroundVerts[3].tv = 0.0f;	
	m_SampleBackgroundVerts[4].tu = 1.0f;	m_SampleBackgroundVerts[4].tv = 0.0f;	
	m_SampleBackgroundVerts[5].tu = 1.0f;	m_SampleBackgroundVerts[5].tv = 1.0f;	

	
	return;

}



// -----------------------------------------------------------------------
// void CFiretest::KeyUp(WPARAM wParam)
//
//   Keyboard keypress handler. 
// -----------------------------------------------------------------------
void CFiretest::KeyDown(WPARAM wParam)
{
	//we need a variable to modify our rotation speed with frame rate	
	float scaler = 1.0f;

	//if exiting, the wrapper could be invalid, but we don't care, since we will exit anyway
	if (wParam == 'X') 
		return;

	if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
	{
		if (m_pWrapper->m_fFrameTime >0.0f)
			scaler = m_pWrapper->m_fFrameTime;  
	}
		
	switch (wParam) 
	{
	case VK_UP:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, -3.14159f*scaler );
		break;
	
	case VK_DOWN:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler );
		break;
	
	case VK_LEFT:
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, -3.14159f*scaler );
		break;
	
	case VK_RIGHT:	
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler );
		break;
	case 'T':
		m_iSceneTarget = m_iSceneTarget++;
		if (m_iSceneTarget == 3)
			m_iSceneTarget = 0;
		break;

	}

	if (m_pWrapper)
	{
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	}

	
	return;
}

void CFiretest::KeyUp(WPARAM wParam)
{		
	switch (wParam) 
	{
		//add code here to handle key releases, if you need them (as in remmed out example)
	case VK_UP:
		//OutputDebugString("The up arrow was released;");			
		break;
	}
	
	return;
}


void CFiretest::MouseLeftUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CFiretest::MouseLeftDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CFiretest::MouseRightUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CFiretest::MouseRightDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CFiretest::MouseMove(WPARAM wParam, LPARAM lParam)
{
	if ( wParam & MK_LBUTTON )	// Left button down plus mouse move == rotate
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 2.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (x - m_iMousePrevX)
		{
			m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler*(float)(x - m_iMousePrevX) );
		}
			
		if (y - m_iMousePrevY)
		{
			m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler*(float)(y - m_iMousePrevY) );
		}

		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}
	else if ( wParam & MK_RBUTTON )	// Right button down plus mouse move == Zoom camera in/out
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 5.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (y - m_iMousePrevY)
		{
			DRGMatrix Temp(true);

			Temp.Translate( 0,0,scaler * (y - m_iMousePrevY) );
			m_MxView.PostMultiply( Temp );
		}

		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}
	else
	{
		//need to update these, or the next time someone clicks, the delta is massive
		m_iMousePrevX = LOWORD(lParam);
		m_iMousePrevY = HIWORD(lParam);	
	}
	return;
}

void CFiretest::SetupTextureBackgroundVerts(float hsize, float vsize)
{
	memset( m_TextureBackgroundVerts, 0, sizeof( D3DTLVERTEX ) * NUMSAMPLEBACKGROUNDVERTS );

	float width = hsize;
	float height = vsize;

	float margin = 0.00f;

	m_TextureBackgroundVerts[0].sx = 0.0f + margin;	m_TextureBackgroundVerts[0].sy = 0.0f + margin;	
	m_TextureBackgroundVerts[1].sx = width - margin;	m_TextureBackgroundVerts[1].sy = height - margin;
	m_TextureBackgroundVerts[2].sx = 0.0f + margin;	m_TextureBackgroundVerts[2].sy = height - margin;	
	m_TextureBackgroundVerts[3].sx = 0.0f + margin;	m_TextureBackgroundVerts[3].sy = 0.0f + margin;	
	m_TextureBackgroundVerts[4].sx = width - margin;	m_TextureBackgroundVerts[4].sy = 0.0f + margin;	
	m_TextureBackgroundVerts[5].sx = width - margin;	m_TextureBackgroundVerts[5].sy = height - margin;	
		
	// common elements
	for (int i=0;i<NUMSAMPLEBACKGROUNDVERTS;i++)
	{
		m_TextureBackgroundVerts[i].sz = 0.9999f;
		m_TextureBackgroundVerts[i].rhw = 0.0001f;
		m_TextureBackgroundVerts[i].dcSpecular = 0;
		m_TextureBackgroundVerts[i].dcColor = D3DRGB( 0.99f, 0.99f, 0.99f );
	}


	float fScroll = 1.25f/(float)TEXSIZE;
	float fTexMargin = 0.5f/(float)TEXSIZE; 

	m_TextureBackgroundVerts[0].tu = 0.0f + fTexMargin ;	m_TextureBackgroundVerts[0].tv = 0.0f + fScroll;	
	m_TextureBackgroundVerts[1].tu = 1.0f + fTexMargin ;	m_TextureBackgroundVerts[1].tv = 1.0f + fScroll;	
	m_TextureBackgroundVerts[2].tu = 0.0f + fTexMargin ;	m_TextureBackgroundVerts[2].tv = 1.0f + fScroll;	
	m_TextureBackgroundVerts[3].tu = 0.0f + fTexMargin ;	m_TextureBackgroundVerts[3].tv = 0.0f + fScroll;	
	m_TextureBackgroundVerts[4].tu = 1.0f + fTexMargin ;	m_TextureBackgroundVerts[4].tv = 0.0f + fScroll;	
	m_TextureBackgroundVerts[5].tu = 1.0f + fTexMargin ;	m_TextureBackgroundVerts[5].tv = 1.0f + fScroll;	

	return;


}

