// MirrorDemo.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// MirrorDemo.h: declaration of the MirrorDemo class.
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
#define AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000



#define NUMCUBEVERTS	36
#define	NUMMIRRORVERTS	6
#define NUMFLOORVERTS	6

class CMirrorDemo  
{
    CDRGWrapper *m_pWrapper;


	D3DVERTEX  m_CubeVerts[NUMCUBEVERTS];
	D3DLVERTEX m_MirrorVerts[NUMMIRRORVERTS];	
	D3DVERTEX  m_FloorVerts[NUMFLOORVERTS];

	// World matrices
	DRGMatrix m_MxWorld;
	DRGMatrix m_MxProjection;
	DRGMatrix m_MxView;
	DRGMatrix m_MxViewInv;
	
	// Local mirror matrices
	DRGMatrix m_MxMirrorWorld;
	DRGMatrix m_MxMirrorRotate;
	DRGMatrix m_MxMirrorTranslate;
	DRGMatrix m_MxMirrorWorldInv;


	// View along reflection vector
	DRGMatrix m_MxMirrorView;
	DRGMatrix m_MxMirrorViewInv;
	DRGMatrix m_MxMirrorViewInvBS;  // ( BS -> before shear )
	DRGMatrix m_MxMirrorProjection;
	
	
	// Local cube matrices
	DRGMatrix m_MxCubeWorld;
	DRGMatrix m_MxCubeTransform;

	// Shear matrix applied to the mirror view
	DRGMatrix m_MxShear;

	
	CDRGTexture *m_pMirrorTexture;
	CDRGTexture *m_pMirrorSurfaceTexture;
	CDRGTexture *m_pFloorTexture;


	// Viewport used in rendering to a texture
	D3DVIEWPORT7 m_vpTexViewport;

	
	D3DMATERIAL7	m_Material;
    D3DLIGHT7 m_Light;


	int  m_iMousePrevX;
	int  m_iMousePrevY;


public:
    CMirrorDemo();
    virtual ~CMirrorDemo();
	
    virtual HRESULT InitWorld( CDRGWrapper *pWrapper );
	virtual HRESULT DestroyWorld( void );

	virtual HRESULT UpdateWorld();
	virtual HRESULT RenderWorld( void );

	virtual HRESULT TargetChanged( void );

	void UpdateMirrorTextureSize( int iTextureSize );
	
	void KeyDown(WPARAM wParam);
	void KeyUp(WPARAM wParam);

	void MouseLeftUp(WPARAM wParam, LPARAM lParam);
	void MouseLeftDown(WPARAM wParam, LPARAM lParam);
	void MouseRightUp(WPARAM wParam, LPARAM lParam);
	void MouseRightDown(WPARAM wParam, LPARAM lParam);
	void MouseMove(WPARAM wParam, LPARAM lParam);


private:

	void SetupCubeVerts(float scale);
	void SetupMirrorVerts(float scale);
	void SetupFloorVerts(float scale);

	HRESULT DrawCube();

	unsigned int m_iTextureSize;

	// scale used for the scene objects
	float m_fScale;

	float m_fMirrorSize;

	DRGVector m_drgvMirrorCenter;
	
	DRGVector m_drgvCubeCenter;

	// Cube rotation speed
	float m_fRotPerSecX;
	float m_fRotPerSecY;
	float m_fRotPerSecZ;

	// Vectors used for figuring out the
	// reflection
	DRGVector m_drgvMirrorNormal;
	DRGVector m_drgvCameraCenter;
	DRGVector m_drgvCameraView;
	DRGVector m_drgvCameraViewReflected;
	DRGVector m_drgvCameraUpVector;
	DRGVector m_drgvMirrorUpVector;
	DRGVector m_drgvCameraCenterReflected;
	DRGVector m_drgvPointBehindMirror;
	
	// In Mirror View Coordinates (IMVC)
	DRGVector m_drgvCameraCenterReflectedIMVC;
	DRGVector m_drgvPointBehindMirrorIMVC;


	float m_fCameraToMirrorDist;
	float m_fDistToFrontPlane;
	float m_fFov;
	
	// Shear amount applied to the initial view frustum
	float m_fShearOnX;
	float m_fShearOnY;

	bool m_bDrawCube;
	bool m_bDrawMirror;
	bool m_bDrawFloor;
	
	bool m_bDrawMirrorToCameraReflection;
	bool m_bDrawMirrorNormal;
	bool m_bDrawCamUpVector;
	bool m_bDrawViewFrustum;
	bool m_bDrawCrosshairs;

	bool m_bViewFromMirror;
	bool m_bAutoRotateCube;
	bool m_bRotateMirror;
	
	bool m_bUpdateReflection;
	
	// m_iFrameNumber is a counter that ranges from 0
	// to m_iReflectionUpdateFrequency. When it reaches
	// 0, the reflection is updated. 
	int m_iFrameNumber;
	int m_iReflectionUpdateFrequency;

};

#endif // !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
