// ShadowTex.h - Created with DR&E AppWizard 1.0B8
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// ShadowTex.h: implementation the ShadowTex class.
// This class is the "blank" class created by the DRG 3D Appwizard, where you will put your stuff
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////

#include "DRGShell.h"

#define	SHADOWTEXSIZE	256


// -----------------------------------------------------------------------
// CShadowTex::CShadowTex() 
// The ShadowTex class constructor initializes some of it's member vars (which
// are instances of the DRGMatrix class
// -----------------------------------------------------------------------
CShadowTex::CShadowTex()
{
	m_MxWorld.SetIdentity();
	//set the camera at 0,0,-5; the target at 0,0,0 and the up vector at 0,1,0
	m_MxView.SetView( DRGVector( 0, 0, -5 ), DRGVector( 0, 0, 0 ), DRGVector( 0, 1, 0 ) );
	//set the projection matrix to 1.57 field of view (radians), 1.33 aspect ratio 
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxProjection.SetProjection( 1.57f, 1.333f, 1.0f, 100.0f );

	//set the camera at 0,0,-5; the target at 0,0,0 and the up vector at 0,1,0
	m_MxLightView.SetView( DRGVector( 0, 0, -8 ), DRGVector( 0, 0, 0 ), DRGVector( 0, 1, 0 ) );
	//set the projection matrix to 1.57 field of view (radians), 1.33 aspect ratio 
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxLightProjection.SetProjection( 1.11f, 1.333f, 1.0f, 100.0f );

	
	m_pDefaultTexture = NULL;
	m_pTigerTexture = NULL;
	m_pFloorTexture = NULL;
	//m_pCubeTexture = NULL;
	m_pShadowTexture = NULL;

	m_pDefaultMaterial = NULL;
	m_pTigerMaterial = NULL;
	m_pFloorMaterial = NULL;
	
	m_pRootObj = NULL;
	for (int i=0;i<9;i++)
		m_pFloorPlaneArray[i] = NULL;
	
	m_pCubeObj = NULL;
	m_pTigerObj = NULL;
	m_pLightObj = NULL;
	m_pLightDummyObj = NULL;
	m_pBallDummyObj = NULL;	
	m_pBallObj1 = NULL;
	m_pBallObj2 = NULL;

	ZeroMemory( &m_DefaultLight, sizeof(m_DefaultLight) );
	
	m_bShowShadowRect = true;
	
}

// -----------------------------------------------------------------------
// CShadowTex::~CShadowTex()
// destructor does nothing for now.
// -----------------------------------------------------------------------
CShadowTex::~CShadowTex()
{
	for (int i=0;i<9;i++)
	{
		if(m_pFloorPlaneArray[i])
		{
			delete m_pFloorPlaneArray[i];
			m_pFloorPlaneArray[i] = NULL;
		}		
	}
}

// -----------------------------------------------------------------------
// HRESULT CShadowTex::InitWorld( CDRGWrapper *pWrapper )
//
// -----------------------------------------------------------------------
HRESULT CShadowTex::InitWorld( CDRGWrapper *pWrapper )
{
	//
	// Make local copy of the pointer to the wrapper class
	//
	m_pWrapper = pWrapper;

	InitShadowTexture(SHADOWTEXSIZE);
	

	// Create a new texture, and init it, with an alpha channel. 
	// (try swapping the flag field in the line below for the remmed
	// one to compare the quality of a 4444 vs 8888 ARGB texture, if 
	// your device supports it (if the device doesn't, it will get 
	// converted to a supported format automatically))
	//
	m_pDefaultTexture = new CDRGTexture;
	if( m_pDefaultTexture->CreateTexture( CBMPLoader( CFileStream( "Images\\base.bmp" ) ),
						  /*CBMPLoader( CFileStream( "Images\\test_a.bmp" ) ),*/
						  (DWORD)0, (DWORD)0/*DRGTEXTR_32BITSPERPIXEL*/) == S_OK )

		m_pDefaultTexture->Restore(pWrapper->m_pD3DDevice7);

	m_pTigerTexture = new CDRGTexture;
	if( m_pTigerTexture->CreateTexture( CBMPLoader( CFileStream( "Images\\tiger.bmp" ) ),						
						  (DWORD)0, (DWORD)0/*DRGTEXTR_32BITSPERPIXEL*/) == S_OK )
		m_pTigerTexture->Restore(pWrapper->m_pD3DDevice7);

	m_pFloorTexture = new CDRGTexture;
	if( m_pFloorTexture->CreateTexture( CBMPLoader( CFileStream( "Images\\floor.bmp" ) ),						
						  (DWORD)0, (DWORD)0/*DRGTEXTR_32BITSPERPIXEL*/) == S_OK )
		m_pFloorTexture->Restore(pWrapper->m_pD3DDevice7);
	

	m_pRootObj		= new CDRGObject;
	m_pRootObj->Init(pWrapper, NULL, NULL, 0, 0,						 
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f),  DRGVector(0.0f),  0.0f,
					0x0);

	float floorheight = -0.69f; //hack to get floor to sit at tigers feet.
	
	for (int i=0;i<9;i++)
	{
		m_pFloorPlaneArray[i]	= new CDRGObjectPlane;
		m_pFloorPlaneArray[i]->Init(pWrapper, 1,
					DRGVector(-2.0f+(i/3)*2.0f,floorheight,-2.0f+(i%3)*2.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 0.0f,
					0x0);
		m_pRootObj->AddChild(m_pFloorPlaneArray[i]);
	}
	
	m_pCubeObj		= new CDRGObjectCube;
	m_pCubeObj->Init(pWrapper, 
					DRGVector(-1.5f,-0.2f,-0.5f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 0.7f,
					0x0);
//	m_pRootObj->AddChild(m_pCubeObj);

	m_pBallDummyObj = NULL;	
	m_pBallObj1 = NULL;
	m_pBallObj2 = NULL;

	float ballradius = 0.3f;

	m_pBallDummyObj		= new CDRGObject;
	m_pBallDummyObj->Init(pWrapper, NULL, NULL, 0, 0,						 
					DRGVector(0.0f,floorheight + ballradius,0.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f),  DRGVector(0.0f),  -0.5f,
					0x0);
	m_pRootObj->AddChild(m_pBallDummyObj);

	m_pBallObj1		= new CDRGObjectSphere;
	m_pBallObj1->Init(pWrapper, ballradius, 12,
					DRGVector(1.3f,0.0f,0.0f), DRGVector(1.0f), DRGVector(-1.0f,0.0f,0), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), -2.0f,
					0x0);
	m_pBallDummyObj->AddChild(m_pBallObj1);

	m_pBallObj2		= new CDRGObjectSphere;
	m_pBallObj2->Init(pWrapper, ballradius, 12,
					DRGVector(-1.3f,0.0f,0.0f), DRGVector(1.0f), DRGVector(1.0f,0.0f,0), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), -2.0f,
					0x0);
	m_pBallDummyObj->AddChild(m_pBallObj2);
	
	m_pTigerObj		= new CDRGObject;
	m_pTigerObj->Init(pWrapper, "Images\\tiger.x",
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 0.0f,
					0x0);

	m_pRootObj->AddChild(m_pTigerObj);

	
	// a dummy frame centered at the origin, with a rotational velocity, to which we will attach the light
	m_pLightDummyObj		= new CDRGObject;
	m_pLightDummyObj->Init(pWrapper, NULL, NULL, 0, 0,						 
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.0f), DRGVector(0.0f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f),  DRGVector(0.0f),  1.0f,
					0x0);
	m_pRootObj->AddChild(m_pLightDummyObj);


	m_pLightObj		= new CDRGObjectSphere;
	m_pLightObj->Init(pWrapper, 0.7f, 4,
					DRGVector(1.9f,2.2f,0.25f), DRGVector(0.1f), DRGVector(0.0f,1.0f,0), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 0.0f,
					0x0);
	m_pLightDummyObj->AddChild(m_pLightObj);
	

	//now we call target changed, which sets up device dependant items
	TargetChanged();
	
	return S_OK;
}

HRESULT CShadowTex::InitShadowTexture(int size)
{
	if (m_pWrapper)
		m_pWrapper->m_pD3D7->EvictManagedTextures();

	if (m_pShadowTexture)
		delete m_pShadowTexture;
	
	m_pShadowTexture = new CDRGTexture;

	//if doing dualtexture, we could set the stage here to one
	if( m_pShadowTexture->CreateTexture(m_pWrapper,size,size,16,
		0,DRGTEXTR_RENDERTARGET,NULL) == S_OK)
	{
		if (m_pWrapper)
			m_pShadowTexture->Restore(m_pWrapper->m_pD3DDevice7);
	}

	return S_OK;

}

// -----------------------------------------------------------------------
// HRESULT CShadowTex::DestroyWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CShadowTex::DestroyWorld( void )
{
		for (int i=0;i<9;i++)
	{
		if(m_pFloorPlaneArray[i])
		{
			delete m_pFloorPlaneArray[i];
			m_pFloorPlaneArray[i] = NULL;
		}		
	}
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CShadowTex::UpdateWorld( DWORD dwTime, int InputFlag )
//
// -----------------------------------------------------------------------
HRESULT CShadowTex::UpdateWorld()
{
	

	//Call the Update method of the object at the root of our hierarchy,
	// specifying to propogate the update down the hierarchy. This will
	// update all the objects positions/orientations according to their
	// velocities. By specifying the frame time, our animation speed is
	// independant of frame rate
	if(m_pRootObj)
		m_pRootObj->Update(m_pWrapper->m_fFrameTime, true);

	if (m_pRootObj)
	{
		m_DefaultLight.dvPosition	= D3DVECTOR(m_pLightObj->m_MxLocalToWorld.m_Mx._41,
												m_pLightObj->m_MxLocalToWorld.m_Mx._42,
												m_pLightObj->m_MxLocalToWorld.m_Mx._43);
		m_pWrapper->m_pD3DDevice7->SetLight( 0, &m_DefaultLight );
			

	}
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CShadowTex::RenderWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CShadowTex::RenderWorld( void )
{
	HRESULT hr;

	
	//1.0 render the shadow texture
	//		1.1 srt to shadow, clear to white
	//		1.2 calc view angle and the like
	//		1.3 set up view frustum
	//		1.4 draw object casting shadow
	//2.0 translate objects back to that space, calc texture UV's
	//3.0 render scene

	m_pWrapper->m_pD3DDevice7->EndScene();


	D3DVIEWPORT7	VPTemp, VPOld;

	m_pWrapper->m_pD3DDevice7->GetViewport(&VPOld);
	memcpy(&VPTemp,&VPOld,sizeof(D3DVIEWPORT7));
	VPTemp.dwWidth = m_pShadowTexture->m_ddsd.dwWidth;
	VPTemp.dwHeight = m_pShadowTexture->m_ddsd.dwHeight;

	UpdateLightFrustum();
	
	//draw one texture one, using texture 2
	m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pShadowTexture->ptexTexture, 0);
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPTemp);

	D3DRECT rcClear;
	rcClear.x1 = 0;	rcClear.y1 = 0;
	rcClear.x2 = m_pShadowTexture->m_ddsd.dwWidth;
	rcClear.y2 = m_pShadowTexture->m_ddsd.dwHeight;
	m_pWrapper->m_pD3DDevice7->Clear( 1, &rcClear, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
					0xffffffff, 1.0f, 0 );


	
	m_pWrapper->m_pD3DDevice7->BeginScene();
	m_pWrapper->m_pD3DDevice7->SetTexture(0,NULL);
	m_pWrapper->m_pD3DDevice7->SetTexture(1,NULL);

	//need to set the light to zero though...
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, FALSE);	

	//draw tiger as black on white
	if( FAILED(m_pWrapper->m_pD3DDevice7->LightEnable( 0, FALSE )))
		return hr;

	if(m_pTigerObj)
		hr = m_pTigerObj->Render( m_pRootObj->m_MxLocalToWorld * m_MxWorld,true, false);

	if( FAILED(m_pWrapper->m_pD3DDevice7->LightEnable( 0, TRUE )))
		return hr;


	if(m_pRootObj)
		hr = m_pRootObj->ProcessVerts(m_MxWorld, m_pShadowTexture->m_ddsd.dwWidth/*SHADOWTEXSIZE*/, true);
	
	m_pWrapper->m_pD3DDevice7->EndScene();

	//switch back to back buffer
	m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pWrapper->m_pBackBuffer, 0);
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPOld);
	
	m_pWrapper->m_pD3DDevice7->BeginScene();

	
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxProjection.m_Mx );

	//first pass - render shadow
	hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pShadowTexture->dwStage, m_pShadowTexture->ptexTexture);	//stage 1
	hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pShadowTexture->dwStage,D3DTSS_ADDRESS, D3DTADDRESS_CLAMP);
	
	m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_WRAP0 , 0);
	
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_WRAP0, 0);
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_WRAP1, 0);
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_WRAP2, 0);
	

	hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState(m_pShadowTexture->dwStage, D3DTSS_TEXCOORDINDEX, 0);

	m_pTigerObj->m_bVisible = false;
	if(m_pRootObj)
		hr = m_pRootObj->Render(m_MxWorld, true, false);
	m_pTigerObj->m_bVisible = true;
	
	// now enable alpha blending with the frame buffer
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_SRCBLEND, D3DBLEND_DESTCOLOR );
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_DESTBLEND, D3DBLEND_SRCCOLOR );

	//second pass - render base texture
	hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pDefaultTexture->dwStage, m_pDefaultTexture->ptexTexture); // stage 0	 
	
	m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage,D3DTSS_ADDRESS, D3DTADDRESS_WRAP);

	m_pWrapper->m_pD3DDevice7->SetTextureStageState(m_pDefaultTexture->dwStage, D3DTSS_TEXCOORDINDEX, 1);

	if(m_pRootObj)
		hr = m_pRootObj->Render(m_MxWorld, true, true);
	m_pTigerObj->m_bVisible = true;


	//reset things
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE );
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO );

	if(m_pRootObj)
		hr = m_pTigerObj->Render(m_pRootObj->m_MxLocalToWorld/*m_MxWorld * m_pRootObj->m_MxLocalToParent*/, true, true);
	
	m_pWrapper->m_pD3DDevice7->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	m_pWrapper->m_pD3DDevice7->SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 1);

	if (m_bShowShadowRect) 
	{
		//enable texture
		hr = m_pWrapper->m_pD3DDevice7->SetTexture( 0, m_pShadowTexture->ptexTexture);			
		//draw the shadow rectangle
		hr = m_pWrapper->m_pD3DDevice7->DrawPrimitive( D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX,
			m_ShadowRectVerts, 6, 0);		
	}
	
	return S_OK;
}


// -----------------------------------------------------------------------
// HRESULT CShadowTex::TargetChanged( void )
//
// -----------------------------------------------------------------------
HRESULT CShadowTex::TargetChanged( void )
{
	HRESULT hr;
	//
	// This indicates that the devices of the DRGWrapper class have changed.
	// If you have anything that depends on the current rendering device, you'll
	// have to re-initialize it here
	//
	m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_DITHERENABLE, TRUE );

	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxProjection.m_Mx );

	hr = SetupDependantItems();

	SetupShadowRect(m_pWrapper->m_dwRenderWidth/5.0f, m_pWrapper->m_dwRenderHeight/5.0f);
	
	return hr;
}

HRESULT CShadowTex::SetupDependantItems(void)
{
	HRESULT hr;

	InitShadowTexture(SHADOWTEXSIZE);
	
	//restore texture
	if (m_pDefaultTexture)
		m_pDefaultTexture->Restore(m_pWrapper->m_pD3DDevice7);

	if (m_pTigerTexture)
		m_pTigerTexture->Restore(m_pWrapper->m_pD3DDevice7);

	if (m_pFloorTexture)
		m_pFloorTexture->Restore(m_pWrapper->m_pD3DDevice7);

	if(m_pRootObj)
		hr = m_pRootObj->TargetChanged(m_pWrapper, true);

	//create a default material
	D3DMATERIAL7	*ptempmat = new D3DMATERIAL7;
	ZeroMemory( ptempmat, sizeof(D3DMATERIAL7) );
  
	ptempmat->ambient.r = 0.2f;    ptempmat->ambient.g = 0.2f;    ptempmat->ambient.b = 0.2f;
	ptempmat->diffuse.r = 0.7f;    ptempmat->diffuse.g = 0.7f;    ptempmat->diffuse.b = 0.7f;
	ptempmat->specular.r = 0.2f;    ptempmat->specular.g = 0.2f;    ptempmat->specular.b = 0.2f;

	m_pDefaultMaterial = new CDRGMaterial;
		
	m_pDefaultMaterial->SetMaterial(ptempmat);
	m_pDefaultMaterial->SetTexture(m_pDefaultTexture);
	
	m_pRootObj->SetMaterial(m_pDefaultMaterial, true);

	m_pTigerMaterial = new CDRGMaterial;
	m_pTigerMaterial->SetTexture(m_pTigerTexture);

	m_pTigerObj->SetMaterial(m_pTigerMaterial, true);
	
	m_pFloorMaterial = new CDRGMaterial;
	m_pFloorMaterial->SetTexture(m_pFloorTexture);

	for (int i=0;i<9;i++)
	{
		m_pFloorPlaneArray[i]->SetMaterial(m_pFloorMaterial, false);
	}
	

	
	// Set up a default light
    
    ZeroMemory( &m_DefaultLight, sizeof(m_DefaultLight) );
    m_DefaultLight.dltType       = D3DLIGHT_POINT;
    m_DefaultLight.dcvDiffuse.r  = 0.9f;
    m_DefaultLight.dcvDiffuse.g  = 0.9f;
    m_DefaultLight.dcvDiffuse.b  = 0.9f;
	m_DefaultLight.dvPosition	= D3DVECTOR(m_pLightObj->m_vPosition.x,
											m_pLightObj->m_vPosition.y,
											m_pLightObj->m_vPosition.z);
    m_DefaultLight.dvRange       = 10.0f;//D3DLIGHT_RANGE_MAX;
	m_DefaultLight.dvAttenuation0       = 0.001f;
	m_DefaultLight.dvAttenuation1       = 0.010001f;
	m_DefaultLight.dvAttenuation2       = 0.03001f;
	
    // Set the light
    if( FAILED(m_pWrapper->m_pD3DDevice7->SetLight( 0, &m_DefaultLight )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->LightEnable( 0, TRUE )))
		return hr;

    if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_LIGHTING, TRUE )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_AMBIENT, 0xff333333 ) ) )
		return hr;
	
	if (FAILED(hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_COLORVERTEX, TRUE) ))
		return hr;

	return S_OK;
}


// -----------------------------------------------------------------------
// void CShadowTex::KeyUp(WPARAM wParam)
//
//   Keyboard keypress handler. 
// -----------------------------------------------------------------------
void CShadowTex::KeyDown(WPARAM wParam)
{
	//we need a variable to modify our rotation speed with frame rate	
	float scaler = 1.0f;

	//if exiting, the wrapper could be invalid, but we don't care, since we will exit anyway
	if (wParam == 'X') 
		return;

	if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
	{
		if (m_pWrapper->m_fFrameTime >0.0f)
			scaler = m_pWrapper->m_fFrameTime;  
	}
		
	switch (wParam) 
	{
	case VK_UP:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, -3.14159f*scaler );
		break;
	
	case VK_DOWN:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler );
		break;
	
	case VK_LEFT:
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, -3.14159f*scaler );
		break;
	
	case VK_RIGHT:	
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler );
		break;
	}

	if (m_pWrapper)
	{
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	}

	
	return;
}

void CShadowTex::KeyUp(WPARAM wParam)
{		
	switch (wParam) 
	{
		//add code here to handle key releases, if you need them (as in remmed out example)
	case VK_UP:
		//OutputDebugString("The up arrow was released;");			
		break;
	}
	
	return;
}


void CShadowTex::MouseLeftUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CShadowTex::MouseLeftDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CShadowTex::MouseRightUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CShadowTex::MouseRightDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CShadowTex::MouseMove(WPARAM wParam, LPARAM lParam)
{
	if ( wParam & MK_LBUTTON )	// Left button down plus mouse move == rotate
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 2.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (x - m_iMousePrevX)
		{
			m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler*(float)(x - m_iMousePrevX) );
		}
			
		if (y - m_iMousePrevY)
		{
			m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler*(float)(y - m_iMousePrevY) );
		}

		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}
	else if ( wParam & MK_RBUTTON )	// Right button down plus mouse move == Zoom camera in/out
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 5.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (y - m_iMousePrevY)
		{
			DRGMatrix Temp(true);

			Temp.Translate( 0,0,scaler * (y - m_iMousePrevY) );
			m_MxView.PostMultiply( Temp );
		}

		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}
	else
	{
		//need to update these, or the next time someone clicks, the delta is massive
		m_iMousePrevX = LOWORD(lParam);
		m_iMousePrevY = HIWORD(lParam);	
	}
	return;
}

void CShadowTex::SetupShadowRect(float hsize, float vsize)
{
	memset( m_ShadowRectVerts, 0, sizeof( D3DTLVERTEX ) * 6 );

	float width = hsize;
	float height = vsize;

	float margin = 0.00f;

	m_ShadowRectVerts[0].sx = 0.0f + margin;	m_ShadowRectVerts[0].sy = 0.0f + margin;	
	m_ShadowRectVerts[1].sx = width - margin;	m_ShadowRectVerts[1].sy = height - margin;
	m_ShadowRectVerts[2].sx = 0.0f + margin;	m_ShadowRectVerts[2].sy = height - margin;	
	m_ShadowRectVerts[3].sx = 0.0f + margin;	m_ShadowRectVerts[3].sy = 0.0f + margin;	
	m_ShadowRectVerts[4].sx = width - margin;	m_ShadowRectVerts[4].sy = 0.0f + margin;	
	m_ShadowRectVerts[5].sx = width - margin;	m_ShadowRectVerts[5].sy = height - margin;	
		
	// common elements
	for (int i=0;i<6;i++)
	{
		m_ShadowRectVerts[i].sz = 0.0f;
		m_ShadowRectVerts[i].rhw = 0.00001f;
		m_ShadowRectVerts[i].dcSpecular = 0;
		m_ShadowRectVerts[i].dcColor = D3DRGB(1.0f,1.0f,1.0f );
	}

	m_ShadowRectVerts[0].tu = 0.0f;	m_ShadowRectVerts[0].tv = 0.0f  ;	
	m_ShadowRectVerts[1].tu = 1.0f;	m_ShadowRectVerts[1].tv = 1.0f  ;	
	m_ShadowRectVerts[2].tu = 0.0f;	m_ShadowRectVerts[2].tv = 1.0f  ;	
	m_ShadowRectVerts[3].tu = 0.0f;	m_ShadowRectVerts[3].tv = 0.0f  ;	
	m_ShadowRectVerts[4].tu = 1.0f;	m_ShadowRectVerts[4].tv = 0.0f  ;	
	m_ShadowRectVerts[5].tu = 1.0f ;	m_ShadowRectVerts[5].tv = 1.0f  ;	


	return;


}

HRESULT CShadowTex::UpdateLightFrustum()
{
	//need to figure out how wide to make it.
	float r = m_pTigerObj->CalcBoundRadius(true);

	DRGVector len =    DRGVector(m_pTigerObj->m_MxLocalToWorld.m_Mx._41,m_pTigerObj->m_MxLocalToWorld.m_Mx._42,m_pTigerObj->m_MxLocalToWorld.m_Mx._43)
					-  DRGVector(m_pLightObj->m_MxLocalToWorld.m_Mx._41,m_pLightObj->m_MxLocalToWorld.m_Mx._42,m_pLightObj->m_MxLocalToWorld.m_Mx._43);

	
	float h = (float)sqrt( (len.x * len.x) + (len.y * len.y) + (len.z * len.z));

	//if we are inside the objects bounding sphere, we have a problem
	if (h<r)
		return -1;

	float angle = 2.0f * (float)asin(r/h);

	float aspect = 1.0f;//cause we have a square texture we are rendering the shadow to


	DRGVector from = DRGVector(m_pLightObj->m_MxLocalToWorld.m_Mx._41,m_pLightObj->m_MxLocalToWorld.m_Mx._42,m_pLightObj->m_MxLocalToWorld.m_Mx._43);
	DRGVector to = DRGVector (m_pTigerObj->m_MxLocalToWorld.m_Mx._41,m_pTigerObj->m_MxLocalToWorld.m_Mx._42,m_pTigerObj->m_MxLocalToWorld.m_Mx._43);

	m_MxLightView.SetView( from,to, DRGVector( 0.0f, 1.0f, 0.0f ) );

	//set the projection matrix to 1.57 field of view (radians), 1.0 aspect ratio (square texture)
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxLightProjection.SetProjection( angle, 1.0f, h-r/*1.0f*/, h+r/*100.0f*/ );

	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxLightView.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxLightProjection.m_Mx );

	return S_OK;

}

