// Sphere.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// Sphere.h: declaration of the Sphere class.
// This class is used to create a sphere object
//
// ------------------------------------------------------------------------------------
//
// Author: Mike Rohman
// 
////////////////////////////////////////////////////////////////////////////////////////

struct DualTexVertex {
    float x, y, z;			
    
    float nx, ny, nz;		
    
    float tu, tv;
 
    float tu2, tv2;

};


class Sphere {

public:
	Sphere();
	~Sphere();
	
	// Draw the sphere
	HRESULT DrawSphere();
	
	// Initialize a sphere object
	HRESULT Init( float rad, int nDiv, CDRGWrapper *pWrapper );
	
	void UpdateTextureCoords( DRGMatrix reflectionViewMatrix, DRGMatrix mirrorTransform, 
		                      DRGVector cameraCenter);
private:
	
	HRESULT SetupIndexedVertices();
	HRESULT SetupIndexedIndices();
	
		
	DualTexVertex *m_pIndexedVertices;
//	D3DVERTEX *m_pIndexedVertices;
	DRGVector *m_pRVSVertices;

	WORD *m_pVertexIndices;

	int m_iNumIndexedVertices;	    // number of indexed vertices
	int m_iNumVertexIndices;		// number of indexed vertex indices

	float m_fRadius;		// radius
	int m_iNumDiv;			// number of latitude and longitude sections
	
    CDRGWrapper *m_pWrapper;

};