// SphericalMirror.h
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// SphericalMirror.h: declaration of the SphericalMirror class.
// This class is the "blank" class created by the DRG 3D Appwizard, where you will put your stuff
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
#define AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000



#define	NUMFLOORVERTS	6
#define NUMCUBEVERTS	36
#define NUMWALLVERTS	24


class CSphericalMirror  
{
    CDRGWrapper *m_pWrapper;


	D3DVERTEX m_FloorVerts[NUMFLOORVERTS];	
	D3DVERTEX m_CubeVerts[NUMCUBEVERTS];
	D3DVERTEX m_WallVerts[NUMWALLVERTS];


	Sphere m_sphSphere;

	// Viewport for the texture
	D3DVIEWPORT7 m_vpTexViewport;

	D3DMATERIAL7 m_Material;
	D3DMATERIAL7 m_MaterialLowAlpha;

    D3DLIGHT7 m_Light;

	DRGMatrix m_MxWorld;
	DRGMatrix m_MxProjection;
	DRGMatrix m_MxView;
	DRGMatrix m_MxViewInverse;

	DRGMatrix m_MxMirrorWorld;
	DRGMatrix m_MxMirrorProjection;
	DRGMatrix m_MxMirrorView;
	DRGMatrix m_MxMirrorTransform;
	
	DRGMatrix m_MxCameraToMirrorViewInv;

	DRGMatrix m_MxSphereWorld;
	DRGMatrix m_MxSphereRotation;
	DRGMatrix m_MxSphereTranslation;

	DRGMatrix m_MxCubeWorld;
	DRGMatrix m_MxCubeTransform;
	DRGMatrix m_MxLeftCubeWorld;
	DRGMatrix m_MxLeftCubeTransform;
	DRGMatrix m_MxRightCubeWorld;
	DRGMatrix m_MxRightCubeTransform;
	
	CDRGTexture *m_pMirrorTexture;	
	CDRGTexture *m_pFloorTexture;
	CDRGTexture *m_pCubeTexture;
	CDRGTexture *m_pBaseTexture;


	int  m_iMousePrevX;
	int  m_iMousePrevY;


public:
    CSphericalMirror();
    virtual ~CSphericalMirror();
	
    virtual HRESULT InitWorld( CDRGWrapper *pWrapper );
	virtual HRESULT DestroyWorld( void );

	virtual HRESULT UpdateWorld();
	virtual HRESULT RenderWorld( void );

	virtual HRESULT TargetChanged( void );

	void UpdateMirrorTextureSize( int iTextureSize );

	void KeyDown(WPARAM wParam);
	void KeyUp(WPARAM wParam);

	void MouseLeftUp(WPARAM wParam, LPARAM lParam);
	void MouseLeftDown(WPARAM wParam, LPARAM lParam);
	void MouseRightUp(WPARAM wParam, LPARAM lParam);
	void MouseRightDown(WPARAM wParam, LPARAM lParam);
	void MouseMove(WPARAM wParam, LPARAM lParam);


private:

	HRESULT DrawCube();

	void SetupCubeVerts(float scale);
	void SetupFloorVerts(float scale);
	void SetupWallVerts(float scale);

	
	
	DRGVector m_drgvCubeCenter;
	DRGVector m_drgvMirrorCenter;

	DRGVector m_drgvCameraCenter;
	DRGVector m_drgvCameraUpVector;
	DRGVector m_drgvCameraToMirrorUpVector;


	float m_fMirrorRadius;
	
	// The field of view used for the mirror's reflection
	float m_fFov;

	// The rotational speeds of the cubes
	float m_fRotPerSecX;
	float m_fRotPerSecY;
	float m_fRotPerSecZ;

	// The rotational speed of the sphere
	// when rotated
	float m_fSphereRotPerSecX;
	float m_fSphereRotPerSecY;
	float m_fSphereRotPerSecZ;
	
	// The reflection mapping method
	int m_iMethod;

	// The shininess of the base sphere 
	float m_fBaseShininess;

	// A value that is used for toggling
	// between a mirror, normal sphere, and
	// a shiny sphere
	int m_iTypeOfSphere;
	
	bool	m_bDrawFloor;
	bool	m_bDrawCubes;
	bool	m_bDrawMirror;
	bool	m_bDrawWalls;

	// View from the mirror's point of view
	bool	m_bViewFromMirror;

	bool	m_bAutoRotateCube;
	bool	m_bKeysRotateCube;	// Use keys to rotate the cubes or world
	bool	m_bRotateSphere;	// Rotate the mirror sphere and base sphere

	int m_iTextureSize;

	int m_iReflectionUpdateFrequency;
	int m_iFrameNumber;
	bool m_bUpdateReflection;


};

#endif // !defined(AFX_DRGAPP_H__CFF24425_47D6_11D2_B1A5_00A0C90D2C1B__INCLUDED_)
