// DRGMaterial.cpp - Created with DR&E AppWizard 1.0B8
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// DRGMaterial.h: implementation of the CDRGMaterial class.
// This class is used to contain one or more material properties, so that objects within
// a hierarchy can have knowledge of and set their materials prior to rendering
// There are 3 key member vars, of which one or more can be valid. THese member vars are
// a D3DMaterial7, and DRGTexture, and a state block, which is a collection of 
// renderstates. Also, there is a flags var used to determine which of the 3 key member 
// vars are valid.
//
// This class was added at the last minute to beta 8 of hte appwizard, and so isn't 
// robustly tested
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister. 
// 
////////////////////////////////////////////////////////////////////////////////////////
#include "..\DRGShell.h"


CDRGMaterial::CDRGMaterial()
{
	m_pDRGTexture	= NULL;
	m_pD3DMaterial	= NULL;
	m_dwStateBlock	= 0;

	m_pWrapper		= NULL;
	
	m_dwFlags			= 0;

}

CDRGMaterial::~CDRGMaterial()
{
	// currently nothing here, but I'm not sure if I have to invalidate the state block
}
	
	
HRESULT CDRGMaterial::GetTexture(CDRGTexture **ppTex)
{
	if (m_dwFlags & DRGMATERIAL_TEXTURE)
	{
		ppTex = &m_pDRGTexture;
		return S_OK;
	}
	return -1;
}

HRESULT CDRGMaterial::SetTexture(CDRGTexture *pTex)
{
	m_pDRGTexture = pTex;
	if (pTex == NULL)
		m_dwFlags |= !DRGMATERIAL_TEXTURE;
	else
		m_dwFlags |= DRGMATERIAL_TEXTURE;
	
	return S_OK;

}

HRESULT CDRGMaterial::GetMaterial(LPD3DMATERIAL7 *pMat)
{
	if (m_dwFlags & DRGMATERIAL_MATERIAL)
	{
		pMat = &m_pD3DMaterial;
		return S_OK;
	}
	return -1;
}

HRESULT CDRGMaterial::SetMaterial(LPD3DMATERIAL7 pMat)
{
	m_pD3DMaterial = pMat;
	if (pMat == NULL)
		m_dwFlags |= !DRGMATERIAL_MATERIAL;
	else
		m_dwFlags |= DRGMATERIAL_MATERIAL;
	
	return S_OK;

}

HRESULT CDRGMaterial::GetStateBlock(DWORD	*pBlock)
{
	if (m_dwFlags & DRGMATERIAL_STATEBLOCK)
	{
		pBlock = &m_dwStateBlock;
		return S_OK;
	}
	return -1;
}

HRESULT CDRGMaterial::SetStateBlock(DWORD	Block)
{
	m_dwStateBlock = Block;
	m_dwFlags |= DRGMATERIAL_STATEBLOCK;

	//should we be validating the state block?
	return S_OK;

}

HRESULT CDRGMaterial::Activate(DWORD	dwFlags)
{
	HRESULT	hr = S_OK;

	if ((m_dwFlags & DRGMATERIAL_TEXTURE) && (dwFlags & DRGMATERIAL_TEXTURE) && (m_pWrapper) && (m_pWrapper->m_pD3DDevice7))
		hr |= m_pWrapper->m_pD3DDevice7->SetTexture(m_pDRGTexture->dwStage,m_pDRGTexture->ptexTexture);

	if ((m_dwFlags & DRGMATERIAL_MATERIAL) && (dwFlags & DRGMATERIAL_MATERIAL) && (m_pWrapper) && (m_pWrapper->m_pD3DDevice7))
		hr |= m_pWrapper->m_pD3DDevice7->SetMaterial(m_pD3DMaterial);

	if ((m_dwFlags & DRGMATERIAL_STATEBLOCK) && (dwFlags & DRGMATERIAL_STATEBLOCK) && (m_pWrapper) && (m_pWrapper->m_pD3DDevice7))
		hr |= m_pWrapper->m_pD3DDevice7->ApplyStateBlock(m_dwStateBlock);

	return hr;
}

HRESULT CDRGMaterial::TargetChanged(CDRGWrapper *pDRGWrapper)
{
	m_pWrapper = pDRGWrapper;

	//The app is responsible for re-init'ing all the material properties in it's targetchanged()

	return S_OK;
}

