// TestRTT.h - Created with DR&E AppWizard 1.0B8
// ------------------------------------------------------------------------------------
// Copyright  1999 Intel Corporation
// All Rights Reserved
// 
// Permission is granted to use, copy, distribute and prepare derivative works of this 
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  This software is provided "AS IS." 
//
// Intel specifically disclaims all warranties, express or implied, and all liability,
// including consequential and other indirect damages, for the use of this software, 
// including liability for infringement of any proprietary rights, and including the 
// warranties of merchantability and fitness for a particular purpose.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
// ------------------------------------------------------------------------------------
//
//  PURPOSE:
//    
// TestRTT.h: implementation the TestRTT class.
// This class is the "blank" class created by the DRG 3D Appwizard, where you will put your stuff
//
// ------------------------------------------------------------------------------------
//
// Author: Kim Pallister - Intel Developer Relations & Engineering Group
//
////////////////////////////////////////////////////////////////////////////////////////

#include "DRGShell.h"


// -----------------------------------------------------------------------
// CTestRTT::CTestRTT() 
// The TestRTT class constructor initializes some of it's member vars (which
// are instances of the DRGMatrix class
// -----------------------------------------------------------------------
CTestRTT::CTestRTT()
{
	m_MxWorld.SetIdentity();
	//set the camera at 0,0,-5; the target at 0,0,0 and the up vector at 0,1,0
	m_MxView.SetView( DRGVector( 0, 0, -5 ), DRGVector( 0, 0, 0 ), DRGVector( 0, 1, 0 ) );
	//set the projection matrix to 1.57 field of view (radians), 1.33 aspect ratio 
	// (640x480), //the near clip plane to 1 and the far to clip plane 100
	m_MxProjection.SetProjection( 1.57f, 1.333f, 1.0f, 100.0f );

	
	m_pDefaultTexture = NULL;

	m_pDefaultMaterial = NULL;
	m_pRenderTargetTexture = NULL;
	m_pFallbackTexture = NULL;
	m_pFallbackTemp = NULL;


	m_pSampleCubeObj = NULL;
	m_pSampleSphereObj = NULL;

	m_bUseFallback = false;
	m_bForceFallback = false;
	
}

// -----------------------------------------------------------------------
// CTestRTT::~CTestRTT()
// destructor does nothing for now.
// -----------------------------------------------------------------------
CTestRTT::~CTestRTT()
{

}

// -----------------------------------------------------------------------
// HRESULT CTestRTT::InitWorld( CDRGWrapper *pWrapper )
//
// -----------------------------------------------------------------------
HRESULT CTestRTT::InitWorld( CDRGWrapper *pWrapper )
{
	//
	// Make local copy of the pointer to the wrapper class
	//
	m_pWrapper = pWrapper;



	// Create a texture that will be the default material for our object.
	m_pDefaultTexture = new CDRGTexture;
	if( m_pDefaultTexture->CreateTexture( CBMPLoader( CFileStream( "Images\\test.bmp" ) ), 
						  (DWORD)0, (DWORD)0) == S_OK )
		OutputDebugString("Texture object created. Still need to upload to vidmem in TargetChanged\n");

	// Create a texture with the RENDERTARGET flag (this tells our wrapper class to use both the 3DDevice
	// and Texture flags
	m_pRenderTargetTexture = new CDRGTexture;
	if( m_pRenderTargetTexture->CreateTexture(m_pWrapper,256,256,16, 0, DRGTEXTR_RENDERTARGET,NULL) == S_OK)
		OutputDebugString("Texture object created. Still need to upload to vidmem in TargetChanged\n");

	// This is just a regular DD Surface with the Texture flag, which we will blit to if RTT not suppported
	m_pFallbackTexture = new CDRGTexture;
	if( m_pFallbackTexture->CreateTexture(m_pWrapper,256,256,16, 0, DRGTEXTR_FORCEINVIDMEM,NULL) == S_OK)
		OutputDebugString("Texture object created. Still need to upload to vidmem in TargetChanged\n");
	
	// This is a sysmem surface that will be used as an intermediary step if need be
	m_pFallbackTemp = new CDRGTexture;
	if( m_pFallbackTemp->CreateTexture(m_pWrapper,256,256,16, 0, DRGTEXTR_FORCEINSYSMEM,NULL) == S_OK)
		OutputDebugString("Texture object created. Still need to create the actual surface in TargetChanged\n");

	m_pSampleCubeObj		= new CDRGObjectCube;
	m_pSampleCubeObj->Init(pWrapper,
					DRGVector(0.0f,0.0f,0.0f), DRGVector(1.5f), DRGVector(0.5f,1.0f,0.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 0.7f,
					0x0);

	m_pSampleSphereObj		= new CDRGObjectSphere;
	m_pSampleSphereObj->Init(pWrapper, 0.7f, 16,
					DRGVector(0.0f,0.0f,0.0f), DRGVector(3.0f), DRGVector(0.0f,1.0f,1.0f), 0.0f,
					DRGVector(0.0f), DRGVector(0.0f), 3.0f,
					0x0);



	//now we call target changed, which sets up device dependant items, render states
	TargetChanged();
	
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CTestRTT::DestroyWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CTestRTT::DestroyWorld( void )
{
	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CTestRTT::UpdateWorld( DWORD dwTime, int InputFlag )
//
// -----------------------------------------------------------------------
HRESULT CTestRTT::UpdateWorld()
{
	HRESULT hr;

	if(m_pSampleCubeObj)
		hr = m_pSampleCubeObj->Update(m_pWrapper->m_fFrameTime, true);

	if(m_pSampleSphereObj)
		hr = m_pSampleSphereObj->Update(m_pWrapper->m_fFrameTime, true);

	return S_OK;
}

// -----------------------------------------------------------------------
// HRESULT CTestRTT::RenderWorld( void )
//
// -----------------------------------------------------------------------
HRESULT CTestRTT::RenderWorld( void )
{
	HRESULT hr;

	D3DVIEWPORT7	VPTemp, VPOld;

	m_pWrapper->m_pD3DDevice7->GetViewport(&VPOld);
	memcpy(&VPTemp,&VPOld,sizeof(D3DVIEWPORT7));

	//flags we'll use for clearing both the back buffer and texture depend on whether
	// we're using a Z buffer or not.
	DWORD dwClearFlags;
	if (m_pWrapper->m_pZBuffer)
		dwClearFlags = D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER ;
	else
		dwClearFlags = D3DCLEAR_TARGET;


	//we have to switch alpha back off, since the text class the wrapper uses enables it
	hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, FALSE);

	//The big If/Else below shows the procedure for rendering to a texture or working 
	// around a lack of the ability to do so. It's worth noting that if the rendering to be 
	// done to the texture were more significant than to two lines of code I use below, it 
	// would be worth making an if/else loop to setup the render, then rendering, then another 
	// if/else for the finishing up of the rendering and setup for the 'main' rendering to the
	// back buffer. For this demo though, this is a bit more readable.

	// FALLBACK (BLT) METHOD
	if (m_bUseFallback | m_bForceFallback)
	{

		//Render sphere to back buffer, blit to sysmem texture, then blit to vidmem texture
		// this should work with all cards. We could potentially test for different methods (e.g.
		// vidmem-vidmem blit) and handle some cards better
		
		VPTemp.dwWidth = m_pFallbackTemp->m_ddsd.dwWidth;
		VPTemp.dwHeight = m_pFallbackTemp->m_ddsd.dwHeight;

		//these next 2 ifs are to keep things from blowing up if the window size is smaller
		// that the texture size
		if (VPTemp.dwWidth > m_pWrapper->m_dwRenderWidth)
			VPTemp.dwWidth = m_pWrapper->m_dwRenderWidth;
	
		if (VPTemp.dwHeight > m_pWrapper->m_dwRenderHeight)
			VPTemp.dwHeight = m_pWrapper->m_dwRenderHeight;
	
		m_pWrapper->m_pD3DDevice7->SetViewport(&VPTemp);

		D3DRECT rcClear;
	
		rcClear.x1 = 0;
		rcClear.y1 = 0;
		rcClear.x2 = m_pRenderTargetTexture->m_ddsd.dwWidth;
		rcClear.y2 = m_pRenderTargetTexture->m_ddsd.dwHeight;

		//clear the buffer. Note that if your app isn't using a Z buffer, then your texture shouldn't have
		// one either, and so you'd omit that flag below
		hr = m_pWrapper->m_pD3DDevice7->Clear( 0, NULL, dwClearFlags,
								0xffff00ff, 1.0f, 0 );


		m_pWrapper->m_pD3DDevice7->BeginScene();


		//we've already cleared the back and Z buffers in our D3D wrapper class, but normally, you'd do that here
		// same goes for calling BeginScene

		// now we can draw our object to the back buffer
		if(m_pSampleSphereObj)
			hr = m_pSampleSphereObj->Render(m_MxWorld,true);

		m_pWrapper->m_pD3DDevice7->EndScene();

		RECT rc; 
		rc.left = 0;	rc.top = 0;
		rc.right = m_pFallbackTemp->m_ddsd.dwWidth;//-1;
		rc.bottom = m_pFallbackTemp->m_ddsd.dwHeight;//-1;

		// now blit from the back buffer to the temp texture. Ideally, you wouldn't want to specify the
		// WAIT flag, but I have done so to make the demo simpler. In a real app, you'd want to go off and do 
		// some stuff before coming back to make sure the scene has been rendered to the back buffer by the HW
		hr = m_pFallbackTemp->ptexTexture->Blt(NULL,m_pWrapper->m_pBackBuffer, &rc,DDBLT_WAIT,NULL );
				
		// now blit from the sysmem texture to the vidmem texture
		hr = m_pFallbackTexture->ptexTexture->Blt(NULL,m_pFallbackTemp->ptexTexture, NULL,DDBLT_WAIT,NULL );

		m_pWrapper->m_pD3DDevice7->SetViewport(&VPOld);

		//clear the back buffer
		m_pWrapper->Clear( dwClearFlags );
		
		//go back to rendering our regular scene.
		m_pWrapper->m_pD3DDevice7->BeginScene();


	}
	else //RENDER TO TEXTURE METHOD
	{
		
		VPTemp.dwWidth = m_pRenderTargetTexture->m_ddsd.dwWidth;
		VPTemp.dwHeight = m_pRenderTargetTexture->m_ddsd.dwHeight;
	
		// To point the render target to the texture, adjust the viewport size, etc
		m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pRenderTargetTexture->ptexTexture, 0);
		m_pWrapper->m_pD3DDevice7->SetViewport(&VPTemp);
	
		D3DRECT rcClear;
	
		rcClear.x1 = 0;
		rcClear.y1 = 0;
		rcClear.x2 = m_pRenderTargetTexture->m_ddsd.dwWidth;
		rcClear.y2 = m_pRenderTargetTexture->m_ddsd.dwHeight;

		//clear the buffer. Note that if your app isn't using a Z buffer, then your texture shouldn't have
		// one either, and so you'd omit that flag below
		hr = m_pWrapper->m_pD3DDevice7->Clear( 1, &rcClear, dwClearFlags,
								0xffffffff, 1.0f, 0 );


		// do a beginscene, set the texture to the previous frames scene, and draw our primitives
		m_pWrapper->m_pD3DDevice7->BeginScene();
	
		if(m_pSampleSphereObj)
			hr = m_pSampleSphereObj->Render(m_MxWorld,true);

		m_pWrapper->m_pD3DDevice7->EndScene();

		m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pWrapper->m_pBackBuffer, 0);
		m_pWrapper->m_pD3DDevice7->SetViewport(&VPOld);

		//clear the back buffer
		m_pWrapper->Clear( dwClearFlags );
		
		//go back to rendering our regular scene.
		m_pWrapper->m_pD3DDevice7->BeginScene();

	}


	if (m_bUseFallback | m_bForceFallback)
	{
		hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pFallbackTexture->dwStage, m_pFallbackTexture->ptexTexture);	
	}
	else
	{
		hr = m_pWrapper->m_pD3DDevice7->SetTexture( m_pRenderTargetTexture->dwStage, m_pRenderTargetTexture->ptexTexture);	
	}

	if(m_pSampleCubeObj)
		hr = m_pSampleCubeObj->Render(m_MxWorld,true);

	//Our main.cpp does the endscene for us, after it draws the text strings, frame counter, etc
	
	return S_OK;
}


// -----------------------------------------------------------------------
// HRESULT CTestRTT::TargetChanged( void )
//
// -----------------------------------------------------------------------
HRESULT CTestRTT::TargetChanged( void )
{
	HRESULT hr;

	//
	// This indicates that the devices of the DRGWrapper class have changed.
	// If you have anything that depends on the current rendering device, you'll
	// have to re-initialize it here
		

	m_bUseFallback = false; //we reset this whenever the device has changed, and retest the capability

	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_DITHERENABLE, TRUE );

	hr = m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_SPECULARENABLE, FALSE );

    hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage, D3DTSS_COLORARG2, D3DTA_CURRENT);
    hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage, D3DTSS_COLOROP,   D3DTOP_MODULATE );	


	hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage, D3DTSS_MINFILTER, D3DTFN_LINEAR );
	hr = m_pWrapper->m_pD3DDevice7->SetTextureStageState( m_pDefaultTexture->dwStage, D3DTSS_MAGFILTER, D3DTFN_LINEAR );


	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &m_MxProjection.m_Mx );

	// Set up a default light
    D3DLIGHT7 light;
    ZeroMemory( &light, sizeof(light) );
    light.dltType       = D3DLIGHT_DIRECTIONAL;
    light.dcvDiffuse.r  = 1.0f;
    light.dcvDiffuse.g  = 1.0f;
    light.dcvDiffuse.b  = 1.0f;
    light.dvRange       = D3DLIGHT_RANGE_MAX;
    
    light.dvDirection    = D3DVECTOR( 0.2f, 0.1f, 0.8f);

    // Set the light
    if( FAILED(m_pWrapper->m_pD3DDevice7->SetLight( 0, &light )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->LightEnable( 0, TRUE )))
		return hr;

    if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_LIGHTING, TRUE )))
		return hr;

	if( FAILED(m_pWrapper->m_pD3DDevice7->SetRenderState( D3DRENDERSTATE_AMBIENT, 0xff777777 ) ) )
		return hr;
	
	if (FAILED(hr = m_pWrapper->m_pD3DDevice7->SetRenderState(D3DRENDERSTATE_COLORVERTEX, TRUE) ))
		return hr;

	hr = SetupDeviceDependantItems();

	hr = SetupDeviceDependantItems();
	//if(!m_bUseFallback)
		hr = TestRTT();

	return hr;
}

HRESULT CTestRTT::SetupDeviceDependantItems(void)
{
	HRESULT hr;


	//restore textures. The render target surface, and the vidmem- and sysmem-forced surfaces
	// are not being handed off to DX texture manager, so we need to create them first

	if (m_pRenderTargetTexture)
	{
		if (m_pRenderTargetTexture->Restore(m_pWrapper->m_pD3DDevice7))
		{
			OutputDebugString("Surface creation with the 3DDevice and Texture flags succeeded.\n");
			OutputDebugString("This should only happen if the card supports RTT. Otherwise we \n");
			OutputDebugString("are dealing with a naughty DX7 driver, or an old DX6 driver \n");
		}
		else
		{
			OutputDebugString("Surface creation with the 3DDevice and Texture flags failed.\n");
			m_bUseFallback = true;
		}
	}


	if (m_pFallbackTemp)
		m_pFallbackTemp->Restore(m_pWrapper->m_pD3DDevice7);

	if (m_pFallbackTexture)
		m_pFallbackTexture->Restore(m_pWrapper->m_pD3DDevice7);
	
	if (m_pDefaultTexture)
		m_pDefaultTexture->Restore(m_pWrapper->m_pD3DDevice7);


	if(m_pSampleCubeObj)
		hr = m_pSampleCubeObj->TargetChanged(m_pWrapper, true);

	if(m_pSampleSphereObj)
		hr = m_pSampleSphereObj->TargetChanged(m_pWrapper, true);

	//create a default material
	D3DMATERIAL7	*ptempmat = new D3DMATERIAL7;
	ZeroMemory( ptempmat, sizeof(D3DMATERIAL7) );
  
	ptempmat->ambient.r = 0.7f;    ptempmat->ambient.g = 0.7f;    ptempmat->ambient.b = 0.7f;
	ptempmat->diffuse.r = 1.0f;    ptempmat->diffuse.g = 1.0f;    ptempmat->diffuse.b = 1.0f;
	ptempmat->specular.r = 1.0f;    ptempmat->specular.g = 1.0f;    ptempmat->specular.b = 1.0f;

	if( FAILED( hr = m_pWrapper->m_pD3DDevice7->SetMaterial(ptempmat) ) )
		return hr;

	m_pDefaultMaterial = new CDRGMaterial;
		
	m_pDefaultMaterial->SetMaterial(ptempmat);
	m_pDefaultMaterial->SetTexture(m_pDefaultTexture);
	
	m_pSampleSphereObj->SetMaterial(m_pDefaultMaterial);

	return S_OK;
}


HRESULT CTestRTT::TestRTT(void)
{

	//By the time we get here, the textures have been restored to video memory, and all
	// that is left to do is check the render to texture capability. If the HW can't do it,
	// if should fail the call to SetRenderTarget(texture). If it is a naughty driver though
	// we have to hack our way around it. This is what we do in the steps below
	// 
	// 1. Easy out check on the voodoo via Caps bit for separate texture memories, 
	//    resort to fallback method if found
	// 2. Clear the back buffer to black
	// 3. Set the render target to the texture surface
	//		4. if step 3 fails, resort to fallback method
	// 5. Clear the texture (new render target) to white
	// 6. Render something to it
	// 7. Lock the back buffer, and make sure it is all still black. If it isn't,
	//    then the driver can't RTT, but is misbehaving. We need to resort to the fallback method


	HRESULT hr;

	/*
	//this is just something you can put in to do a quick fix for voodoo boards
	if (m_pWrapper->m_pCurrentDevice->ddDeviceDesc.dwDevCaps & D3DDEVCAPS_SEPARATETEXTUREMEMORIES)
	{
		m_bUseFallback = true;
		return E_FAIL;
	}
	*/
	
	
	// clear back buffer to black
	DWORD dwClearColor = 0;
	D3DVALUE dvZClearValue = 1.0f;
	DWORD dwStencilClearValue = 0;
	D3DRECT rcClear;
	DWORD dwClearFlags;

	if (m_pWrapper->m_pZBuffer)
		dwClearFlags = D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER ;
	else
		dwClearFlags = D3DCLEAR_TARGET;

	
	// now switch the render target to the texture surface, but save off the viewport info first
	D3DVIEWPORT7	VPTemp, VPOld;

	m_pWrapper->m_pD3DDevice7->GetViewport(&VPOld);
	memcpy(&VPTemp,&VPOld,sizeof(D3DVIEWPORT7));
	VPTemp.dwWidth = m_pRenderTargetTexture->m_ddsd.dwWidth;
	VPTemp.dwHeight = m_pRenderTargetTexture->m_ddsd.dwHeight;


	if (m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pRenderTargetTexture->ptexTexture, 0) != D3D_OK)
	{
		OutputDebugString("\n\nSetting the render target failed. Resorting to fallback method\n\n");
		m_bUseFallback = true;
		return E_FAIL;
	}
	
	//clear to black, and set the viewport for the texture
	dwClearColor = 0x00000000;
	rcClear.x1 = 0;
	rcClear.y1 = 0;
	rcClear.x2 = m_pRenderTargetTexture->m_ddsd.dwWidth;
	rcClear.y2 = m_pRenderTargetTexture->m_ddsd.dwHeight;

	hr = m_pWrapper->m_pD3DDevice7->Clear( 1, &rcClear,   dwClearFlags,
							dwClearColor, dvZClearValue, dwStencilClearValue );
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPTemp);

	// now try rendering an object to the texture;
	m_pWrapper->m_pD3DDevice7->BeginScene();

	if(m_pSampleSphereObj)
		hr = m_pSampleSphereObj->Render(m_MxWorld,true);
	
	m_pWrapper->m_pD3DDevice7->EndScene();

	m_pWrapper->m_pD3DDevice7->SetRenderTarget(m_pWrapper->m_pBackBuffer, 0);
	m_pWrapper->m_pD3DDevice7->SetViewport(&VPOld);

	//now lock the back buffer and try reading it.
	DDSURFACEDESC2 SurfDesc;
	ZeroMemory( &SurfDesc, sizeof(DDSURFACEDESC2) );
	SurfDesc.dwSize = sizeof(DDSURFACEDESC2);
	
	//hr = m_pRenderTargetTexture->ptexTexture->Lock( NULL, &SurfDesc, DDLOCK_WAIT , NULL );
	
	hr = m_pFallbackTemp->ptexTexture->Blt(NULL, m_pRenderTargetTexture->ptexTexture,NULL,DDBLT_WAIT ,NULL);
	hr = m_pFallbackTemp->ptexTexture->Lock( NULL, &SurfDesc, DDLOCK_WAIT , NULL );

	
	DWORD dwColor = 0;
	DWORD colormask = SurfDesc.ddpfPixelFormat.dwRBitMask | 
		              SurfDesc.ddpfPixelFormat.dwGBitMask | 
					  SurfDesc.ddpfPixelFormat.dwBBitMask;
  
	for( DWORD y=0; y<SurfDesc.dwHeight; y++ )
	{
		WORD*  p16 =  (WORD*)((BYTE*)SurfDesc.lpSurface + y*SurfDesc.lPitch);
		DWORD* p32 = (DWORD*)((BYTE*)SurfDesc.lpSurface + y*SurfDesc.lPitch);
       
		for( DWORD x=0; x<SurfDesc.dwWidth; x++ )
		{
			if( 32 == SurfDesc.ddpfPixelFormat.dwRGBBitCount )
                dwColor |= (p32[x] & colormask);
            else
                dwColor |= (p16[x] & (WORD)colormask);
			
		}
		p16++;
		p32++;

	}
	// if dwColor is still 0, then the texture never got rendered to
	if (dwColor != 0)
	{
		OutputDebugString("\n\nnot zero! SRT works \n\n");
		m_bUseFallback = false;
	}
	else
	{
		OutputDebugString("\n\nzero! SRT fails\n\n");
		m_bUseFallback = true;
	}
			
	
	m_pFallbackTemp->ptexTexture->Unlock(NULL);
	

	return S_OK;
}












// -----------------------------------------------------------------------
// void CTestRTT::KeyUp(WPARAM wParam)
//
//   Keyboard keypress handler. 
// -----------------------------------------------------------------------
void CTestRTT::KeyDown(WPARAM wParam)
{
	//we need a variable to modify our rotation speed with frame rate	
	float scaler = 1.0f;

	//if exiting, the wrapper could be invalid, but we don't care, since we will exit anyway
	if (wParam == 'X') 
		return;

	if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
	{
		if (m_pWrapper->m_fFrameTime >0.0f)
			scaler = m_pWrapper->m_fFrameTime;  
	}
		
	switch (wParam) 
	{
	case VK_UP:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, -3.14159f*scaler );
		break;
	
	case VK_DOWN:
		m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler );
		break;
	
	case VK_LEFT:
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, -3.14159f*scaler );
		break;
	
	case VK_RIGHT:	
		m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler );
		break;
	case 'B':
		m_bForceFallback = !m_bForceFallback;
		break;
	}

	if (m_pWrapper)
	{
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );
	}

	
	return;
}

void CTestRTT::KeyUp(WPARAM wParam)
{		
	switch (wParam) 
	{
		//add code here to handle key releases, if you need them (as in remmed out example)
	case VK_UP:
		//OutputDebugString("The up arrow was released;");			
		break;
	}
	
	return;
}


void CTestRTT::MouseLeftUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CTestRTT::MouseLeftDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CTestRTT::MouseRightUp(WPARAM wParam, LPARAM lParam)
{	
	return;
}

void CTestRTT::MouseRightDown(WPARAM wParam, LPARAM lParam)
{		
	return;
}

void CTestRTT::MouseMove(WPARAM wParam, LPARAM lParam)
{
	if ( wParam & MK_LBUTTON )	// Left button down plus mouse move == rotate
	{
		//we need a variable to modify our rotation speed with frame rate	
		float scaler = 1.0f;
		if(m_pWrapper) //we could get into this loop after the wrapper has been invalidates
		{
			scaler = 2.0f / max(m_pWrapper->m_dwRenderWidth, m_pWrapper->m_dwRenderHeight);  
		}

		//we need to compare the last position to the current position
		int x = LOWORD(lParam);
		int y = HIWORD(lParam);
		
		if (x - m_iMousePrevX)
		{
			m_MxView.Rotate( m_MxView.m_Mx._12, m_MxView.m_Mx._22, m_MxView.m_Mx._32, 3.14159f*scaler*(float)(x - m_iMousePrevX) );
		}
			
		if (y - m_iMousePrevY)
		{
			m_MxView.Rotate( m_MxView.m_Mx._11, m_MxView.m_Mx._21, m_MxView.m_Mx._31, 3.14159f*scaler*(float)(y - m_iMousePrevY) );
		}

		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_WORLD, &m_MxWorld.m_Mx );
		m_pWrapper->m_pD3DDevice7->SetTransform( D3DTRANSFORMSTATE_VIEW, &m_MxView.m_Mx );

	
		m_iMousePrevX = x;
		m_iMousePrevY = y;

	}

	else
	{
		//need to update these, or the next time someone clicks, the delta is massive
		m_iMousePrevX = LOWORD(lParam);
		m_iMousePrevY = HIWORD(lParam);	
	}
	return;
}

