//-----------------------------------------------------------------------------
// 3DDevice.h
//
// Started 06/20/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#if !defined(DEVICE_H_INCLUDED)
#define DEVICE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "Common.h"
#include "DDraw.h"
#include "D3d.h"
#include "Textures.h"

typedef bool (*RenderDeviceCallback)(void* context,D3DDEVICEDESC7* dev_desc);

//-----------------------------------------------------------------------------
// Creates a 3D device, back surface, front surface and Z buffer and then
// initializes it for rendering.
// You can only create one if these
class RenderDevice
{
public:
    //
    // Constructors and Destructor
    //

    //-----------------------------------------------------------------------------
    RenderDevice
    (
    );

    //-----------------------------------------------------------------------------
    ~RenderDevice
    (
    );

    //-----------------------------------------------------------------------------
    bool Initialize
    (
        HWND                    hwnd,
        uint32                  width,
        uint32                  height,
        uint32                  colorbpp,
        uint32                  zbpp,
        bool                    stencil,
        RenderDeviceCallback    func,
        void*                   func_context
    );

    //-----------------------------------------------------------------------------
    void DisplayBackBuffer
    (
        bool wait_vsync
    );

    //-----------------------------------------------------------------------------
    bool Resize
    (
        int32   width,
        int32   height
    );

    //-----------------------------------------------------------------------------
    // This will copy the specified region of frame buffer to the texture map at
    // location specified. No scaling or clipping is performed so the caller is
    // responsible for making sure the coordinates passed in are valid.
    // NOTE: This function will perform format conversion if the destination
    // texture format is different from the frame buffer but there is a HUGE bug
    // in DD; In the surface formats match then there is no issue but it they do
    // not match the surface paramters are backwards and the texture is blited
    // to the frame buffer!!!!!
    bool CopyTargetToTexture
    (
        uint32      render_x,
        uint32      render_y,
        uint32      render_width,
        uint32      render_height,
        uint32      dest_x,
        uint32      dest_y,
        Texture2D*  dest
    );

    //-----------------------------------------------------------------------------
    bool Restore
    (
    );

    //-----------------------------------------------------------------------------
    static IDirectDraw7* GetDD
    (
    )
    //--------------------------------------
    {
        return m_DD;
    }

    //-----------------------------------------------------------------------------
    static IDirect3D7* GetD3D
    (
    )
    //--------------------------------------
    {
        return m_D3D;
    }

    //-----------------------------------------------------------------------------
    // This will return 'true' if a hardware device is being used or 'false' for
    // software devices. Applications can call this to determine if a hardware
    // device is being used as it may need to vary the level of tessllation of
    // geometry when moving from hardware to software.
    //
    // The results of this function also dictate what sort of memory surfaces
    // should be created in:
    // Software - DDSCAPS_SYSTEMMEMORY
    // Hardware - DDSCAPS_VIDEOMEMORY (AGP or LOCAL)
    static bool UsingHardware
    (
    )
    //--------------------------------------
    {
        return m_UsingHardware;
    }

    //-----------------------------------------------------------------------------
    static const GUID& DeviceType
    (
    )
    //--------------------------------------
    {
        return m_DeviceType;
    }

    //-----------------------------------------------------------------------------
    static IDirect3DDevice7* GetDevice
    (
    )
    //--------------------------------------
    {
        return m_D3DDevice;
    }

    //-----------------------------------------------------------------------------
    static IDirectDrawSurface7* GetBackBuffer
    (
    )
    //--------------------------------------
    {
        return m_DDRenderSurface;
    }

    //-----------------------------------------------------------------------------
    static IDirectDrawSurface7* GetFrontBuffer
    (
    )
    //--------------------------------------
    {
        return m_DDPrimarySurface;
    }

    //-----------------------------------------------------------------------------
    static IDirectDrawSurface7* GetDepthBuffer
    (
    )
    //--------------------------------------
    {
        return m_DepthBuffer;
    }

    //-----------------------------------------------------------------------------
    uint32 GetRenderWidth
    (
    )
    //--------------------------------------
    {
        return m_RenderWidth;
    }

    //-----------------------------------------------------------------------------
    uint32 GetRenderHeight
    (
    )
    //--------------------------------------
    {
        return m_RenderHeight;
    }

    //-----------------------------------------------------------------------------
    // Static function to create a 2D texture directly from a file, you can pass
    // any of the extensions known by the class load function,  a texture class
    // will be created.
    //
    // This function also displays error messages for the user if a file cannot
    // be found. If texture are manually loaded you are responsible for your
    // own error handling.
    //
    // A specifc format to create the with must be specified, if you want to pick
    // a best fit format the call MatchFormat first.
    static Texture2D* CreateTexture
    (
        const char*     bitmap,
        TextureFormat   format
    );


    //-----------------------------------------------------------------------------
    // As above but creates the texture from the resources. This function looks in
    // the RT_BITMAP resource section for the given ID.
    static Texture2D* CreateTexture
    (
        uint32          bitmap,
        TextureFormat   format
    );


    //-----------------------------------------------------------------------------
    static TextureFormat MatchFormat
    (
        TextureFormat   fmt
    );

    //-----------------------------------------------------------------------------
    static bool FindCompatibleD3DTextureFormat
    (
        TextureFormat   fmt,
        DDPIXELFORMAT&  ddfmt
    );

    //-----------------------------------------------------------------------------
    static uint32 TextureFormatCount
    (
    )
    //--------------------------------------
    {
        return m_NumTextureFormats;
    }

    //-----------------------------------------------------------------------------
    static const DDPIXELFORMAT* D3DTextureFormats
    (
    )
    //--------------------------------------
    {
        return m_TexturePixelFormats;
    }

private:

    //-----------------------------------------------------------------------------
    bool Create2D
    (
    );

    //-----------------------------------------------------------------------------
    bool Create3D
    (
        RenderDeviceCallback    func,
        void*                   func_context
    );

    //-----------------------------------------------------------------------------
    bool RenderDevice::CreateRenderBuffer
    (
        uint32                  width,
        uint32                  height,
        uint32                  bpp,
        IDirectDrawSurface7**   render_surface
    );

    //-----------------------------------------------------------------------------
    bool CreateAttachedBackAndDepthBuffer
    (
        uint32                  width,
        uint32                  height,
        uint32                  zbpp,
        bool                    stencil,
        IDirectDrawSurface7**   render_surface,
        IDirectDrawSurface7**   depth_surface
    );

    //-----------------------------------------------------------------------------
    void CreateZBufferDesc
    (
        DDSURFACEDESC2& desc,
        uint32          width,
        uint32          height,
        uint32          zbpp,
        uint32          stencil
    );

    //-----------------------------------------------------------------------------
    bool EnumerateTextureFormats
    (
    );

    //-----------------------------------------------------------------------------
    static HRESULT CALLBACK DeviceCallback
    (
        LPSTR lpDeviceDescription, 
        LPSTR lpDeviceName, 
        LPD3DDEVICEDESC7 lpD3DDeviceDesc,
        LPVOID lpContext
    );

    //-----------------------------------------------------------------------------
    static HRESULT CALLBACK ZBufferCallback
    (
        LPDDPIXELFORMAT pixel_format,  
        LPVOID          context    
    );

    //-----------------------------------------------------------------------------
    static HRESULT CALLBACK RenderDevice::PixelFormatCountCallback
    (
        LPDDPIXELFORMAT pixel_format,  
        LPVOID          context    
    );

    //-----------------------------------------------------------------------------
    static HRESULT CALLBACK RenderDevice::PixelFormatCopyCallback
    (
        LPDDPIXELFORMAT pixel_format,  
        LPVOID          context    
    );

    HWND                    m_RenderWindow;         // The window we are attached to
    uint32                  m_RenderWidth;          // Width of the current back and Z buffers
    uint32                  m_RenderHeight;         // Width of the current back and Z buffers
    uint32                  m_RenderBpp;            // Bit depth of render buffer or 0xffffffff
    bool                    m_StencilBuffer;        // Set if we asked for a stencil buffer
    uint32                  m_ZBits;                // Bit depth of Z buffer (0 if no Z, 0xffffffff for some format)
    bool                    m_FoundHW;              // Set if we found a usefil hardware device
    bool                    m_FoundSW;              // Set if we found a useful software device
    bool                    m_FoundTnL;             // Set if we found a useful hardware T&L device
    bool                    m_FoundRef;             // Set if the reference rasterizer is useful to use

    static IDirectDrawSurface7*    m_DDPrimarySurface;
    static IDirectDrawSurface7*    m_DDRenderSurface;
    static IDirectDrawSurface7*    m_DepthBuffer;
    static IDirect3DDevice7*       m_D3DDevice;
    DDPIXELFORMAT           m_ZCreateFormat;
    static GUID             m_DeviceType;
    static bool             m_UsingHardware;
    static IDirectDraw7*    m_DD;
    static IDirect3D7*      m_D3D;
    static uint32           m_NumTextureFormats;
    static DDPIXELFORMAT*   m_TexturePixelFormats;
};

//
// Include the correct DX libs
//
#pragma comment(lib, "ddraw.lib")
#pragma comment(lib, "dxguid.lib")

// Include the correct library
#if defined(_DEBUG)
#pragma comment(lib, "DirectXD.lib")
#else
#pragma comment(lib, "DirectXR.lib")
#endif

#endif //#if !defined(DEVICE_H_INCLUDED)
