//-----------------------------------------------------------------------------
// Euler.h
//
// Started 06/21/2000
//
// Shrike Framework -  Rob Wyatt 
//-----------------------------------------------------------------------------
#if !defined(EULER_H_INCLUDED)
#define EULER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Common.h"
#include "Matrix.h"
#include "Debug.h"


//----------------------------------------------------------------------------
// Euler rotation orders for the Euler matrix construstor
enum EulerOrder
{
    // Special cases for convinence
    EULER_X,        // Rotate just around X
    EULER_Y,        // Rotate just around Y
    EULER_Z,        // Rotate just around Z

    // Normal euler orders...
    EULER_XYZ,      // Rotate around X, then Y, then Z
};


//----------------------------------------------------------------------------
// Basic Euler angle class, the above orders are defined. For ease of use some
// single angle rotations are specified for performing axis aligned rotations.
class EulerAngle
{
public:

    //----------------------------------------------------------------------------
    EulerAngle
    (
    )
    //--------------------------------------
    {
    }

    //----------------------------------------------------------------------------
    // Using the single angle constructor we must only use one of the orders that
    // expect a single angle
    EulerAngle
    (
        EulerOrder  order,
        float       ang0
    )
    //--------------------------------------
    {
        Assert((order == EULER_X) || (order == EULER_Y) || (order == EULER_Z));
        m_MultiplyOrder = order;
        m_FirstAngle = ang0;
    }

    //----------------------------------------------------------------------------
    EulerAngle
    (
        EulerOrder  order,
        float       ang0,
        float       ang1,
        float       ang2
    )
    //--------------------------------------
    {
        m_MultiplyOrder = order;
        m_FirstAngle    = ang0;
        m_SecondAngle   = ang1;
        m_ThirdAngle    = ang2;
    }

    //----------------------------------------------------------------------------
    // Convert the euler to a matrix
    operator Matrix4
    (
    ) const;


    //
    // Data members
    //
    EulerOrder  m_MultiplyOrder;
    float       m_FirstAngle;
    float       m_SecondAngle;
    float       m_ThirdAngle;
};

#endif
