//-----------------------------------------------------------------------------
// HlpLib.h
//
// Started 06/21/2000
//
// Shrike Framework -  Rob Wyatt 
//-----------------------------------------------------------------------------
#if !defined(HLPLIB_H_INCLUDED)
#define HLPLIB_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif

//
// Bit operators
//

//----------------------------------------------------------------------------
// returns true if the val passed in is a power of 2
// NOTE: the type must support the & operator and be able to subtract one in
//       its own type
template<typename T>  __forceinline bool IsPowerofTwo
(
    T   val
)
//--------------------------------------
{
    return (val & (val - T(1))) == 0;
}

//----------------------------------------------------------------------------
// Returns the number of set bits in the variable passed in.
// NOTE: the type must support the & operator
template<typename T> inline uint32 BitCount
(
    T   val
)
//--------------------------------------
{
    uint32 type_bits = sizeof(T)*8;
    uint32 set_bits = 0;
    uint32 bit = 0;

    // Do the operation without shifting the type T to the right, if we do shift
    // we have to worry about signed and unsigned types.

    while (bit<type_bits)
    {
        if (val & (1<<bit))
            set_bits++;
        bit++;
    }

    return set_bits;
}


//----------------------------------------------------------------------------
// Return the bit position of the lowest set bit
// NOTE: the type must support the & operator
template<typename T> inline uint32 LowestBit
(
    T   val
)
//--------------------------------------
{
    uint32 type_bits = sizeof(T)*8;
    uint32 bit = 0; 

    if (val == 0)
        return 0;

    // Do the operation without shifting the type T to the right, if we do shift
    // we have to worry about signed and unsigned types.

    while (bit<type_bits)
    {
        if (val & (1<<bit))
            return bit;
        bit++;
    }

    return 0;
}



//
// General Operators
//
#undef Max
//----------------------------------------------------------------------------
// Returns the max of the two input parameters, becareful because windows
// defines a Max macro, hence the undef abaove
template<typename T>  __forceinline T Max
(
    T   a,
    T   b
)
//--------------------------------------
{
    return a>=b ? a : b;
}

#undef Min
//----------------------------------------------------------------------------
// Returns the Min of the two input parameters, becareful because windows
// defines a Min macro, hence the undef abaove
template<typename T> __forceinline T Min
(
    T   a,
    T   b
)
//--------------------------------------
{
    return a<=b ? a : b;
}

//----------------------------------------------------------------------------
// Return the absolute value of the the value passed in
template<typename T> __forceinline T Abs
(
    T   a
)
//--------------------------------------
{
    return a<=(T)0 ? -a : a;
}

//----------------------------------------------------------------------------
// Sgn function of the type passed in:
//  -1  if less than zero
//  0   if zro
//  +1  if greater than zero
template<typename T> __forceinline T Sgn
(
    T   a
)
//--------------------------------------
{
    return a < (T)0 ? T(-1) : a > (T)0 ? T(1) : T(0);
}


//----------------------------------------------------------------------------
// Returns true if the inputs a and b are within Range of each other
template<typename T> __forceinline T Fuzzy
(
    T   a,
    T   b,
    T   Range
)
//--------------------------------------
{
    return (Abs(a-b))<Range);
}

#endif
