//-----------------------------------------------------------------------------
// MeshHelp.h
//
// Started 06/24/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#if !defined(MESHHELP_H_INCLUDED)
#define MESHHELP_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Common.h"
#include "Vertex.h"
#include "D3D.h"


//-----------------------------------------------------------------------------
// Namespace with functions to create common test objects. These could be global
// but that is messy.
namespace TestObject
{
    //-----------------------------------------------------------------------------
    // Enum of the predefined objects
    enum TestObj
    {
        TEST_OBJECT_INDEXED_UNIQUE_CUBE
    };

    //-----------------------------------------------------------------------------
    // Create the vertex list for the specified object.
    // We return an actual copy of the vertex data and not just a pointer,
    // this enables the caller to modify the data and not affect other test models.
    // Therefore this list is unique and the caller is responsible for deleting it.
    DiffuseTextureVertex1* MakeVertexList
    (
        TestObj object,
        uint32& count
    );

    //-----------------------------------------------------------------------------
    // Create the index list for the specified object, an assert will fire if this
    // function is called for a non-index model. The list returned is copied so
    // like the above function the caller is responsible for deleting the returned
    // data.
    uint16* MakeTriangleListIndices
    (
        TestObj object,
        uint32& count
    );
};


//-----------------------------------------------------------------------------
// Namespace with mesh helper functions, such as generate strips and converting
// system memory buffer to vertex buffers
namespace MeshHelp
{
    //-----------------------------------------------------------------------------
    // Converts the given list of vertices in system memory into a vertex buffer,
    // the vertex buffer is identical in format and contents as the source buffer.
    IDirect3DVertexBuffer7* GenerateVertexBuffer
    (
        void*   mesh_data,
        uint32  vert_count,
        uint32  vert_size,
        uint32  vert_format,
        bool    write_only = true
    );
};


// Include the correct library
#if defined(_DEBUG)
#pragma comment(lib, "GeomD.lib")
#else
#pragma comment(lib, "GeomR.lib")
#endif


#endif
