//-----------------------------------------------------------------------------
// Plane.h
//
// Started 06/20/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#if !defined(PLANE_H_INCLUDED)
#define PLANE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Common.h"
#include "Vector.h"
#include "Debug.h"


//----------------------------------------------------------------------------
// D3D has no built in plane type
class Plane
{
public:
    //----------------------------------------------------------------------------
    Plane
    (
    )
    //--------------------------------------
    {
    }

    //----------------------------------------------------------------------------
    // Construct a plane from 3 points, the normal to plane created by this function
    // is not normalized.
    Plane
    (
        Vector3&    pos_a,
        Vector3&    pos_b,
        Vector3&    pos_c
    )
    //--------------------------------------
    {
        m_Normal = (pos_c-pos_b) ^ (pos_a-pos_b);
        m_Distance = -(m_Normal*pos_a);
    }

    //----------------------------------------------------------------------------
    // Construct a plane directly from a normal and distance
    Plane
    (
        Vector3&    n,
        float       d 
    )
    : m_Normal(n), m_Distance(d)
    //--------------------------------------
    {
    }

    //
    // Operators
    //

    //----------------------------------------------------------------------------
    // Multiply plane by a constant just moves it relative to the origin
    inline Plane& operator *=
    (
        float scale
    )
    //--------------------------------------
    {
        m_Distance*=scale;

        return *this;
    }

    //----------------------------------------------------------------------------
    inline Plane operator *
    (
        float scale
    )
    //--------------------------------------
    {
        return Plane(m_Normal,m_Distance*scale);
    }


    //
    // Member functions
    //

    //----------------------------------------------------------------------------
    // Normalize the plane so the normal is a unit normal and the distance is a
    // real distance.
    Plane& Normalize
    (
    )
    //--------------------------------------
    {
        float f = 1.0f/m_Normal.Magnitude();

        m_Normal.x *= f;
        m_Normal.y *= f;
        m_Normal.z *= f;
        m_Distance *= f;

        return *this;
    }


    //----------------------------------------------------------------------------
    // Return the distance from the point to the plane.
    float Distance
    (
        Vector3&    point
    )
    //--------------------------------------
    {
        // Distance from plane is point dot normal + distance
        return (point*m_Normal)+m_Distance;
    }

    //----------------------------------------------------------------------------
    // Return the distance from the plane to the specified 4D vector.
    // The point passed in must be a point and not a vector, ie W must not be zero
    float Distance
    (
        Vector4&    point
    )
    //--------------------------------------
    {
        Assert(point.w != 0.0f);
        return point *  ( *((Vector4*)(this)) );
    }


    // Data members
    Vector3 m_Normal;
    float   m_Distance;
};


#endif


