//-----------------------------------------------------------------------------
// ShrikeMath.h
//
// Started 06/21/2000
//
// Shrike Framework -  Rob Wyatt 
//-----------------------------------------------------------------------------
#if !defined(SHRIKEMATH_H_INCLUDED)
#define SHRIKEMATH_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif

#include "math.h"           // std C lib

// Useful float constants
const float PI              = 3.1415926535899f;                    // Pi  (180 degrees)
const float PI_MUL_2        = 6.2831853071796f;                    // 2 * Pi (360 degrees)
const float PI_DIV_2        = 1.5707963267949f;                    // Pi / 2 (90 degrees)
const float PI_DIV_4        = 0.7853981633974f;                    // Pi / 4 (45 degrees)
const float INV_PI          = 0.3183098861838f;                    // 1 / Pi
const float SQRT2           = 1.414213562f;                       // sqrt(2)
const float SQRT2_DIV_2     = 0.707106781f;                       // sqrt(2)/2
const float EXP_1           = 2.718281828f;                       // e
const float DEG_TO_RAD      = 0.017453293f;                       // Degrees to Radians
const float RAD_TO_DEG      = 57.29577951f;                       // Radians to Degrees
const float FLOAT_SMALL     = 1.0e-6f;                            // Small number for floats
const float FLOAT_HUGE      = 1.0e+38f;                           // Huge number for floats
const float FLOAT_INFINITY  = 3.40282346638528860e+38f;           // Infinity value for float
const float FLOAT_EPSILON   = 1.0e-4f;                            // Tolerance for floats
const float NORMAL_EPSILON  = 0.01f;                              // Tolerance for normal vector lengths

// Floating point manipulation macros
#define FP_BITS(fp) (*(DWORD *)&(fp))
#define FP_ABS_BITS(fp) (FP_BITS(fp)&0x7FFFFFFF)
#define FP_SIGN_BIT(fp) (FP_BITS(fp)&0x80000000)
#define FP_ONE_BITS     0x3F800000
#define FP_HALFH        0x3f000000
#define FP_EXP_BITS     0x7f800000
#define FP_SMAN_BITS    0x807fffff

//-----------------------------------------------------------------------------
// Strucrture to hold the result of the function below
struct SinCosPair
{
    float   c;
    float   s;
};

//-----------------------------------------------------------------------------
// Calculate both sin and cos of an angle at the same time as very often you
// need both of them and this function is the same speed as just on of them.
__forceinline void SinCos(float f,SinCosPair& res)
{
    _asm
    {
        mov eax,res
        fld DWORD PTR[f]
        fsincos
        fstp DWORD PTR[eax]
        fstp DWORD PTR[eax+4]
    }
}

#endif


