//-----------------------------------------------------------------------------
// SuperQuadric.h
//
// Started 06/24/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#if !defined(SUPERQUADRIC_H_INCLUDED)
#define SUPERQUADRIC_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Tessellate2D.h"


//-----------------------------------------------------------------------------
// Graphics Gems III.8	Alan Barr, "Physically Based Superquadrics"
class SuperQuadric : public Tessellate2D
{
public:

    //-----------------------------------------------------------------------------
    // Constructor for the base class does not do much
    SuperQuadric
    (
        float   scale_x,
        float   scale_y,
        float   scale_z,
        float   curv_e,
        float   curv_n,
        uint32  tess_u,
        uint32  tess_v
    );

    //-----------------------------------------------------------------------------
    // Virtual destructor
    virtual ~SuperQuadric
    (
    );

    //-----------------------------------------------------------------------------
    // This is the function that we have to implement. This function takes the U,V
    // passed in (0.0 to 1.0) and generates a point on the surface of the super
    // quadric.
    virtual Vector3 GeneratePosition
    (
        float   u,
        float   v
    );

    //-----------------------------------------------------------------------------
    // For quadric shapes the normals have to be analytically generated, the default
    // partial derirative method in the base class is not enough due to the nature
    // of the super quadrics. The problem is that as the super quadric becomes more
    // square the u,v space gets compressed and therefore the deltas used by the
    // default implementation approach zero and the resulting normal is degenerate.
    virtual Vector3 GenerateNormal
    (
        float   u,
        float   v
    );

private:
    float   m_ScaleX;
    float   m_ScaleY;
    float   m_ScaleZ;
    float   m_CurvE;
    float   m_CurvN;
};

#endif
