//-----------------------------------------------------------------------------
// Textures.h
//
// Started 06/09/2000
//
// Shrike Framework -  Rob Wyatt 
//-----------------------------------------------------------------------------
#if !defined(TEXTURES_H_INCLUDED)
#define TEXTURES_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Common.h"
#include "DDraw.h"
#include "D3D.h"
#include "PixelFormat.h"

//-----------------------------------------------------------------------------
// Create a conventional 2D texture map
class Texture2D
{
public:
    //-----------------------------------------------------------------------------
    // Constructor does nothing apart from make the class safe to delete.
    Texture2D
    (
    );

    //-----------------------------------------------------------------------------
    // Destructor will delete the surface if it has been initialized otherwise it
    // does nothing.
    ~Texture2D
    (
    );

    //-----------------------------------------------------------------------------
    // Initialize a texture surface in the given format with the specified width and
    // height. The data within the surface is undefined.
    bool Initialize
    (
        uint32          width,
        uint32          height,
        TextureFormat   format
    );

    //----------------------------------------------------------------------------
    // Loads a graphic file into the surface.
    // If the surface has not been initialized this function will automatically
    // initialize the surface with the specified format and dimensions to match
    // the source file. If the surface has been initialized then it is restored
    // and the specified file is reloaded (useful for when surfaces get lost).
    //
    // If a surface with alpha is created and the file loaded has no alpha then
    // the resulting surface alpha is undefined.
    //
    //
    // Currently the following formats can be loaded:
    //
    //  BMP 1 bit (paletized)                       ->  Any surface format
    //  BMP 4 bit (paletized)                       ->  Any surface format
    //  BMP 8 bit (paletized)                       ->  Any surface format
    //  BMP 16 bit (true colour)                    ->  Any surface format
    //  BMP 24 bit (true colour)                    ->  Any surface format
    //
    bool LoadFile
    (
        const char*     bitmap,
        TextureFormat   format = TF_NONE
    );

    //----------------------------------------------------------------------------
    // As above but looks in the resource section for the file instead of looking
    // for a file.
    // Various file types must be different resource sections, as follows:
    //  BMP     -> Standard Bitmap folder   ->  RT_BITMAP
    //  
    bool LoadFile
    (
        uint32          bitmap,
        TextureFormat   format = TF_NONE
    );


    //----------------------------------------------------------------------------
    bool LoadBMP
    (
        BITMAPINFO*     pbmp,
        TextureFormat   format
    );


    //----------------------------------------------------------------------------
    // Return a pointer to the actual DX surface so it can be used in API calls.
    IDirectDrawSurface7* GetTexture
    (
    )
    //--------------------------------------
    {
        return m_TextureSurface;
    }


    //----------------------------------------------------------------------------    
    // Return the format of the texture
    PixelFormat GetTextureFormat
    (
    )
    //--------------------------------------
    {
        return m_Format;
    }

private:
    IDirectDrawSurface7*    m_TextureSurface;
    TextureFormat           m_Format;
    uint32                  m_Width;
    uint32                  m_Height;
};


//
// Include the correct DX libs
//
#pragma comment(lib, "ddraw.lib")
#pragma comment(lib, "dxguid.lib")

// Include the correct library
#if defined(_DEBUG)
#pragma comment(lib, "DirectXD.lib")
#else
#pragma comment(lib, "DirectXR.lib")
#endif


#endif
