//-----------------------------------------------------------------------------
// Vertex.h
//
// Started 06/22/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#if !defined(VERTEX_H_INCLUDED)
#define VERTEX_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "common.h"
#include "D3d.h"
#include "Vector.h"
#include "Colors.h"
#include "debug.h"

// As these structures are defined by hardware so pack as they are
#pragma pack(push,1)

//------------------------------------------------------------------------------
// A class to hold a 2D teture co-ordinate
class TextureCoord2D
{
public:
    float   m_TexU;
    float   m_TexV;
};

//------------------------------------------------------------------------------
// A class to hold a 3D teture co-ordinate. Class also has some conversion 
// operators.
class TextureCoord3D
{
public:
    //------------------------------------------------------------------------------
    inline TextureCoord3D& operator = 
    (
        const Vector3& v
    )
    //--------------------------------------
    {
        m_TexU = v.x;
        m_TexV = v.y;
        m_TexQ = v.z;

        return *this;
    }

    float   m_TexU;
    float   m_TexV;
    float   m_TexQ;
};

//------------------------------------------------------------------------------
// A class to hold a 4D teture co-ordinate.  Class also has some conversion 
// operators.
class TextureCoord4D
{
public:
    //------------------------------------------------------------------------------
    inline TextureCoord4D& operator = 
    (
        const Vector4& v
    )
    //--------------------------------------
    {
        m_TexR = v.x;
        m_TexS = v.y;
        m_TexT = v.z;
        m_TexQ = v.w;

        return *this;
    }

    float   m_TexR;
    float   m_TexS;
    float   m_TexT;
    float   m_TexQ;
};


//------------------------------------------------------------------------------
// Define a version of the above with a diffuse color
struct DiffuseBasicVertex
{
    //------------------------------------------------------------------------------
    enum
    {
        FORMAT = D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE
    };

    //------------------------------------------------------------------------------
    DiffuseBasicVertex
    (
    )
    //--------------------------------------
    {
    }

    //------------------------------------------------------------------------------
    DiffuseBasicVertex
    (
        Vector3&        v,
        Vector3&        n,
        RGBColor&       c
    ) :  m_Pos(v),m_Normal(n),m_DiffuseCol(c)
    //--------------------------------------
    {
    }

    //
    // Data for this vertex type consists of a position and a normal
    //
    Vector3     m_Pos;
    Vector3     m_Normal;
    D3DCOLOR    m_DiffuseCol;
};



//------------------------------------------------------------------------------
// Define a diffuse textured vertex with 1 texture coordinate
struct DiffuseTextureVertex1
{
    //------------------------------------------------------------------------------
    enum
    {
        FORMAT = D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX1
    };

    //------------------------------------------------------------------------------
    // Define a default constructor so we have have arrays of uninitialized vertieces
    DiffuseTextureVertex1
    (
    )
    //--------------------------------------
    {
    }

    //------------------------------------------------------------------------------
    // Define a constructor for our vertex class/structure so we can construct them
    // in place.
    DiffuseTextureVertex1
    (
        Vector3&        vpos,
        Vector3&        n,
        RGBColor&       c,
        float           u,
        float           v
    ) :  m_Pos(vpos),m_Normal(n),m_DiffuseCol(c)
    //--------------------------------------
    {
        m_Textures[0].m_TexU   = u;
        m_Textures[0].m_TexV   = v;
    }

    //
    // Data for this vertex type consists of a position and a normal
    //
    Vector3         m_Pos;
    Vector3         m_Normal;
    D3DCOLOR        m_DiffuseCol;
    TextureCoord2D  m_Textures[1];
};


#pragma pack(pop)

#endif


