//-----------------------------------------------------------------------------
// TestApp.h
//
// Started 06/09/2000
//
// Shrike Framework -  Rob Wyatt 
//-----------------------------------------------------------------------------
#if !defined(TESTAPP_H_INCLUDED)
#define TESTAPP_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "WinShell.h"
#include "Textures.h"
#include "Vector.h"
#include "Matrix.h"
#include "Euler.h"
#include "Lights.h"
#include "Material.h"
#include "Vertex.h"
#include "MeshHelp.h"
#include "resource.h"

//------------------------------------------------------------------------------
// Test app class, derived from the shrike Shell so all that needs to be done is
// implement a few fey virtual functions and you have a fully functioning app.
class TestApp : public Shell
{
public:

    //------------------------------------------------------------------------------
    // Called when a menu is selected
    virtual bool MenuSelect
    (
        uint32  menu_id     // resource ID of the selected menu item
    );

    //------------------------------------------------------------------------------
    // Called when the mouse is moved or clicked, see Shell.h for action codes
    virtual bool MouseClick
    (
        MouseAction     action,
        int32           x,
        int32           y
    );

    //------------------------------------------------------------------------------
    // Called to check a device, if the function returns true the device is considered
    // for use. If false is returned then the device is ignored.
    virtual bool DeviceCheck
    (
        D3DDEVICEDESC7* device
    );

    //------------------------------------------------------------------------------
    // First function to be called while initializing the shell, windows and the
    // registry are going but D3D is not. This function is only called once per
    // application.
    virtual bool Setup
    (
        bool    first_run
    );

    //------------------------------------------------------------------------------
    // Close function to pair with the above init, this is called after D3D has been
    // destroyed but windows is still OK. This is only called once per app run, at
    // the very end.
    virtual bool Closedown
    (
    );

    //------------------------------------------------------------------------------
    // Called once a device has been selcted, the created device interface is passed
    // in. D3D is now fully functioning and in this function textures can be loaded
    // and vertex buffers created. This can be called multiple times because it is
    // called when a new device is created such as a mode change. It is also called
    // with resize set to true when the window is resized, useful for reseting the
    // projection matrix.
    virtual bool DeviceSetup
    (
        IDirect3DDevice7*   device,
        bool                resize
    );

    //------------------------------------------------------------------------------
    // Close function for the deivce, called just before the device is destroyed.
    // Delete textures and vertex buffers, this function can be called more than
    // once such as when changing mode as the old device needs to be closed and a
    // new one created.
    virtual bool DeviceClose
    (
        IDirect3DDevice7*   device
    );

    //------------------------------------------------------------------------------
    // Called when surfaces have been lost. This function should restore all surfaces
    // and vertex buffers via whatever means works.
    virtual bool Restore
    (
        IDirect3DDevice7*   device
    );

    //------------------------------------------------------------------------------
    // Handle frame step, teh times passed in are paused when the application becomes
    // inactive.
    virtual bool Step
    (
        IDirect3DDevice7*   device,
        float               frame_time,
        float               frame_step
    );

    //------------------------------------------------------------------------------
    // Handle frame render, return true if all went well and the back buffer needs
    // to be displayed in the window.
    virtual bool Render
    (
        IDirect3DDevice7*   device
    );


    // Working variables
    float                   m_MouseX;
    float                   m_MouseY;
    Matrix4                 m_WorldMat;
    DiffuseTextureVertex1*  m_UnitCube;
    uint16*                 m_Index;
    uint32                  m_VertCount;
    uint32                  m_IndexCount;
    Texture2D*              m_BoxTexture;
};

#endif
