//-----------------------------------------------------------------------------
// Texture.cpp
//
// Started 06/22/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "PixelFormat.h"
#include "HlpLib.h"
#include "Debug.h"
#include "3DDevice.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------------------
PixelFormat::PixelFormat
(
    TextureFormat   fmt
)
//--------------------------------------
{
    m_DDFmt.dwFlags = 0;
    DDPIXELFORMAT   ddfmt;
    if (RenderDevice::FindCompatibleD3DTextureFormat(fmt,ddfmt))
    {
        InitFromD3DPixelFormat(ddfmt);
    }

    // If not format was found then the Valid() member of this class
    // will fail
}


//-----------------------------------------------------------------------------
PixelFormat::PixelFormat
(
    DDPIXELFORMAT& ddfmt
)
//--------------------------------------
{
    // Pixelformat only currently supports RGB
    Assert(ddfmt.dwFlags & DDPF_RGB);

    InitFromD3DPixelFormat(ddfmt);
}


//-----------------------------------------------------------------------------
bool PixelFormat::InitFromD3DPixelFormat
(
    DDPIXELFORMAT& fmt
)
//--------------------------------------
{
    // Make sure what is passed in is the correct size
    Assert(fmt.dwSize == sizeof(DDPIXELFORMAT));

    //At the moment we only support RGB surfaces
    Assert(fmt.dwFlags & DDPF_RGB);
    m_DDFmt = fmt;

    // Now decode the specified pixel format to get the extra info taht we hold
    // in this class
    m_NumRedBits   = BitCount(fmt.dwRBitMask);
    m_NumGreenBits = BitCount(fmt.dwGBitMask);
    m_NumBlueBits  = BitCount(fmt.dwBBitMask);
    m_NumAlphaBits = BitCount(fmt.dwRGBAlphaBitMask);

    m_RedShift = LowestBit(fmt.dwRBitMask);
    m_GreenShift = LowestBit(fmt.dwGBitMask);
    m_BlueShift = LowestBit(fmt.dwBBitMask);
    m_AlphaShift = LowestBit(fmt.dwRGBAlphaBitMask);

    return true;
}


//-----------------------------------------------------------------------------
uint32 PixelFormat::PixelColorFromRGB
(
    const RGBColor& col
) const
//--------------------------------------
{
    uint32 red;
    uint32 green;
    uint32 blue;
    uint32 alpha;

    // convert floating point to integer with the required number of bits
    red   = (DWORD) (col.r * ((1<<m_NumRedBits)-1));
    green = (DWORD) (col.g * ((1<<m_NumGreenBits)-1));
    blue  = (DWORD) (col.b * ((1<<m_NumBlueBits)-1));
    alpha = (DWORD) (col.a * ((1<<m_NumAlphaBits)-1));

    red   = (red<<m_RedShift) & GetRedMask();
    green = (green<<m_GreenShift) & GetGreenMask();
    blue  = (blue<<m_BlueShift) & GetBlueMask();
    alpha = (alpha<<m_AlphaShift) & GetAlphaMask();

    return red|green|blue|alpha;
}
