//-----------------------------------------------------------------------------
// BiCubicPatch.cpp
//
// Started 06/28/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "BiCubicPatch.h"
#include "ShrikeMath.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------------------
CubicPatch::CubicPatch
(
	Vector3& c00, Vector3& c01, Vector3& c02, Vector3& c03,
	Vector3& c10, Vector3& c11, Vector3& c12, Vector3& c13,
	Vector3& c20, Vector3& c21, Vector3& c22, Vector3& c23,
    Vector3& c30, Vector3& c31, Vector3& c32, Vector3& c33,
    uint32  tess_u,
    uint32  tess_v
) 
: Tessellate2D(tess_u,tess_v)
//--------------------------------------
{
	m_Control[0][0] = c00;
	m_Control[0][1] = c01;
	m_Control[0][2] = c02;
    m_Control[0][3] = c03;

	m_Control[1][0] = c10;
	m_Control[1][1] = c11;
	m_Control[1][2] = c12;
    m_Control[1][3] = c13;

	m_Control[2][0] = c20;
	m_Control[2][1] = c21;
	m_Control[2][2] = c22;
    m_Control[2][3] = c23;

	m_Control[3][0] = c30;
	m_Control[3][1] = c31;
	m_Control[3][2] = c32;
    m_Control[3][3] = c33;
}


//-----------------------------------------------------------------------------
CubicPatch::~CubicPatch
(
)
//--------------------------------------
{
    // Does not do anything...
}


//-----------------------------------------------------------------------------
Vector3 CubicPatch::GeneratePosition
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;

	float       u0,u1,u2,u3;
	float       v0,v1,v2,v3;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    // Calculate the Cubic Bernstien weightings [(1-t), 3t(1-t), 3t(1-t), t] for u and v
	v0 = (1-v)*(1-v)*(1-v);
	v1 = 3*v*(1-v)*(1-v);
	v2 = 3*v*v*(1-v);
	v3 = v*v*v;

	u0 = (1-u)*(1-u)*(1-u);
	u1 = 3*u*(1-u)*(1-u);
	u2 = 3*u*u*(1-u);
	u3 = u*u*u;

    // Calculate the tensor product of the two splines...
	res = ((m_Control[0][0]) * (u0 * v0)) + ((m_Control[0][1]) * (u1 * v0)) + ((m_Control[0][2]) * (u2 * v0)) + ((m_Control[0][3]) * (u3 * v0));
	res +=((m_Control[1][0]) * (u0 * v1)) + ((m_Control[1][1]) * (u1 * v1)) + ((m_Control[1][2]) * (u2 * v1)) + ((m_Control[1][3]) * (u3 * v1));
	res +=((m_Control[2][0]) * (u0 * v2)) + ((m_Control[2][1]) * (u1 * v2)) + ((m_Control[2][2]) * (u2 * v2)) + ((m_Control[2][3]) * (u3 * v2));
    res +=((m_Control[3][0]) * (u0 * v3)) + ((m_Control[3][1]) * (u1 * v3)) + ((m_Control[3][2]) * (u2 * v3)) + ((m_Control[3][3]) * (u3 * v3));

    return res;
}
