//-----------------------------------------------------------------------------
// MeshHelp.cpp
//
// Started 06/24/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "MeshHelp.h"
#include "Debug.h"
#include "3DDevice.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
DiffuseTextureVertex1* TestObject::MakeVertexList
(
    TestObj object,
    uint32& count
)
//-------------------------------------
{
    DiffuseTextureVertex1* mesh = NULL;

    switch (object)
    {
    case TEST_OBJECT_INDEXED_UNIQUE_CUBE:
        // Define a unit cube at the origin with each vertex being replicated 3 times
        // so each face has unique normals. We return an actual copy of the vertex
        // data and not just a pointer, this enables the caller to modify the data
        // and not affect other test models.
        count = 24;
        mesh = new DiffuseTextureVertex1[count];

        mesh[0] = DiffuseTextureVertex1( Vector3(-1.0f, 1.0f,-1.0f), Vector3( 0.0f, 0.0f,-1.0f), RGBColor(1.0f,0.0f,1.0f), 0.0f, 0.0f );
        mesh[1] = DiffuseTextureVertex1( Vector3( 1.0f, 1.0f,-1.0f), Vector3( 0.0f, 0.0f,-1.0f), RGBColor(1.0f,0.0f,1.0f), 1.0f, 0.0f );
        mesh[2] = DiffuseTextureVertex1( Vector3( 1.0f,-1.0f,-1.0f), Vector3( 0.0f, 0.0f,-1.0f), RGBColor(1.0f,0.0f,1.0f), 1.0f, 1.0f );
        mesh[3] = DiffuseTextureVertex1( Vector3(-1.0f,-1.0f,-1.0f), Vector3( 0.0f, 0.0f,-1.0f), RGBColor(1.0f,0.0f,1.0f), 0.0f, 1.0f );

        mesh[4] = DiffuseTextureVertex1( Vector3( 1.0f, 1.0f,-1.0f), Vector3( 1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 0.0f  );
        mesh[5] = DiffuseTextureVertex1( Vector3( 1.0f, 1.0f, 1.0f), Vector3( 1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 0.0f  );
        mesh[6] = DiffuseTextureVertex1( Vector3( 1.0f,-1.0f, 1.0f), Vector3( 1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 1.0f  );
        mesh[7] = DiffuseTextureVertex1( Vector3( 1.0f,-1.0f,-1.0f), Vector3( 1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 1.0f  );

        mesh[8] = DiffuseTextureVertex1( Vector3(-1.0f, 1.0f, 1.0f), Vector3(-1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 0.0f  );
        mesh[9] = DiffuseTextureVertex1( Vector3(-1.0f, 1.0f,-1.0f), Vector3(-1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 0.0f  );
        mesh[10]= DiffuseTextureVertex1( Vector3(-1.0f,-1.0f,-1.0f), Vector3(-1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 1.0f  );
        mesh[11]= DiffuseTextureVertex1( Vector3(-1.0f,-1.0f, 1.0f), Vector3(-1.0f, 0.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 1.0f  );

        mesh[12]= DiffuseTextureVertex1( Vector3( 1.0f, 1.0f, 1.0f), Vector3( 0.0f, 0.0f, 1.0f), RGBColor(0.0f,1.0f,1.0f), 0.0f, 0.0f  );
        mesh[13]= DiffuseTextureVertex1( Vector3(-1.0f, 1.0f, 1.0f), Vector3( 0.0f, 0.0f, 1.0f), RGBColor(0.0f,1.0f,1.0f), 1.0f, 0.0f  );
        mesh[14]= DiffuseTextureVertex1( Vector3(-1.0f,-1.0f, 1.0f), Vector3( 0.0f, 0.0f, 1.0f), RGBColor(0.0f,1.0f,1.0f), 1.0f, 1.0f  );
        mesh[15]= DiffuseTextureVertex1( Vector3( 1.0f,-1.0f, 1.0f), Vector3( 0.0f, 0.0f, 1.0f), RGBColor(0.0f,1.0f,1.0f), 0.0f, 1.0f  );

        mesh[16]= DiffuseTextureVertex1( Vector3( 1.0f, 1.0f, 1.0f), Vector3( 0.0f, 1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 0.0f  );
        mesh[17]= DiffuseTextureVertex1( Vector3( 1.0f, 1.0f,-1.0f), Vector3( 0.0f, 1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 0.0f  );
        mesh[18]= DiffuseTextureVertex1( Vector3(-1.0f, 1.0f,-1.0f), Vector3( 0.0f, 1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 1.0f  );
        mesh[19]= DiffuseTextureVertex1( Vector3(-1.0f, 1.0f, 1.0f), Vector3( 0.0f, 1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 1.0f  );

        mesh[20]= DiffuseTextureVertex1( Vector3( 1.0f,-1.0f,-1.0f), Vector3( 0.0f,-1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 0.0f  );
        mesh[21]= DiffuseTextureVertex1( Vector3( 1.0f,-1.0f, 1.0f), Vector3( 0.0f,-1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 0.0f  );
        mesh[22]= DiffuseTextureVertex1( Vector3(-1.0f,-1.0f, 1.0f), Vector3( 0.0f,-1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 1.0f, 1.0f  );
        mesh[23]= DiffuseTextureVertex1( Vector3(-1.0f,-1.0f,-1.0f), Vector3( 0.0f,-1.0f, 0.0f), RGBColor(1.0f,1.0f,1.0f), 0.0f, 1.0f  );
        break;

    default:
        DBGSTR("ERROR: TestObject::MakeVertexList Unknown test mesh type.\n");
        Assert(0);
    }

    return mesh;
}


//-----------------------------------------------------------------------------
// Build triangle index lists for the test  models, this array indexes the
// data created by the above function  in a per triangle clockwise order.
// Unique triangles are not the most efficient way to draw a mesh but it
// gives us the most flexibility and these are test models.
// We return an actual copy of the tringle list data and not just a pointer,
// this enables the caller to modify the data and not affect other test models.
uint16* TestObject::MakeTriangleListIndices
(
    TestObj object,
    uint32& count
)
//-------------------------------------
{
    uint16* index = NULL;

    switch (object)
    {
    case TEST_OBJECT_INDEXED_UNIQUE_CUBE:
        // Indicies for the unit cube object
        count = 36;
        index = new uint16[count];

        index[0] = 0;    index[1] = 1;    index[2] = 2;     // Each line is a single triangle
        index[3] = 2;    index[4] = 3;    index[5] = 0;
        index[6] = 4;    index[7] = 5;    index[8] = 6;
        index[9] = 6;    index[10]= 7;    index[11]= 4;
        index[12]= 8;    index[13]= 9;    index[14]= 10;
        index[15]= 10;   index[16]= 11;   index[17]= 8;
        index[18]= 12;   index[19]= 13;   index[20]= 14;
        index[21]= 14;   index[22]= 15;   index[23]= 12;
        index[24]= 16;   index[25]= 17;   index[26]= 18;
        index[27]= 18;   index[28]= 19;   index[29]= 16;
        index[30]= 20;   index[31]= 21;   index[32]= 22;
        index[33]= 22;   index[34]= 23;   index[35]= 20;
        break;

    default:
        DBGSTR("ERROR: TestObject::MakeTriangleListIndices Unknown test mesh type.\n");
        Assert(0);
    }

    return index;
}


//-----------------------------------------------------------------------------
IDirect3DVertexBuffer7* MeshHelp::GenerateVertexBuffer
(
    void*   mesh_data,
    uint32  vert_count,
    uint32  vert_size,
    uint32  vert_format,
    bool    write_only
)
//-------------------------------------
{
    IDirect3DVertexBuffer7* vb;
    D3DVERTEXBUFFERDESC     vbdesc;

    ZeroMemory(&vbdesc,sizeof(vbdesc));
    vbdesc.dwSize = sizeof(vbdesc);
    vbdesc.dwCaps = 0;
    
    if (write_only)
        vbdesc.dwCaps |= D3DVBCAPS_WRITEONLY;

    // We are not using a hardware device so create this vertex buffer in
    // system memory
    if (!RenderDevice::UsingHardware())
    {
        vbdesc.dwCaps = D3DVBCAPS_SYSTEMMEMORY;
    }

    vbdesc.dwFVF = vert_format;
    vbdesc.dwNumVertices = vert_count;

    if (RenderDevice::GetD3D()->CreateVertexBuffer(&vbdesc,&vb,0)!=S_OK)
    {
        DBGSTR("ERROR: MeshHelp::GenerateVertexBuffer() CreateVertexBuffer failed.\n");
        return NULL;
    }

    void* vb_data;
    uint32 vb_size;

    if (vb->Lock(DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT|DDLOCK_WRITEONLY|DDLOCK_DISCARDCONTENTS, 
                                                        &vb_data,(unsigned long*)&vb_size) != S_OK)
    {
        DBGSTR("ERROR: MeshHelp::GenerateVertexBuffer() lock vertex buffer failed.\n");
        vb->Release();
        return NULL;
    }

    // Make sure our VB is big enough before we copy data
    Assert(vb_size>=vert_count*vert_size);

    memcpy(vb_data,mesh_data,vert_count*vert_size);

    vb->Unlock();

    return vb;
}
