//-----------------------------------------------------------------------------
// Sphere.cpp
//
// Started 06/28/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "Sphere.h"
#include "ShrikeMath.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
Sphere::Sphere
(
    float   radius,
    uint32  tess_u,
    uint32  tess_v
) 
: Tessellate2D(tess_u,tess_v)
//--------------------------------------
{
    m_Radius = radius;
}


//-----------------------------------------------------------------------------
Vector3 Sphere::GeneratePosition
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  sc_theta;
    SinCosPair  sc_phi;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    SinCos( (v*PI)-PI_DIV_2, sc_theta );        // theta from -PI/2 to PI/2
    SinCos( u*PI_MUL_2, sc_phi );               // phi from 0 to 2PI

    res.x = sc_phi.c * sc_theta.c * m_Radius;
    res.y = sc_phi.s * sc_theta.c * m_Radius;
    res.z = sc_theta.s * m_Radius;

    return res;
}


//-----------------------------------------------------------------------------
Vector3 Sphere::GenerateNormal
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  sc_theta;
    SinCosPair  sc_phi;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    // Normal is a point on the unit sphere so this code is the same
    // as the above code execpt its not scaled.
    SinCos( (v*PI)-PI_DIV_2, sc_theta );        // theta from -PI/2 to PI/2
    SinCos( u*PI_MUL_2, sc_phi );               // phi from 0 to 2PI

    res.x = sc_phi.c * sc_theta.c;
    res.y = sc_phi.s * sc_theta.c;
    res.z = sc_theta.s;

    return res;
}