//-----------------------------------------------------------------------------
// SuperQuadric.cpp
//
// Started 06/24/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "SuperQuadric.h"
#include "ShrikeMath.h"
#include "HlpLib.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------------------
SuperQuadric::SuperQuadric
(
    float   scale_x,
    float   scale_y,
    float   scale_z,
    float   curv_e,
    float   curv_n,
    uint32  tess_u,
    uint32  tess_v
) 
: Tessellate2D(tess_u,tess_v)
//--------------------------------------
{
    m_ScaleX = scale_x;
    m_ScaleY = scale_y;
    m_ScaleZ = scale_z;
    m_CurvE  = curv_e;
    m_CurvN  = curv_n;
}


//-----------------------------------------------------------------------------
SuperQuadric::~SuperQuadric
(
)
//--------------------------------------
{
    // Does not do anything...
}


//-----------------------------------------------------------------------------
Vector3 SuperQuadric::GeneratePosition
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  scu;
    SinCosPair  scv;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    // The range of u is from -PI to PI
    // The range of v is from -PI/2 to PI/2
    SinCos( (u*PI_MUL_2)-PI, scu );
    SinCos( (v*PI)-PI_DIV_2, scv );

    // claculate the position at the given U,V co-ordinate
    res.x = (m_ScaleX*powf(fabsf(scv.c),m_CurvN)*powf(fabsf(scu.c),m_CurvE)*Sgn(scu.c)*Sgn(scv.c));
    res.y = (m_ScaleY*powf(fabsf(scv.c),m_CurvN)*powf(fabsf(scu.s),m_CurvE)*Sgn(scu.s)*Sgn(scv.c));
    res.z = (m_ScaleZ*powf(fabsf(scv.s),m_CurvN)*Sgn(scv.s));

    return res;
}


//-----------------------------------------------------------------------------
Vector3 SuperQuadric::GenerateNormal
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  scu;
    SinCosPair  scv;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    // The range of u is from -PI to PI
    // The range of v is from -PI/2 to PI/2
    SinCos( (u*PI_MUL_2)-PI, scu );
    SinCos( (v*PI)-PI_DIV_2, scv );

    // claculate the normal
    res.x = ((powf(fabsf(scv.c),2-m_CurvN)*powf(fabsf(scu.c),2-m_CurvE)*Sgn(scv.c)*Sgn(scu.c))/m_ScaleX);
    res.y = ((powf(fabsf(scv.c),2-m_CurvN)*powf(fabsf(scu.s),2-m_CurvE)*Sgn(scu.s)*Sgn(scv.c))/m_ScaleY);
    res.z = ((powf(fabsf(scv.s),2-m_CurvN)*Sgn(scv.s))/m_ScaleZ);

    // Normalize normal vector
    res.Normalize();

    return res;
}