//-----------------------------------------------------------------------------
// Torus.cpp
//
// Started 06/24/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "Torus.h"
#include "ShrikeMath.h"

//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
Torus::Torus
(
    float   min_radius,
    float   max_radius,
    uint32  tess_u,
    uint32  tess_v
) 
: Tessellate2D(tess_u,tess_v)
//--------------------------------------
{
    m_MinRadius = min_radius;
    m_MaxRadius = max_radius;
}


//-----------------------------------------------------------------------------
Torus::~Torus
(
)
//--------------------------------------
{
    // Does not do anything...
}


//-----------------------------------------------------------------------------
Vector3 Torus::GeneratePosition
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  sc_theta;
    SinCosPair  sc_phi;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    SinCos( v*PI_MUL_2, sc_theta );
    SinCos( u*PI_MUL_2, sc_phi );

    float h = sc_theta.c*m_MinRadius + m_MaxRadius;

    res.x = h*sc_phi.c;
    res.y = h*sc_phi.s;
    res.z = sc_theta.s*m_MinRadius;

    return res;
}

//-----------------------------------------------------------------------------
Vector3 Torus::GenerateNormal
(
    float   u,
    float   v
)
//--------------------------------------
{
    Vector3     res;
    SinCosPair  sc_theta;
    SinCosPair  sc_phi;

    // Parametric coordinate must be within the range 0.0 to 1.0
    Assert(u>(0.0f-FLOAT_EPSILON));
    Assert(v>(0.0f-FLOAT_EPSILON));
    Assert(u<(1.0f+FLOAT_EPSILON));
    Assert(v<(1.0f+FLOAT_EPSILON));

    SinCos( v*PI_MUL_2, sc_theta );
    SinCos( u*PI_MUL_2, sc_phi );

    res.x = sc_phi.c*sc_theta.c;
    res.y = sc_phi.s*sc_theta.c;
    res.z = sc_theta.s;

    return res;
}

