//-----------------------------------------------------------------------------
// Timer.cpp
//
// Started 06/16/2000
//
// Shrike Framework -  Rob Wyatt
//-----------------------------------------------------------------------------

#include "Timer.h"
#include "Debug.h"


//------------------------------------------------------------------------------
// Standard macro to reroute new allocations to the debug allocator
#if defined(_DEBUG)
#define new DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK,THIS_FILE,__LINE__)
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//------------------------------------------------------------------------------
double Timer::m_TimerFrequency = 0.0f;


//------------------------------------------------------------------------------
Timer::Timer
(
)
//--------------------------------------
{
    //
    // We will use the windows high performance timer for this timer class, we
    // could also use RDTSC.
    //
    int64 frequency;
    QueryPerformanceFrequency((LARGE_INTEGER*)&frequency);
    m_TimerFrequency = (double)frequency;

    // Get the initial time
    QueryPerformanceCounter((LARGE_INTEGER*)&m_TimeStart);

    // We have spent no cycles paused
    m_PauseAdjust = 0;
}


//------------------------------------------------------------------------------
void Timer::Pause
(
    bool request_pause
)
//--------------------------------------
{
    static int64 pause_time;
    static bool paused = false;

    if (request_pause)
    {
        // All ready paused
        if (paused)
            return;

        // We are pausing this timer so get the time now
        QueryPerformanceCounter((LARGE_INTEGER*)&pause_time);
        paused = true;
    }
    else
    {
        // This timer is not paused so return;
        if (!paused)
            return;

        // This timer is being restarted.
        int64 time;

        // We are unpausing this timer so get the time now and calculate how
        // long this timer has been paused for, then use this to adjust all
        // future timings until the class is reset.
        QueryPerformanceCounter((LARGE_INTEGER*)&time);
        m_PauseAdjust += (time - pause_time);

        paused = false;
    }
}