//-----------------------------------------------------------------------------
// TGReflect.h
//
// Started 06/23/2000
//
//  Rob Wyatt 2000 for Game Developer Magazine/Gamasutra
//-----------------------------------------------------------------------------
#if !defined(TGREFLECT_H_INCLUDED)
#define TGREFLECT_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "WinShell.h"
#include "Textures.h"
#include "Matrix.h"
#include "Vector.h"
#include "Vertex.h"
#include "resource.h"


//------------------------------------------------------------------------------
// Define a vertex with a Position, Normal, and 1 3D set of textures coordinates
struct ReflectVertex
{
    Vector3         m_Pos;
    Vector3         m_Normal;

    //------------------------------------------------------------------------------
    enum
    {
        FORMAT = D3DFVF_XYZ|D3DFVF_NORMAL
    };
};



//------------------------------------------------------------------------------
class Reflect : public Shell
{
public:

    //------------------------------------------------------------------------------
    // A menu item has been selected, 'menu_id' is the resouce ID of the selected
    // item.
    virtual bool MenuSelect
    (
        uint32  menu_id
    );


    //------------------------------------------------------------------------------
    // Handle mouse clicks or moves/drags etc
    virtual bool MouseClick
    (
        MouseAction action,
        int32       x,
        int32       y
    );


    //------------------------------------------------------------------------------
    // Called at app startup after the window is created. In this case the 'just_created'
    // variable will always be true because the registry is not being used.
    virtual bool Setup
    (
        bool    just_created
    );


    //------------------------------------------------------------------------------
    // Close down the app, called when the app is exiting. The device has already
    // been closed down but the window handle is still valid.
    bool Reflect::Closedown
    (
    );


    //------------------------------------------------------------------------------
    // Setup the device render states and load and allocate textures, make vertex
    // buffers.
    virtual bool DeviceSetup
    (
        IDirect3DDevice7*   device,
        bool                resize
    );


    //------------------------------------------------------------------------------
    // Release device related items, textures and vertex buffers
    virtual bool DeviceClose
    (
        IDirect3DDevice7*   device
    );


    //------------------------------------------------------------------------------
    // Restore textures and vertex buffers
    virtual bool Restore
    (
        IDirect3DDevice7*   device
    );


    //------------------------------------------------------------------------------
    // Handle frame step, basically rotate the object around its origin.
    virtual bool Step
    (
        IDirect3DDevice7*   device,
        float               frame_time,
        float               frame_step
    );


    //------------------------------------------------------------------------------
    // Handle frame render
    virtual bool Render
    (
        IDirect3DDevice7*   device
    );


    //------------------------------------------------------------------------------
    // Generates a polygon mesh with normals and 3d texture co-ords that are a copy
    // of the normal. It places the mesh in a vertex buffer. m_MeshVB points to this
    // buffer.
    //  0   Torus
    //  1   Quadric Elipsoid
    //  2   Bi Quadratic Patch
    void CreateReflectionMesh
    (
        uint32  model
    );


    //----------------------------------------------------------------------------
    // Called at strtup to set on constant texture mat
    void SetTextureMatrix
    (
        IDirect3DDevice7*   device
    );


    float                    m_MouseX;
    float                    m_MouseY;
    Matrix4                  m_WorldMat;

    uint32                   m_MeshType;
    Texture2D*               m_ReflectionTexture;
    IDirect3DVertexBuffer7*  m_MeshVB;
    uint16*                  m_TriListIndices;
    uint32                   m_VertCount;
    uint32                   m_IndexCount;
};

#endif
