
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CPtrNode
	\ingroup g_data_structures
	\author  Sean O'Neil
	\warning Conversion work in progress
	\date    2000

	This class creates a pointer node that can be used in a pointer list similar
	to MFC's CPtrList. It is useful when you don't want to derive your class from
	CListNode or when you want to keep a list of pointers to items that have been
	allocated elsewhere. It is templatized to keep you from having to cast
	pointers manually, making the code easier to read and maintain.

	Example:
	#define CObjectPtrNode CPtrNode<CObject>
	#define CObjectPtrList CLinkedList< CObjectPtrNode >	// Spaces needed
*/


#ifndef TEDDY_DATA_STRUCTURES_C_PTR_NODE_H
#define TEDDY_DATA_STRUCTURES_C_PTR_NODE_H


#include "DataStructures/CListNode.h"


namespace DataStructures {


#define CVoidPtrNode CPtrNode<void>
#define CVoidPtrList CLinkedList< CVoidPtrNode >

template <class A> class CPtrNode : public CListNode< CPtrNode<A> >{
public:
	CPtrNode()				{ m_pObject = NULL; }
	CPtrNode(A *p)			{ m_pObject = p; }
	void SetObject(A *p)	{ m_pObject = p; }
	A *GetObject()			{ return m_pObject; }
	unsigned long GetID()			{ return (unsigned long)m_pObject; }  // FIX unsigned long

protected:
	A *m_pObject;
};


};  //  namespace DataStructures


#endif  //  TEDDY_DATA_STRUCTURES_C_PTR_NODE_H

