
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CTreeNode
	\ingroup g_data_structures
	\author  Sean O'Neil
	\warning Conversion work in progress
	\date    2000
*/


#ifndef TEDDY_DATA_STRUCTURES_C_TREE_NODE_H
#define TEDDY_DATA_STRUCTURES_C_TREE_NODE_H


namespace DataStructures {


template <class A, int nOrder> class CTreeNode{
public:
	// Constructors/destructors
	CTreeNode(){
		m_pParent = NULL;
		for( register int i=0; i<nOrder; i++ ){
			m_pChild[i] = NULL;
		}
	}
	~CTreeNode()				{ Remove(); }

	// Members to get the various pointers (must use Add/Remove members to set them)
	A *GetParent()				{ return m_pParent; }
	A *GetChild(int i)			{ return m_pChild[i]; }

	// Members to insert into, remove from, and check status in a tree
	bool IsInTree()				{ return (m_pParent != NULL); }
	bool IsLeaf(){
		for( register int i=0; i<nOrder; i++ ){
			if( m_pChild[i] ){
				return false;
			}
		}
		return true;
	}

	void AddChild( int i, A *pNode ){
		ASSERT( !m_pChild[i] && pNode && !pNode->m_pParent );
		m_pChild[i] = pNode;
		pNode->m_pParent = (A *)this;
	}

	void Remove(){
		register int i;
		
		for( i=0; i<nOrder; i++ ){
			delete m_pChild[i];
		}
		if( m_pParent ){
			for( i=0; i<nOrder; i++ ){
				if( m_pParent->m_pChild[i] == (A *)this ){
					m_pParent->m_pChild[i] = NULL;
				}
			}
			m_pParent = NULL;
		}
	}

protected:
	A *m_pParent;			//!< Pointer to parent
	A *m_pChild[nOrder];	//!< Pointer to children
};


};  //  namespace DataStructures


#endif  //  TEDDY_DATA_STRUCTURES_C_TREE_NODE_H

