
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Graphics/Font.h"
#include "Graphics/OpenGL.h"
#include "Maths/trigmath.h"
#include "SysSupport/EndianIn.h"
#include <iostream>
#include <string>
#include <stdlib.h>
using namespace SysSupport;
using namespace std;


namespace Graphics {


/*static*/ Font *Font::default_font = NULL; // Can't initialize here; there is no OpenGL context yet. Done in View constructor.

static float DIV_256 = 1.0/256.0;


/*!
	Font constructor which loads given 256x256 alpha channel raw data file
	and creates map.
*/
Font::Font( const string &fname ){
	EndianIn      *s      = NULL;
	unsigned char *filler = NULL;
	unsigned char *data   = NULL;

	try{
		s = new EndianIn( fname );
		s->set_bigendian();
		filler = data = new unsigned char[256*256];
		for(int i=0; i<256*256;i++){
			*filler++ = s->read_byte();
		}
		
	}catch( ... ){
		//  FIX
		//cout << "Escape" << endl;
	}
	s->close();
	
	glGenTextures  ( 1 , &texture );
	glBindTexture  ( GL_TEXTURE_2D, texture );
	glPixelStorei  ( GL_UNPACK_ALIGNMENT, 1 );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_ALPHA, 256, 256, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data );

	delete[] data;

	unsigned char array[10][21] = {
//		0         1         2
//		012345678901234567890
		"!\"#$%&'()*+,-./01234", // 0 
		"56789:;<=>?@ABCDEFGH",  // 1
		"IJKLMNOPQRSTUVWXYZ[\\", // 2
		"]^_`abcdefghijklmnop",  // 3
		"qrstuvwxyz{|}~ ",  // 4
		"",  // 5
		"",  // 6
		"",  // 7
		"",  // 8
		"??????????"   // 9
	};
/*
		!"#$%&'()*+,-./01234
		56789:;<=>?@ABCDEFGH
		IJKLMNOPQRSTUVWXYZ[\
		]^_`abcdefghijklmnop
		qrstuvwxyz{|}~
		
		
		
		
		??????????
*/
	int row;
	int col;
	int i;

	for( i=0; i<256; i++ ){
		x[i] = 255-8;
		y[i] = 0;
	}

	cw  =  8;
	ch  =  8;
	cwl = cw-1;
	tw  = cw *DIV_256;
	twl = cwl*DIV_256;
	th  = ch *DIV_256;

	for( row=0; row<10; row++ ){
		for( col=0; col<20; col++ ){
			x[ array[row][col] ] = (col*cw)*DIV_256;
			y[ array[row][col] ] = (row*ch)*DIV_256;
		}
	}
}


//!  Render string str to (xp,yp) with this font
void Font::drawString( const string &str, int xp, int yp ){
	const unsigned char *ptr = (const unsigned char *)(str.c_str());
	glBindTexture( GL_TEXTURE_2D, texture );
/*	GLboolean blend = glIsEnabled( GL_BLEND      );
	GLboolean cull  = glIsEnabled( GL_CULL_FACE  );
	GLboolean depth = glIsEnabled( GL_DEPTH_TEST );
	GLboolean light = glIsEnabled( GL_LIGHTING   );
	GLboolean text  = glIsEnabled( GL_TEXTURE_2D );
	GLboolean istxt = glIsTexture( texture );*/
	
	glBegin( GL_QUADS );
	while( *ptr != '\0' ){
		glTexCoord2f( x[*ptr]   , y[*ptr]    ); glVertex2i( xp   , yp    );
		glTexCoord2f( x[*ptr]+tw, y[*ptr]    ); glVertex2i( xp+cw, yp    );
		glTexCoord2f( x[*ptr]+tw, y[*ptr]+th ); glVertex2i( xp+cw, yp+ch );
		glTexCoord2f( x[*ptr]   , y[*ptr]+th ); glVertex2i( xp   , yp+ch );
		xp += cw;
		ptr++;
	}
	glEnd();
}

//!  Return font width in pixels
int Font::getWidth(){
	return cw;
}


//!  Return font height in pixels
int Font::getHeight(){
	return ch;
}


};  //  namespace Graphics


