
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Graphics/GlFeature.h"


namespace Graphics {


/*static*/ GlFeature GlFeature::blend        = GlFeature( "blend",          GL_BLEND        );
/*static*/ GlFeature GlFeature::cull_face    = GlFeature( "cull face",      GL_CULL_FACE    );
/*static*/ GlFeature GlFeature::depth_test   = GlFeature( "depth test",     GL_DEPTH_TEST   );
/*static*/ GlFeature GlFeature::fog          = GlFeature( "fog",            GL_FOG          );
/*static*/ GlFeature GlFeature::lighting     = GlFeature( "lighting",       GL_LIGHTING     );
/*static*/ GlFeature GlFeature::line_smooth  = GlFeature( "line smooth",    GL_LINE_SMOOTH  );
/*static*/ GlFeature GlFeature::normalize    = GlFeature( "normalize",      GL_NORMALIZE    );
/*static*/ GlFeature GlFeature::texture_2d   = GlFeature( "texture 2d",     GL_TEXTURE_2D   );


//!  Constructor. By default all features all disabled.
GlFeature::GlFeature( const string &name, GLint gl_code ){
	this->name         = name;
	this->gl_code      = gl_code;
	this->master_state = false;
}


//!  Set feature state
void GlFeature::setMasterState( bool master_state ){
	this->master_state = master_state;
}


//!  Apply feature
void GlFeature::apply( bool client ){
	if( master_state==true ){
		if( client==true ){
			glEnable( gl_code );
		}else{
			glDisable( gl_code );
		}
	}
}


//!  Enable feature
void GlFeature::masterEnable(){
	master_state = true;
}


//!  Disable feature
void GlFeature::masterDisable(){
	master_state = false;
	glDisable( gl_code );
}


//!  Query master state
bool GlFeature::isMasterEnabled(){
	return master_state;
}


};  //  namespace Graphics

