
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   GlFeature
	\ingroup g_graphics
	\author  Timo Suoranta
	\brief   Wrapper for OpenGL context features that can be enabled and disabled
	\todo    OpenGL context could be separate base class for this class
	\date    1999, 2000
*/


#ifndef TEDDY_GRAPHICS_GL_FEATURE_H
#define TEDDY_GRAPHICS_GL_FEATURE_H


#include "Graphics/OpenGL.h"
#include <string>
using namespace std;


namespace Graphics {


class GlFeature {
public:
	GlFeature( const string &name, GLint gl_code );

	void apply          ( bool client );
	void setMasterState ( bool master_state );
	void masterEnable   ();
	void masterDisable  ();
	bool isMasterEnabled();

	static GlFeature blend;
	static GlFeature cull_face;
	static GlFeature depth_test;
	static GlFeature fog;
	static GlFeature lighting;
	static GlFeature line_smooth;
	static GlFeature normalize;
	static GlFeature texture_2d;

protected:
	string name;
	GLenum gl_code;
	bool   state;
	bool   master_state;
};


};  //  namespace Graphics


#endif  //  TEDDY_GRAPHICS_GL_FEATURE_H


