
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Graphics/Renderable.h"
#include "Graphics/Renderer.h"
#include "SysSupport/Exception.h"
#include <iostream>
using namespace std;


namespace Graphics {


//!  Constructor
Renderable::Renderable(){
}


//!  Destructor
Renderable::~Renderable(){
	RenderContext_to_Renderer::iterator r_it = renderers.begin();
	while( r_it != renderers.end() ){
		delete (*r_it).second;
		r_it++;
	}
}


//!  Begin Renderer OpenGL displaylist update
/*virtual*/ void Renderable::beginUpdate( RenderContext *rc ){
	getRenderer( rc )->beginUpdate();
}


//!  End Renderer OpenGL displaylist update
/*virtual*/ void Renderable::endUpdate( RenderContext *rc ){
	getRenderer( rc )->endUpdate();
}


//!  Drawing a renderable to specific render context
/*virtual*/ void Renderable::draw( RenderContext *rc ){
/*	cout << "Renderable::draw()" << endl;
	cout.flush();*/
	Renderer *r = getRenderer( rc );
	if( r!=NULL ){
		r->draw();
	}
}


//!  Find out which renderer does rendering for specific render context
Renderer *Renderable::getRenderer( RenderContext *rc ){
	if( rc==NULL ){
		cerr << "Renderable::getRenderer() NULL RenderContext!" << endl;
		cerr.flush();
		return NULL;
	}

	Renderer *r = NULL;
	RenderContext_to_Renderer::iterator r_it = renderers.find( rc );
	if( r_it!=renderers.end() ){
		r = (*r_it).second;
		if( r == NULL ){
			cerr << "Renderable::getRenderer() Found NULL Renderer" << endl;
			cerr.flush();
			return NULL;
		}
	}else{
		r = new Renderer( this, rc );
		renderers.insert( pair<RenderContext*,Renderer*>(rc,r) );
	}
	return r;
}


};  //  namespace Graphics

