
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Graphics/Renderable.h"
#include "Graphics/Renderer.h"
#include "SysSupport/Exception.h"
#include <iostream>
using namespace std;


namespace Graphics {


//!  Constructor
Renderer::Renderer( Renderable *renderable, RenderContext *render_context ){
	this->renderable     = renderable;
	this->render_context = render_context;
	this->gldl           = 0;
}


//!  Destructor
/*virtual*/ Renderer::~Renderer(){
	if( gldl!= 0 ){
		glDeleteLists( gldl, 1 );
	}
}


//!  Begin Renderer OpenGL displaylist update
/*virtual*/ void Renderer::beginUpdate(){
	if( gldl==0 ){
		gldl = glGenLists( 1 );
		if( gldl==0 ){
	        throw( Exception( "OGL could not create displaylist") );
		}
	}
	glNewList( gldl, GL_COMPILE );
}


//!  Update renderer
/*virtual*/ void Renderer::update(){
	renderable->update( render_context );
}


//!  End Renderer OpenGL displaylist update
/*virtual*/ void Renderer::endUpdate(){
	glEndList();
}


//!  Default Renderer draw
void Renderer::draw(){
	if( gldl==0 ){
		//  If displaylist seems to be uninitialized, request
		//  renderble to update it.
		cerr << "No displaylist!" << endl;
		cerr.flush();
		renderable->update( render_context );
		if( gldl!=0 ){  //  Still might have gone wrong
			glCallList( gldl );
		}
	}else{
		glCallList( gldl );
	}
}


//!  Get RenderContext
RenderContext *Renderer::getRenderContext() const {
	return this->render_context;
}


//!  Set RenderContext
void Renderer::setRenderContext( RenderContext *rc ){
	this->render_context = rc;
}


//!  Get Renderable
Renderable *Renderer::getRenderable() const {
	return this->renderable;
}


//!  Set Renderable
void Renderer::setRenderable( Renderable *r ){
	this->renderable = r;
}


};  //  namespace Graphics

