
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   View
	\ingroup g_graphics
	\author  Timo Suoranta
	\brief   OpenGL drawing context, implemented as SDL window
	\warning Will change a lot
	\date    1999, 2000
*/


#ifndef TEDDY_GRAPHICS_VIEW_H
#define TEDDY_GRAPHICS_VIEW_H


namespace PhysicalComponents {
	class Layer;
};
#include "Graphics/OpenGL.h"
#include "Graphics/RenderContext.h"
#include <list>
#include <stack>
using namespace PhysicalComponents;
using namespace std;


namespace Graphics {


class View : public RenderContext {
public:
	View( char *title, bool fullscreen );

	void   insert     ( Layer *layer );
	GLint *getViewport();
	GLint  getHeight  ();
	float  getRatio   ();
	void   debug      ();
	void   begin2d    ();
	void   end2d      ();
	void   begin3d    ();
	void   end3d      ();

	void   reshape ( int w, int h );
	void   display ();

	void   setBlendFunc   ( const    GLenum sfactor, const GLenum dfactor );
	void   setClearDepth  ( const    GLfloat depth );
	void   setFogMode     ( const    GLenum mode );
	void   setFogColor    ( GLfloat *color );
	void   setFogStart    ( const    GLfloat start );
	void   setFogEnd      ( const    GLfloat end );
	void   setPolygonMode ( const    GLenum polygon_mode );
	void   setShademodel  ( const    GLenum shade_model );
	void   setFrontFace   ( const    GLenum front_face );

	static void check ();

	int    frames;

protected:				   
	list<Layer*> layers;

	double ratio;
	int    width;
	int    height;
	GLint  viewport[4];

	static View *active;

	static void leave     ();
	static void displayOne();
	static void reshapeOne( int w, int h );

	//  GL context wrapper
	GLenum  gl_polygon_mode;
	GLenum  gl_shade_model;
	GLenum  gl_blend_func_sfactor;
	GLenum  gl_blend_func_dfactor;
	GLenum  gl_fog_mode;
	GLfloat gl_fog_color[3];
	GLfloat gl_fog_start;
	GLfloat gl_fog_end;
	GLfloat gl_clear_depth;
};


};  //  namespace Graphics


extern int SCREEN_X;
extern int SCREEN_Y;


#endif  //  TEDDY_GRAPHICS_VIEW_H

