
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Hud.h"
#include "ui.h"

#include "Graphics/View.h"
#include "Maths/trigmath.h"
#include "Maths/Vector.h"
#include "Models/CSphere.h"
#include "Models/ModelInstance.h"
#include "Physics/System.h"
#include "Physics/RigidBody.h"
#include "PhysicalComponents/GradientFill.h"
#include "PhysicalComponents/LayoutConstraint.h"
#include "PhysicalComponents/Style.h"
#include "PhysicalComponents/WindowFrame.h"
#include "Scenes/Camera.h"
#include "Scenes/Scene.h"
#include <stdio.h>
#include <string>
using namespace Graphics;
using namespace Models;
using namespace Physics;
using namespace PhysicalComponents;
using namespace Scenes;
using namespace std;


//!  Constructor
Hud::Hud( Camera *camera, System *system ):Area(){
	this->camera = camera;
	this->system = system;
	this->target = NULL;
	constraint = new LayoutConstraint();
	constraint->local_x_offset_pixels =  4;
	constraint->local_y_offset_pixels =  20;
	constraint->local_x_fill_pixels   = SCREEN_X-8;
	constraint->local_y_fill_pixels   =  20;

#ifdef USE_WINDOW_SYSTEM	
	this->insert(
		new PhysicalComponents::GradientFill(  
			Color( 0.7f, 0.8f, 1.0f, 1.0f ),
			Color( 0.7f, 0.8f, 1.0f, 1.0f ),
			Color( 0.7f, 0.8f, 1.0f, 1.0f ),
			Color( 0.7f, 0.8f, 1.0f, 1.0f )
		)
	);

	this->insert(
		new WindowFrame( "Heads Up Display" )
	);
#endif

	ordering = post_self;

}


//!  Set HUD target
void Hud::setTarget( ModelInstance *target ){
	this->target = target;
}


extern double speed;


//!  Drawing code - FIX, ugly code...
void Hud::drawSelf(){
	char         hud[100];
	const char  *name;
	char        *unit;
	double       distance;
	string       a;

	glPushAttrib( GL_ALL_ATTRIB_BITS );
#ifdef USE_WINDOW_SYSTEM
	glColor4f   ( 0.0f, 0.0f, 0.0f, 1.0f );
#else
	glColor4f   ( 0.5f, 1.0f, 0.5f, 1.0f );
#endif
	glEnable    ( GL_BLEND );
	glEnable    ( GL_TEXTURE_2D );
	glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	Vector   cpos    = camera->getLoc();
	double   radius  = 0.0;
	double   angle   = 0.0;
	double   kal_h   = 0.0;

	if( target!=NULL ){
		name     = target->getMesh()->getName().c_str();
		distance = target->distanceTo( camera->getLoc() );
		if( distance < 1000.0 ){
			unit = "m";
		}else{
			unit = "km";
			distance *= 0.001;
		}
	}else{
		name     = "-";
		distance = 0.0;
		unit     = "m";
	}

/*
	static float last_time = 0.0;	
	float time = system->getTime();
	float age = time - last_time;
	float ag = system->getFrameAge();
	if( age > 1000 ){
		fps = getView()->frames * 1000 / age;
		getView()->frames = 0;
		last_time = time;
	}
	*/
	Vector clook = camera->getLook();
	sprintf(
		hud,
		TITLE_STRING " %3.1f ms %3.1f fps %d culled Target: %s %.1f %s",
		0.0f,//RenderThread::last_frame,
		0.0f,//RenderThread::fps,
		Scene::culled,
		name,
		distance,
		unit
	);

	a = hud;
	drawString( style->small_font, a, 5.0, 25 );

	glPopAttrib();
}

