
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   LWEnvelope
	\ingroup g_imports
	\author  Timo Suoranta
	\brief   Parser for LightWave object file envelope subchunks
	\warning Very incomplete
	\bug     Destructors missing?
	\date    2000

	Only for Lighwave files from version 6.0 and up
*/


#ifndef TEDDY_IMPORTS_LW_ENVELOPE_H
#define TEDDY_IMPORTS_LW_ENVELOPE_H


#include "Imports/LWFile.h"
#include "Imports/LWEnvelopeKey.h"
#include <list>
using namespace std;


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif


namespace Imports {


class LWEnvelope {
public:
	LWEnvelope( LWFile *f );

	void processEnvelope();

protected:
	void readPreBehaviour_U2    ();
	void readPostBehaviour_U2   ();
	void readKeyframe_F4_F4     ();
	void readInterpolation_ID4_d();
	void readChannel_S0_U2_d    ();
	void readName_S0            ();

	string                name;
	U2                    pre_behaviour;
	U2                    post_behaviour;
	list<LWEnvelopeKey*>  keys;
	LWEnvelopeKey        *last_key;

	LWFile *f;
	ID4     envelope_type;
	U4      envelope_length;
};


};  //  namespace Imports


#endif  //  TEDDY_IMPORTS_LW_ENVELOPE_H

