
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   LWFile
	\ingroup g_imports
	\author  Timo Suoranta
	\brief   Parses lightwave types from file
	\warning Very incomplete
	\date    2000

	Parses lightwave types from file stream.
	Chunks have length guardian, such that attempting
	to read past chunk should not mess up.
*/


#ifndef TEDDY_IMPORTS_LW_FILE_H
#define TEDDY_IMPORTS_LW_FILE_H


#include "Imports/lwdef.h"
#include "SysSupport/EndianIn.h"
#include <string>
using namespace SysSupport;
using namespace std;
	


namespace Imports {


#define DOMAIN_STACK_SIZE 10


extern void did( const unsigned long id4 );

class LWFile {
public:
	LWFile( const string &fname );
	~LWFile();

	void  open       ( const string &fname );
	void  close      ();

	void  pushDomain ( unsigned long len );	      //!<  Push new domain length
	U4    popDomain  ( bool skip_rest = false );  //!<  Pop domain, and skip rest of it if requested
	U4    domainLeft ();
	U4    bytesRead  ();

	ID4   read_ID4   ();
	I1    read_I1    ();
	I2    read_I2    ();
	I4    read_I4    ();
	U1    read_U1    ();
	U2    read_U2    ();
	U4    read_U4    ();
	F4    read_F4    ();
	S0    read_S0    ();
	VX    read_VX    ();
	COL12 read_COL12 ();
	VEC12 read_VEC12 ();
	FP4   read_FP4   ();
	ANG4  read_ANG4  ();
	FNAM0 read_FNAM0 ();

protected:
	EndianIn      *is;
	unsigned long  domain_end;
	unsigned long  bytes_read;

	unsigned long  d_ends[DOMAIN_STACK_SIZE];  //!<  Domain guardian stack
	unsigned long *dsp;                        //!<  Domain stack pointer
};


};  //  namespace Imports


#endif  //  TEDDY_IMPORTS_LW_FILE_H


