
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   LWSurfaceBlok
	\ingroup g_imports
	\author  Timo Suoranta
	\brief   Parser for LightWave object file surface blok subchunks
	\warning Very incomplete
	\bug     Destructors missing?
	\date    2000

	A surface may contain any number of 'blocks' which hold texture layers or
	shaders. The blocks are defined by a sub-chunk with the following format. 

	BLOK { 
		type { ordinal[ID4], header[SUB-CHUNK] * },
			attributes[SUB-CHUNK] * 
	} 

	Immediately inside the block sub-chunk is another sub-chunk which is the
	header for the block. The type code for this header block defines the type
	of the block. This header block contains an ordinal string followed by any
	number and type of header sub-chunks. Following the required header sub-chunk,
	sub-chunks which contain the attributes of this block are found in any order.
	The chunks here will depend on the overall type of the block. 

	Ordinary string compare functions are used to sort the ordinal strings. These
	are never read by the user, so they can contain any characters from 1 to 255
	and tend to be very short. The only rule is that an ordinal string must not
	end with a 1 byte, since that prevents arbitrary insertion. 
*/


#ifndef TEDDY_IMPORTS_LW_SURFACE_BLOK_H
#define TEDDY_IMPORTS_LW_SURFACE_BLOK_H


#include "Imports/LWFile.h"
#include <list>
using namespace std;


namespace Imports {


class LWSurfaceBlok {
public:
	LWSurfaceBlok( LWFile *f );
	
	void processBlok();

protected:
	LWFile *f;
	ID4     blok_type;
	U4      blok_length;
};


};  //  namespace Imports


#endif  //  TEDDY_IMPORTS_LW_SURFACE_BLOK_H

