
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Imports/LWTexture.h"


namespace Imports {


LWTexture::LWTexture( LWFile *f, ID4 file_type ){
	this->f         = f;
	this->file_type = file_type;
}


void LWTexture::processTexture(){
	texture_type   = f->read_ID4();
	texture_length = f->read_U2();

	f->pushDomain( texture_length );

	switch( texture_type ){
		case ID_TFLG: readTextureFlags_U2(); break;
		case ID_TSIZ: readTextureSize_VEC12(); break;
		case ID_TCTR: readTextureCenter_VEC12(); break;
		case ID_TFAL: readTextureFallOff_VEC12(); break;
		case ID_TVEL: readTextureVelocity_VEC12(); break;
		case ID_TREF: readTextureReferenceObject_S0(); break;
		case ID_TCLR: readTextureColor_COL4(); break;
		case ID_TVAL: readTextureValue_IP2(); break;
		case ID_TAMP: readBumpTextureAmplitude_FP4(); break;
		case ID_TFP : readTextureAlgorithm_F4(); break;  // ns not yet handled
		case ID_TIP : readTextureAlgorithm_I2(); break;
		case ID_TSP : readTextureAlgorithm_F4(); break;  // obsolete
		case ID_TFRQ: readTextureAlgorithm_I2(); break;  // obsolete
		case ID_TIMG: readImageMap_FNAM0(); break;
		case ID_TALP: readImageAlpha_FNAM0(); break;
		case ID_TWRP: readImageWarpOptions_U2_U2(); break;
		case ID_TAAS: readAntialiasingStrength_FP4(); break;
		case ID_TOPC: readTextureOpacity_FP4(); break;
		default: break;
	}

	f->popDomain( true );

}

void LWTexture::readTextureFlags_U2(){
	int count = 0;
	U2  flags = f->read_U2();	

/*	if( flags & LW_TF_AXIS_X ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "X axis";
	}
	if( flags & LW_TF_AXIS_Y ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "Y axis";
	}
	if( flags & LW_TF_AXIS_Z ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "Z axis";
	}
	if( flags & LW_TF_WORLD_COORDINATES ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "world coordinates";
	}
	if( flags & LW_TF_NEGATIVE_IMAGE ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "negative image";
	}
	if( flags & LW_TF_PIXEL_BLENDING ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "pixel blending";
	}
	if( flags & LW_TF_ANTIALISING ){
		if(count++){cout<<", ";}else{cout<<"Texture flags ";}
		cout << "antialising";
	}

	if( count>0 ){
		cout << endl;
		cout.flush();
	}*/
}

void LWTexture::readTextureSize_VEC12(){
}

void LWTexture::readTextureCenter_VEC12(){
}

void LWTexture::readTextureFallOff_VEC12(){
}

void LWTexture::readTextureVelocity_VEC12(){
}

void LWTexture::readTextureReferenceObject_S0(){
	string reference_object = f->read_S0();

/*	cout << "Texture reference object " << reference_object << endl;
	cout.flush();*/
}

void LWTexture::readTextureColor_COL4(){
}

void LWTexture::readTextureValue_IP2(){
}

void LWTexture::readBumpTextureAmplitude_FP4(){
}

void LWTexture::readTextureAlgorithm_F4(){
}

void LWTexture::readTextureAlgorithm_I2(){
}

void LWTexture::readImageMap_FNAM0(){
	string image_map = f->read_FNAM0();
/*	cout << "Texture image map " << image_map << endl;
	cout.flush();*/
}

void LWTexture::readImageAlpha_FNAM0(){
	string image_alpha = f->read_FNAM0();
	cout << "Texture image alpha " << image_alpha << endl;
	cout.flush();
}

void LWTexture::readImageWarpOptions_U2_U2(){
}

void LWTexture::readAntialiasingStrength_FP4(){
}

void LWTexture::readTextureOpacity_FP4(){
}


};  //  namespace Imports


