
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Inputs/FocusManager.h"


namespace Inputs {


bool           FocusManager::skip_warp   = false;
KeyListener   *FocusManager::key_focus   = NULL;
MouseListener *FocusManager::mouse_focus = NULL;
list<Area*>    FocusManager::areas;


//!  Received key message from device (yeah right - devices not yet created..)
/*virtual*/ void FocusManager::keyDown( SDL_keysym key ){
	if( key_focus != NULL ){
		key_focus->keyDown( key );
	}
};


//!  Received key message from device
/*virtual*/ void FocusManager::keyUp( SDL_keysym key ){
	if( key_focus != NULL ){
		key_focus->keyUp( key );
	}
};


//!  MouseListener interface
/*virtual*/ void FocusManager::mouseKey( int button, int state, int x, int y ){
	list<Area*>::iterator a_it = areas.begin();

	//  Search for all areas
	while( a_it != areas.end() ){
		int x1;
		int y1;
		int w;
		int h;

		//  Which is area of this Area?
		(*a_it)->getPos ( &x1, &y1 );
		(*a_it)->getSize( &w, &h );

		//  Match with coordinates?
		if( (x >= x1)   &&
		    (x <= x1+w) &&
			(y >= y1)   &&
			(y <= y1+h)    ){

			//  Is this area Listener?
			KeyListener   *key_try   = dynamic_cast<KeyListener*  >(*a_it);
			MouseListener *mouse_try = dynamic_cast<MouseListener*>(*a_it);

			//  KeyListener - activate it, if not already active
			if( key_try != NULL ){
				if( key_focus != key_try ){
					key_focus->focusActive( false );
					key_focus = key_try;
					key_focus->focusActive( true );
				}
				//  FIX It is unclear - should use style or something - if
				//  Area wants to receive the event that activated it.
			}

			//  MouseListener - activate it, if not already active
			if( mouse_try != NULL ){
				if( mouse_focus != mouse_try ){
					mouse_focus->focusActive( false );
					mouse_focus = mouse_try;
					mouse_focus->focusActive( true );
				}
				//  FIX same as with keylistener
				mouse_focus->mouseKey( button, state, x, y );
			}
			//break;
		}
		a_it++;
	}
}


//  MouseListener interface
/*virtual*/ void FocusManager::mouseMotion( int x, int y ){
	if( skip_warp ){
		skip_warp = false;
		return;
	}

	if( mouse_focus != NULL ){
		mouse_focus->mouseMotion( x, y );
	}
}


//  FocusManager interface
void FocusManager::registerArea( Area *area ){
	areas.push_back( area );
}


//!  Set focus
void FocusManager::setFocus( KeyListener *key_focus, MouseListener *mouse_focus ){
	if( key_focus!=NULL ){
		FocusManager::key_focus = key_focus;
	}
	if( mouse_focus!=NULL ){
		FocusManager::mouse_focus = mouse_focus;
	}
}


};  //  namespace Inputs

