
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   FocusManager
	\ingroup g_inputs
	\author  Timo Suoranta
	\date    2000

	Input sends events to FocusManager, which further passes events
	to currently active focus, which must implement the Listener
	interface. It is FocusManager's role to choose the active focus.
	
	FocusManager filters input so that it can determine the active
	focus, and finally passes remaining messages to the active focus.
	This scheme may change later.

	Although FocusManager has key_focus and mouse_focus in protected
	part, that will change later; listener will be dynamic such that
	when new input device is plugged in - or virtual device is used
	(split keyboard for example) they will have individual focuses.

	Areas that are interactive must use registerArea() method to make
	themselves known to FocusManager.

*/


#ifndef TEDDY_INPUTS_FOCUS_MANAGER_H
#define TEDDY_INPUTS_FOCUS_MANAGER_H


#include "PhysicalComponents/Area.h"
#include "Inputs/KeyListener.h"
#include "Inputs/MouseListener.h"
#include <list>
using namespace PhysicalComponents;
using namespace std;


namespace Inputs {


class FocusManager {
public:
	//  FocusManager interface
	static void registerArea( Area *area );
	static void setFocus    ( KeyListener *key_focus, MouseListener *mouse_focus );

	//  Callback functions
	static void keyDown    ( SDL_keysym key );
	static void keyUp      ( SDL_keysym key );
	static void mouseKey   ( int button, int state, int x, int y );
	static void mouseMotion( int x, int y );

	static bool skip_warp;

protected:
	static KeyListener   *key_focus;
	static MouseListener *mouse_focus;
	static list<Area*>    areas;

private:
	FocusManager();
};


};  //  namespace Inputs


#endif  //  TEDDY_INPUTS_FOCUS_MANAGER_H


