
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Inputs/Input.h"
#include "Inputs/KeyListener.h"
#include "Inputs/MouseListener.h"
#include "Inputs/FocusManager.h"

#include "Graphics/View.h"
#include "SDL.h"

using namespace Graphics;


namespace Inputs {


//!  Constructor
Input::Input( View *view ){
	this->view = view;
}

//!  Never returning input loop forwarding host system messages to application
void Input::inputLoop(){
    for( ; ; ){
		SDL_Event event;

		// SDL_Delay( 1 );  FIX some kind of delay is needed if going faster than 100 fps
		view->display();

		while( SDL_PollEvent(&event) ) {
			switch( event.type ){

			case SDL_QUIT:
				exit( 0 );
				break;

			case SDL_KEYDOWN:
				FocusManager::keyDown( event.key.keysym );
				break;

			case SDL_KEYUP:
				FocusManager::keyUp( event.key.keysym );
				break;

			case SDL_MOUSEMOTION:
				FocusManager::mouseMotion( event.motion.x, event.motion.y );
				break;

			case SDL_MOUSEBUTTONDOWN:
			case SDL_MOUSEBUTTONUP:
				FocusManager::mouseKey( event.button.button, event.button.state, event.button.x, event.button.y );
				break;

			case SDL_VIDEORESIZE:
				//  This messes OpenGL contexts? at least may need to reupload textures and update displaylists etc?
			    if( SDL_SetVideoMode(event.resize.w, event.resize.h, 0, SDL_OPENGL|SDL_RESIZABLE/*|SDL_FULLSCREEN*/) == NULL ) {
			        fprintf( stderr, "Unable to resize OpenGL screen: %s\n", SDL_GetError() );
				    exit( 2 );
				}
				break;
			}
		}
	}
}


};  //  namespace Inputs;

