
#include "Kamin/Lisp.h"
#include "Kamin/IntegerExpression.h"
#include "Kamin/List.h"
#include "Kamin/ListNode.h"
#include "Kamin/Function.h"
#include "Kamin/Symbol.h"
#include <stdio.h>
#include "ui.h"  //  FIX


namespace Kamin {


//
//	the Arithmetic functions
//

int PlusFunction( int a, int b ){
	return a + b;
}


int MinusFunction( int a, int b ){
	return a - b;
}


int TimesFunction( int a, int b ){
	return a * b;
}


int DivideFunction( int a, int b ){
	if( b != 0 ){
		return a / b;
	}
	error("division by zero");
	return 0;
}


//
//	Relational functions
//

void EqualFunction( Expr & target, Expression *one, Expression *two ){

	// true if both numbers and same number
	IntegerExpression * ione = one->isInteger();
	IntegerExpression * itwo = two->isInteger();
	if( one && itwo && (ione->val() == itwo->val()) ){
//		target = True();  FIX
		return;
	}

	// or both symbols and same symbol
	Symbol * sone = one->isSymbol();
	Symbol * stwo = two->isSymbol();
	if( sone && stwo && (*sone == stwo) ){
//		target = True();  FIX
		return;
	}

	// or both lists and both nil
	ListNode * lone = one->isList();
	ListNode * ltwo = two->isList();
	if( lone && ltwo && lone->isNil() && ltwo->isNil() ){
//		target = True();  FIX
		return;
	}

	// false otherwise
//	target = False();  FIX
}


int IntEqualFunction( int a, int b ){
	return a == b;
}


int LessThanFunction( int a, int b ){
	return a < b;
}


int GreaterThanFunction( int a, int b ){
	return a > b;
}


//
//	Car and Cdr
//

void CarFunction( Expr & target, Expression *arg ){
	ListNode * thelist = arg->isList();
	if( !thelist ){
		target = error("car applied to non list");
		return;
	}
	target = thelist->head()->touch();
}


void CdrFunction( Expr &target, Expression *arg ){
	ListNode * thelist = arg->isList();
	if( !thelist ){
		target = error("car applied to non list");
		return;
	}
	target = thelist->tail()->touch();
}


void ConsFunction( Expr &target, Expression *left, Expression *right ){
	target = new ListNode( left, right );
}


int NumberpFunction(Expression *arg ){
	return 0 != arg->isInteger();
}

int SymbolpFunction( Expression *arg ){
	return 0 != arg->isSymbol();
}

int ListpFunction( Expression *arg ){
	ListNode *x = arg->isList();

	// list? doesn't return true on nil
	if( x && x->isNil() ){
		return 0;
	}
	if( x ){
		return 1;
	}
	return 0;
}

int NullpFunction( Expression *arg ){
	ListNode *x = arg->isList();
	return x && x->isNil();
}

int PrimoppFunction( Expression *arg ){
	Function *funValue = arg->isFunction();

	if( funValue ){  // if not closure then primitive
		if( funValue->isClosure() ){
			return 0;
		}
		return 1;
	}
	return 0;
}

int ClosurepFunction( Expression *arg ){
	Function *funValue = arg->isFunction();
	if( funValue ){
		if( funValue->isClosure() ){
			return 1;
		}
	}
	return 0;
}

void PrintFunction(Expr & target, Expression * arg){	
	target = arg; 
	if( target() ){
		target()->print();
	}
	printf("\n");
	console->newLine();
}


};  //  namespace Kamin

