
#include "Kamin/ListNode.h"
#include "Kamin/Function.h"
#include "Kamin/Environment.h"
#include <stdio.h>
#include <stdlib.h>
#include "ui.h"  //  FIX


namespace Kamin {


ListNode::ListNode( Expression *car, Expression *cdr ){
	h = car;
	t = cdr;
}


void ListNode::free(){
	h = 0;
	t = 0;
}


int ListNode::isNil(){
	// we are nil if we have no value and no rest of list
	return h() == 0 && t() == 0;
}


int ListNode::length(){
	if( isNil() ){
		return 0;
	}
	
	ListNode *cd = tail();
	
	if( cd ){
		return 1 + cd->length();
	}
	return 0;
}


Expression * ListNode::at( int index ){
	if( index <= 0 ){
		return head();
	}

	ListNode *cd = tail();

	if( cd ){
		return cd->at(index - 1);
	}

	return error( "impossible case", "index to list at: illegal" );
}


ListNode *ListNode::tail(){
	if( !t() ){
		error( "impossible case", "tail on empty list??" );
		exit( 1 );  //  FIX
	}

	ListNode *x = t()->isList();

	if( !x ){
		error( "impossible case", "tail can't find list" );
	}
	return x;
}


/*virtual*/ void ListNode::eval( Expr &target, Environment *valueops, Environment *rho, Base *b ){

	//  An empty list evaluates to Nil
	if( isNil() ){
		target = this;
		return;
	}

	//  If first argument is a symbol, see if it is a command
	Expression *firstarg = head();
	Expression *fun      = 0;
	Symbol     *name     = firstarg->isSymbol();

	if( name ){
		fun = valueops->lookup( name );
	}

	//  Otherwise evaluate it in the given environment
	if( !fun ){
		firstarg->eval( target, valueops, rho, b );
		fun = target();
	}

	//  Now see if it is a function
	Function *theFun = 0;

	if( fun ){
		theFun = fun->isFunction();
	}

	if( theFun ){ 
		theFun->apply( target, tail(), rho, b );
	}else{
		target = error( "evaluation of unknown function" );
		// print();	// print out what we know
		return;
	}
}


void ListNode::print(){
	*console << "(";
	printf( "(" );

	if( ! isNil() ){
		//  Not a nil list, print elements
		h()->print();
		Expression *cd = t();

		while( cd ){
			ListNode *cdl = cd->isList();

			if( !cdl ){
				*console << " ";
				printf(" "); cd->print();
				break;
			}

			if( cdl->isNil() ){
				break;
			}

			*console << " ";
			printf( " " );
			cdl->head()->print();
			cd = cdl->tail();	
		}
	}
	*console << ")";
	printf( ")" );
}


ListNode *ListNode::isList(){
	return this;
}


};  //  namespace Kamin

