
#include "Kamin/Expression.h"
#include "Kamin/List.h"
#include "Kamin/Reader.h"
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "ui.h"  //  FIX


namespace Kamin {


void ReaderClass::printPrimaryPrompt(){
	*console << "-> ";
	/*printf( "\n-> " );
	fflush( stdout );*/
}


void ReaderClass::printSecondaryPrompt(){
	*console << "> ";
	/*printf( "> " );
	fflush( stdout );*/
}


void ReaderClass::fillInputBuffer(){
	//  If the user indicates end of file make it a quit
	/*	if( gets(buffer) == NULL ){   TIS: compiler is more happy, and this code is not used...
		strcpy( buffer, "quit" );
	}*/


	//  Initialize the current pointer
	p = buffer;
	skipSpaces();
}


int ReaderClass::isSeparator( int c ){
	switch(c) {
	case ' ': case '\t': case '\n':
	case EOF: case '\0': case '\'':
	case ';': case ')' : case '(':
		return 1;
	}
	return 0;
}


void ReaderClass::skipSpaces(){
	while( (*p == ' ')  ||
	       (*p == '\t')    ){
		p++;
	}

	if( *p == ';' ){     // comment
		while( *p ){
			p++;         // read until end of line
		}
	}
}


void ReaderClass::skipNewlines(){
	skipSpaces();

	while( *p == '\0' ){	// end of line
		printSecondaryPrompt();
		fillInputBuffer();
	}
}


Expression *ReaderClass::promptAndRead( Base *b, char *line ){
	//  Loop until the user types something
	if( line== 0 ){
		do{
			printPrimaryPrompt();
			fillInputBuffer();
		}while( !*p );
	}else{
		p = line;
	}

	//  Now that we have something, break it apart
	Expression *val = readExpression( b );

	//  Make sure we are at and of line
	skipSpaces();
	if( *p ){
		error( "unexpected characters at end of line:", p );
	}
	return val;
}


Expression *ReaderClass::readExpression( Base *b ){
	//  See if it's an integer
	if( isdigit(*p) ){
		return new IntegerExpression(readInteger());
	}

	//  Might be a signed integer
	if( (*p == '-') && isdigit(*(p+1)) ){
		p++;
		return new IntegerExpression(- readInteger());
	}

	//  Or it might be a list
	if( *p == '(' ){
		p++;
		return readList( b );
	}
	
	//  Otherwise it must be a symbol
	return readSymbol();
}


int ReaderClass::readInteger(){
	int val = 0;

	while( isdigit(*p) ){
		val = val * 10 + (*p - '0');
		p++;
	}
	return val;
}


ListNode *ReaderClass::readList( Base *b ){
	//  SkipNewlines will issue secondary prompt
	//  until a valid character is typed
	skipNewlines();

	//  If end of list, return empty list
	if( *p == ')' ){
		p++;
		return b->emptyList;
	}

	//  Now we have a non-empty character
	Expression *val = readExpression( b );
	return new ListNode( val, readList(b) );
}


Symbol *ReaderClass::readSymbol(){
	char token[80], *q;

	for( q = token; !isSeparator(*p); ){
		*q++ = *p++;
	}
	*q = '\0';

	return new Symbol( token );
}


};  //  namespace Kamin

