
#include "Kamin/Scheme.h"
#include "Kamin/LispReader.h"
#include "Kamin/IfStatement.h"
#include "Kamin/WhileStatement.h"
#include "Kamin/SetStatement.h"
#include "Kamin/BeginStatement.h"
#include "Kamin/IntegerBinaryFunction.h"
#include "Kamin/BinaryFunction.h"
#include "Kamin/BooleanBinaryFunction.h"
#include "Kamin/UnaryFunction.h"
#include "Kamin/BooleanUnary.h"
#include "Kamin/LambdaFunction.h"
#include "Kamin/Lisp.h"
#include "ui.h"  //  FIX


namespace Kamin {


/*static*/ int Scheme::isTrue( Expression *cond ){
	// the only thing false is nil
	ListNode *nval = cond->isList();
	if( nval && nval->isNil() ){
		return 0;
	}
	return 1;
}


//!  Constructor - interpreter-specific initialization
Scheme::Scheme(){
	//  Initialize global variables
	reader = new LispReader;

	//  Common initialization
	emptyList         = new ListNode   ( 0, 0 );
	globalEnvironment = new Environment( emptyList, emptyList, 0 );
	valueOps          = new Environment( emptyList, emptyList, 0 );
	commands          = new Environment( emptyList, emptyList, valueOps );

	//  Initialize the value of true
	Symbol *truesym = new Symbol("T");  //  TIS fix "#t"
	True  = truesym;
	False = emptyList();                //  TIS fix "#f"

	//  Initialize the command environment
	//  There are no command or value-ops as such in scheme ???

	//  Initialize the global environment
	Environment *ge = globalEnvironment;
	ge->add( new Symbol("if"),       new IfStatement );
	ge->add( new Symbol("while"),    new WhileStatement );
	ge->add( new Symbol("set"),      new SetStatement );
	ge->add( new Symbol("begin"),    new BeginStatement );
	ge->add( new Symbol("+"),        new IntegerBinaryFunction(PlusFunction) );
	ge->add( new Symbol("-"),        new IntegerBinaryFunction(MinusFunction) );
	ge->add( new Symbol("*"),        new IntegerBinaryFunction(TimesFunction) );
	ge->add( new Symbol("/"),        new IntegerBinaryFunction(DivideFunction ));
	ge->add( new Symbol("="),        new BinaryFunction(EqualFunction) );
	ge->add( new Symbol("<"),        new BooleanBinaryFunction(LessThanFunction) );
	ge->add( new Symbol(">"),        new BooleanBinaryFunction(GreaterThanFunction) );
	ge->add( new Symbol("cons"),     new BinaryFunction(ConsFunction) );
	ge->add( new Symbol("car"),      new UnaryFunction(CarFunction) );
	ge->add( new Symbol("cdr"),      new UnaryFunction(CdrFunction) );
	ge->add( new Symbol("number?"),  new BooleanUnary(NumberpFunction) );
	ge->add( new Symbol("symbol?"),  new BooleanUnary(SymbolpFunction) );
	ge->add( new Symbol("list?"),    new BooleanUnary(ListpFunction) );
	ge->add( new Symbol("null?"),    new BooleanUnary(NullpFunction) );
	ge->add( new Symbol("primop?"),  new BooleanUnary(PrimoppFunction) );
	ge->add( new Symbol("closure?"), new BooleanUnary(ClosurepFunction) );
	ge->add( new Symbol("print"),    new UnaryFunction(PrintFunction) );
	ge->add( new Symbol("lambda"),   new LambdaFunction );
	ge->add( new Symbol("nil"), emptyList() );
	ge->add( truesym, truesym );
}


void Scheme::parse( char *line ){
	Expr entered;  //  Expression as entered by users

	//  Now the read-eval-print
	entered = reader->promptAndRead( this, line );

	console->newLine();
	entered.evalAndPrint( commands, globalEnvironment, this );
	console->newLine();
}


};  //  namespace Kamin

