
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CLight
	\ingroup g_materials
	\author  Sean O'Neil
	\brief   Light class for OpenGL
	\date    2000
*/


#ifndef TEDDY_MATERIALS_C_LIGHT_H
#define TEDDY_MATERIALS_C_LIGHT_H


#include "DataStructures/CListNode.h"
#include "DataStructures/CLinkedList.h"
#include "Graphics/OpenGL.h"
#include <string.h>   //  has NULL :)
//#include <memory.h> does not have NULL :(
using namespace DataStructures;
//using namespace Graphics;


namespace Materials {


#define CLightList CLinkedList<CLight>


class CLight : public CListNode<CLight> {
// Operations
public:
	CLight(){
		memset( (char *)this, 0, sizeof(CLight) );
		m_fAttenuation[0] = 1;
		m_fSpotCutoff = 180;
	}
	void SetCurrent(){
		glLightfv( GL_LIGHT0, GL_DIFFUSE, m_fDiffuse );
		glLightfv( GL_LIGHT0, GL_AMBIENT, m_fAmbient );
		glLightfv( GL_LIGHT0, GL_SPECULAR, m_fSpecular );
		glLightfv( GL_LIGHT0, GL_POSITION, m_fPosition );
		glLightf ( GL_LIGHT0, GL_CONSTANT_ATTENUATION, m_fAttenuation[0] );
		glLightf ( GL_LIGHT0, GL_LINEAR_ATTENUATION, m_fAttenuation[1] );
		glLightf ( GL_LIGHT0, GL_QUADRATIC_ATTENUATION, m_fAttenuation[2] );
		glLightf ( GL_LIGHT0, GL_SPOT_CUTOFF, m_fSpotCutoff );
		if( m_fSpotCutoff < 91 ){
			glLightf  ( GL_LIGHT0, GL_SPOT_EXPONENT, m_fSpotExponent );
			glLightfv ( GL_LIGHT0, GL_SPOT_DIRECTION, m_fSpotDirection );
		}
	}
	void   SetDiffuse      ( float r, float g, float b, float a=1.0f )  { m_fDiffuse[0] = r; m_fDiffuse[1] = g; m_fDiffuse[2] = b; m_fDiffuse[3] = a; }
	void   SetDiffuse      ( float *pf )                                { m_fDiffuse[0] = pf[0]; m_fDiffuse[1] = pf[1]; m_fDiffuse[2] = pf[2]; m_fDiffuse[3] = pf[3]; }
	float *GetDiffuse      ()                                           { return m_fDiffuse; }
	void   SetAmbient      ( float r, float g, float b, float a=1.0f )  { m_fAmbient[0] = r; m_fAmbient[1] = g; m_fAmbient[2] = b; m_fAmbient[3] = a; }
	void   SetAmbient      ( float *pf )                                { m_fAmbient[0] = pf[0]; m_fAmbient[1] = pf[1]; m_fAmbient[2] = pf[2]; m_fAmbient[3] = pf[3]; }
	float *GetAmbient      ()                                           { return m_fAmbient; }
	void   SetSpecular     ( float r, float g, float b, float a=1.0f )  { m_fSpecular[0] = r; m_fSpecular[1] = g; m_fSpecular[2] = b; m_fSpecular[3] = a; }
	void   SetSpecular     ( float *pf )                                { m_fSpecular[0] = pf[0]; m_fSpecular[1] = pf[1]; m_fSpecular[2] = pf[2]; m_fSpecular[3] = pf[3]; }
	float *GetSpecular     ()                                           { return m_fSpecular; }
	void   SetPosition     ( float r, float g, float b, float a=1.0f )  { m_fPosition[0] = r; m_fPosition[1] = g; m_fPosition[2] = b; m_fPosition[3] = a; }
	void   SetPosition     ( float *pf )                                { m_fPosition[0] = pf[0]; m_fPosition[1] = pf[1]; m_fPosition[2] = pf[2]; m_fPosition[3] = pf[3]; }
	float *GetPosition     ()                                           { return m_fPosition; }
	void   SetAttenuation  ( float a, float b, float c )                { m_fAttenuation[0] = a; m_fAttenuation[1] = b; m_fAttenuation[2] = c; }
	void   SetAttenuation  ( float *pf )                                { m_fAttenuation[0] = pf[0]; m_fAttenuation[1] = pf[1]; m_fAttenuation[2] = pf[2]; }
	float *GetAttenuation  ()                                           { return m_fAttenuation; }
	void   SetSpotDirection( float x, float y, float z )                { m_fSpotDirection[0] = x; m_fSpotDirection[1] = y; m_fSpotDirection[2] = z; }
	void   SetSpotDirection( float *pf )                                { m_fSpotDirection[0] = pf[0]; m_fSpotDirection[1] = pf[1]; m_fSpotDirection[2] = pf[2]; }
	float *GetSpotDirection()                                           { return m_fSpotDirection; }
	void   SetSpotExponent ( float f )                                  { m_fSpotExponent = f; }
	float  GetSpotExponent ()                                           { return m_fSpotExponent; }
	void   SetSpotCutoff   ( float f )                                  { m_fSpotCutoff = f; }
	float  GetSpotCutoff   ()                                           { return m_fSpotCutoff; }

// Attributes
protected:
	float m_fDiffuse      [4];
	float m_fAmbient      [4];
	float m_fSpecular     [4];
	float m_fPosition     [4];
	float m_fAttenuation  [3];
	float m_fSpotDirection[3];
	float m_fSpotExponent;
	float m_fSpotCutoff;
};


};  //  namespace Materials


#endif  //  TEDDY_MATERIALS_C_LIGHT_H

