
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CMaterial
	\ingroup g_materials
	\author  Sean O'Neil
	\brief   Material class for OpenGL
	\date    2000
*/


#ifndef TEDDY_MATERIALS_C_MATERIAL_H
#define TEDDY_MATERIALS_C_MATERIAL_H


#include "DataStructures/CListNode.h"
#include "DataStructures/CLinkedList.h"
#include "Graphics/OpenGL.h"
#include <string.h>
using namespace DataStructures;
//using namespace Graphics;


namespace Materials {


#define CMaterialList CLinkedList<CMaterial>


class CMaterial : public CListNode<CMaterial> {
// Operations
public:
	CMaterial(){
		memset( (char *)this, 0, sizeof(CMaterial) );
		m_nType = GL_FRONT;
	}
	void   SetType    ( int n )                                   { m_nType = n; }
	int    GetType    ()                                          { return m_nType; }
	void   SetDiffuse ( float r, float g, float b, float a=1.0f ) { m_fDiffuse[0] = r; m_fDiffuse[1] = g; m_fDiffuse[2] = b; m_fDiffuse[3] = a; }
	void   SetDiffuse ( float *pf )                               { m_fDiffuse[0] = pf[0]; m_fDiffuse[1] = pf[1]; m_fDiffuse[2] = pf[2]; m_fDiffuse[3] = pf[3]; }
	float *GetDiffuse ()                                          { return m_fDiffuse; }
	void   SetAmbient ( float r, float g, float b, float a=1.0f ) { m_fAmbient[0] = r; m_fAmbient[1] = g; m_fAmbient[2] = b; m_fAmbient[3] = a; }
	void   SetAmbient ( float *pf )                               { m_fAmbient[0] = pf[0]; m_fAmbient[1] = pf[1]; m_fAmbient[2] = pf[2]; m_fAmbient[3] = pf[3]; }
	float *GetAmbient ()                                          { return m_fAmbient; }
	void   SetSpecular( float r, float g, float b, float a=1.0f ) { m_fSpecular[0] = r; m_fSpecular[1] = g; m_fSpecular[2] = b; m_fSpecular[3] = a; }
	void   SetSpecular( float *pf)                                { m_fSpecular[0] = pf[0]; m_fSpecular[1] = pf[1]; m_fSpecular[2] = pf[2]; m_fSpecular[3] = pf[3]; }
	float *GetSpecular()                                          { return m_fSpecular; }
	void   SetEmissive( float r, float g, float b, float a=1.0f ) { m_fEmissive[0] = r; m_fEmissive[1] = g; m_fEmissive[2] = b; m_fEmissive[3] = a; }
	void   SetEmissive( float *pf )                               { m_fEmissive[0] = pf[0]; m_fEmissive[1] = pf[1]; m_fEmissive[2] = pf[2]; m_fEmissive[3] = pf[3]; }
	float *GetEmissive()                                          { return m_fEmissive; }
	void   SetPower   ( float f )                                 { m_fPower = f; }
	float  GetPower   ()                                          { return m_fPower; }
	void   SetCurrent (){
		glMaterialfv( m_nType, GL_DIFFUSE, m_fDiffuse );
		glMaterialfv( m_nType, GL_AMBIENT, m_fAmbient );
		glMaterialfv( m_nType, GL_SPECULAR, m_fSpecular );
		glMaterialfv( m_nType, GL_EMISSION, m_fEmissive );
		glMaterialf ( m_nType, GL_SHININESS, m_fPower );
	}

// Attributes
protected:
	int   m_nType;
	float m_fDiffuse [4];
	float m_fAmbient [4];
	float m_fSpecular[4];
	float m_fEmissive[4];
	float m_fPower;

};


};  //  namespace Materials


#endif  //  TEDDY_MATERIALS_C_MATERIAL_H

