
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Materials/FractalTexture.h"
#include "Maths/CFractal.h"
#include <iostream>
using namespace Maths;
using namespace std;


namespace Materials {


//!  Constructor-loader
FractalTexture::FractalTexture( const string &name, const int width, const int height, const int nSeed, const float fH, const float fLacunarity ):
Texture(name){
	CFractal      *fractal;
	unsigned char *data;
	float          pos[2];

	fractal = new CFractal();
	fractal->Init( 2, nSeed, fH, fLacunarity );
	data = new unsigned char[width*height*3];
	for( int y=0; y<height; y++ ){
		for( int x=0; x<width; x++ ){
			pos[0] = 2.0f*(float)(x)/(float)(width)  - 1.0f;
			pos[1] = 2.0f*(float)(y)/(float)(height) - 1.0f;
//			float noise = (0.5f + fractal->Noise( pos ) ) * 255.0;
			float noise_r = 128.0f + fractal->fBm( pos, 8) * 128.0f;
			float noise_g = 128.0f + fractal->fBmTest( pos, 8) * 128.0f;
			float noise_b = 128.0f + fractal->Turbulence( pos, 8) * 128.0f;
/*			if( noise < 0.0f || noise > 255.0f ){
				cout << "ERROR" << endl;
			}*/
//			noise *= 64.0f;
			data[ (x+(y*width))*3 +0 ] = (unsigned char)(noise_b);
			data[ (x+(y*width))*3 +1 ] = (unsigned char)(noise_b);
			data[ (x+(y*width))*3 +2 ] = (unsigned char)(noise_b);
		}
	}

	glBindTexture  ( GL_TEXTURE_2D, gltid );
	glPixelStorei  ( GL_UNPACK_ALIGNMENT, 1 );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexEnvi      ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexImage2D   ( GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data );
	delete[] data;
}


};  //  namespace Materials

