
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Materials/Light.h"
#include "Maths/TranslationMatrix.h"
#include "Maths/TransposeMatrix.h"


namespace Materials {


//!  Reservation status for OpenGL lights.
GLenum Light::light_id    [8] = { GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3, GL_LIGHT4, GL_LIGHT5, GL_LIGHT6, GL_LIGHT7 };
int    Light::light_status[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };


//!  Default constructor for light with name only
Light::Light( const string &name ):ModelInstance(name){
	for( int i=0; i<8; i++ ){
		if( light_status[i] == 0 ){
			light_status[i] = 1;
			id = i;
			break;
		}
	}
	ambient  = Color ( 0.0, 0.0, 0.0 );
	diffuse  = Color ( 1.0, 1.0, 1.0 );
	specular = Color ( 0.0, 0.0, 0.0 );

	float pos[4];

	pos[0] = 0.0f;//position.r;
	pos[1] = 0.0f;//position.s;
	pos[2] = 0.0f;//position.t;
	pos[3] = 1.0f;

	glLightfv( light_id[id], GL_POSITION, pos );
}


//!  Set light ambient component
void Light::setAmbient( const Color &a ){
	ambient = a;
}


//!  Set light diffuse component
void Light::setDiffuse( const Color &d ){
	diffuse = d;
}


//!  Set light specular component
void Light::setSpecular( const Color &s ){
	specular = s;
}


//!  Set position of light
void Light::setPosition( const Vector &p ){
	transformations = TranslationMatrix( p );
}


//!  Enable individual light
void Light::enable(){
	glEnable( light_id[id] );
}


//!  Disable individual light
void Light::disable(){
	glDisable( light_id[id] );
}


//!  Apply light conditions  FIX (transformations are not applied)
/*virtual*/ void Light::applyLight(){
	float pos[4];

	pos[0] = 0.0f;//position.r;
	pos[1] = 0.0f;//position.s;
	pos[2] = 0.0f;//position.t;
	pos[3] = 1.0f;

	glPushMatrix();
	TransposeMatrix( transformations ).glMult();
	glLightfv( light_id[id], GL_POSITION, pos );
	glLightfv( light_id[id], GL_AMBIENT,  ambient.rgba );
	glLightfv( light_id[id], GL_DIFFUSE,  diffuse.rgba );
	glLightfv( light_id[id], GL_SPECULAR, specular.rgba );
	glPopMatrix();

}


};  //  namespace Materials

