
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Light
	\ingroup g_lights
	\author  Timo Suoranta
	\brief   Lightsource baseclass
	\warning Very incomplete
	\bug     Poor id and enable/disable management
	\todo    Destructors
	\date    1999, 2000

	Lights are used to lit objects in Scene. At the moment
	this class implements a simple OpenGL light wrapper
	with minimum featurs.
*/


#ifndef TEDDY_MATERIALS_LIGHT_H
#define TEDDY_MATERIALS_LIGHT_H


#include "Models/ModelInstance.h"
#include "Graphics/Color.h"
#include "Maths/Vector.h"
#include <string>
using namespace Models;
using namespace std;


namespace Materials {


class Light : public ModelInstance {
public:
	Light( const string &name );

	void          setAmbient ( const Color &a );
	void          setDiffuse ( const Color &d );
	void          setSpecular( const Color &s );
	void          setPosition( const Vector &p );
	void          enable     ();
	void          disable    ();
	virtual void  applyLight ();
	const string &getName    ();

protected:
	int    id;
	string name;
	Color  ambient;
	Color  diffuse;
	Color  specular;
		
	static GLenum light_id    [8];  //  FIX (query number of available lights from OpenGL)
	static int    light_status[8];	
};


};  //  namespace Materials


#endif  //  TEDDY_MATERIALS_LIGHT_H

