
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Material
	\ingroup g_materials
	\author  Timo Suoranta
	\brief   Surface rendering and lighting properties and textures
	\bug     Destructors missing
	\todo    Shade model
	\todo    Front face (sidedness)
	\todo    Polygon smooth
	\todo    Textures
	\todo    Multiple Materials with weighting
	\date    1999, 2000
*/


#ifndef TEDDY_MATERIALS_MATERIAL_H
#define TEDDY_MATERIALS_MATERIAL_H


#include "Graphics/Color.h"
#include "Graphics/View.h"
#include "Materials/Texture.h"
#include <string>
#include <list>
#include <map>
using namespace Graphics;
using namespace std;


namespace Materials {


//  These will likely change
#define USE_GL_COLOR         (1<< 0)
#define USE_GL_POLYGON_MODE  (1<< 1)
#define USE_GL_SHADE_MODEL   (1<< 2)
#define USE_GL_FRONT_FACE    (1<< 3)
#define USE_GL_BLEND         (1<< 4)
#define USE_GL_ALPHA_TEST    (1<< 5)
#define USE_GL_CULL_FACE     (1<< 6)
#define USE_GL_DEPTH_TEST    (1<< 7)
#define USE_GL_STENCIL_TEST  (1<< 8)
#define USE_GL_LIGHTING      (1<< 9)
#define USE_GL_FOG           (1<<10)
#define USE_GL_NORMALIZE     (1<<11)
#define USE_GL_TEXTURE_2D    (1<<12)


class Material {
public:
	Material( const string &name = "" );
	Material( const string &name, const bool lighting, const int shade_model );
	Material( const string &name, const Color &diffuse );
	Material( const string &name, Texture *t );

	void    setShadeModel( const int sm );
	void    setAmbient   ( const Color &a );
	void    setDiffuse   ( const Color &d );
	void    setSpecular  ( const Color &s );
	void    setShininess ( const float &s );
	void    setEmission  ( const Color &e );

	void    setFlags     ( unsigned short flags );
	void    enableFlags  ( unsigned short flags );
	void    disableFlags ( unsigned short flags );
	void    glApply      ( RenderContext *rc );
	void    debug        ();
	string  getName      ();

	Color   ambient; //  Hack for Scanner

protected:
	string           name;
	Material        *parent;
	float            shininess;
	unsigned short   flags;
	Color            diffuse;
	Color            specular;
	Color            emission;
	GLenum           shade_model;
	GLenum           front_face;
	GLenum           polygon_mode;
/*	list<Material*>  children;
	int              priority;
	float            weight;*/

	static unsigned short master_flags;
};


};  //  namespace Materials


#endif  //  TEDDY_MATERIALS_MATERIAL_H

