
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class	 CFractal
	\ingroup g_maths
	\author  Sean O'Neil
	\date    2000

	This class implements fBm, or fractal Brownian motion. Since fBm uses Perlin
	noise, this class is derived from CNoise. Initialize it with the number of
	dimensions (1 to 4), a random seed, H (roughness ranging from 0 to 1), and
	the lacunarity (2.0 is often used). Many of the fractal routines came from
	"Texturing & Modeling: A Procedural Approach". fBmTest() is my own creation,
	and I created it to generate my first planet.

*/


#ifndef TEDDY_MATHS_C_FRACTAL_H
#define TEDDY_MATHS_C_FRACTAL_H


#include "Maths/CNoise.h"
#include "Maths/NoiseMath.h"
#include <string.h>


namespace Maths {


class CFractal : public CNoise {

public:
	CFractal(){}
	CFractal( int nDimensions, unsigned int nSeed, float fH, float fLacunarity ){
		Init( nDimensions, nSeed, fH, fLacunarity );
	}

	void Init( int nDimensions, unsigned int nSeed, float fH, float fLacunarity ){
		CNoise::Init( nDimensions, nSeed );
		m_fH          = fH;
		m_fLacunarity = fLacunarity;
		float f = 1;
		int   i;
		for( i=0; i<MAX_OCTAVES; i++ ){
			m_fExponent[i] = powf( f, -m_fH );
			f *= m_fLacunarity;
		}
	}
	float fBm               ( float *f, float fOctaves );
	float Turbulence        ( float *f, float fOctaves );
	float Multifractal      ( float *f, float fOctaves, float fOffset );
	float Heterofractal     ( float *f, float fOctaves, float fOffset );
	float HybridMultifractal( float *f, float fOctaves, float fOffset, float fGain );
	float RidgedMultifractal( float *f, float fOctaves, float fOffset, float fThreshold );
	float fBmTest           ( float *f, float fOctaves );

protected:
	float m_fH;
	float m_fLacunarity;
	float m_fExponent[MAX_OCTAVES];
};


};  //  namespace Maths


#endif  //  TEDDY_MATHS_C_FRACTAL_H


