
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Maths/CMatrix.h"
#include "Maths/CQuaternion.h"


namespace Maths {


void CMatrix::operator=( CQuaternion &q ){
	// 9 muls, 15 adds
	float x2 = q.x + q.x, y2 = q.y + q.y, z2 = q.z + q.z;
	float xx = q.x * x2, xy = q.x * y2, xz = q.x * z2;
	float yy = q.y * y2, yz = q.y * z2, zz = q.z * z2;
	float wx = q.w * x2, wy = q.w * y2, wz = q.w * z2;

	A.f14 = A.f24 = A.f34 = A.f41 = A.f42 = A.f43 = 0; A.f44 = 1;
	A.f11 = 1-(yy+zz);	A.f21 = xy+wz;		A.f31 = xz-wy;
	A.f12 = xy-wz;		A.f22 = 1-(xx+zz);	A.f32 = yz+wx;
	A.f13 = xz+wy;		A.f23 = yz-wx;		A.f33 = 1-(xx+yy);
}

CMatrix CMatrix::operator*( CMatrix &m ){
	// 36 muls, 27 adds
	// | f11 f21 f31 f41 |   | m.f11 m.f21 m.f31 m.f41 |   | f11*m.f11+f21*m.f12+f31*m.f13 f11*m.f21+f21*m.f22+f31*m.f23 f11*m.f31+f21*m.f32+f31*m.f33 f11*m.f41+f21*m.f42+f31*m.f43+f41 |
	// | f12 f22 f32 f42 |   | m.f12 m.f22 m.f32 m.f42 |   | f12*m.f11+f22*m.f12+f32*m.f13 f12*m.f21+f22*m.f22+f32*m.f23 f12*m.f31+f22*m.f32+f32*m.f33 f12*m.f41+f22*m.f42+f32*m.f43+f42 |
	// | f13 f23 f33 f43 | * | m.f13 m.f23 m.f33 m.f43 | = | f13*m.f11+f23*m.f12+f33*m.f13 f13*m.f21+f23*m.f22+f33*m.f23 f13*m.f31+f23*m.f32+f33*m.f33 f13*m.f41+f23*m.f42+f33*m.f43+f43 |
	// | 0   0   0   1   |   | 0     0     0     1     |   | 0                             0                             0                             1                                 |
	CMatrix mRet;
	mRet.A.f11 = A.f11*m.A.f11 + A.f21*m.A.f12 + A.f31*m.A.f13;
	mRet.A.f21 = A.f11*m.A.f21 + A.f21*m.A.f22 + A.f31*m.A.f23;
	mRet.A.f31 = A.f11*m.A.f31 + A.f21*m.A.f32 + A.f31*m.A.f33;
	mRet.A.f41 = A.f11*m.A.f41 + A.f21*m.A.f42 + A.f31*m.A.f43+A.f41;
	mRet.A.f12 = A.f12*m.A.f11 + A.f22*m.A.f12 + A.f32*m.A.f13;
	mRet.A.f22 = A.f12*m.A.f21 + A.f22*m.A.f22 + A.f32*m.A.f23;
	mRet.A.f32 = A.f12*m.A.f31 + A.f22*m.A.f32 + A.f32*m.A.f33;
	mRet.A.f42 = A.f12*m.A.f41 + A.f22*m.A.f42 + A.f32*m.A.f43+A.f42;
	mRet.A.f13 = A.f13*m.A.f11 + A.f23*m.A.f12 + A.f33*m.A.f13;
	mRet.A.f23 = A.f13*m.A.f21 + A.f23*m.A.f22 + A.f33*m.A.f23;
	mRet.A.f33 = A.f13*m.A.f31 + A.f23*m.A.f32 + A.f33*m.A.f33;
	mRet.A.f43 = A.f13*m.A.f41 + A.f23*m.A.f42 + A.f33*m.A.f43+A.f43;
	mRet.A.f14 = mRet.A.f24 = mRet.A.f34 = 0;
	mRet.A.f44 = 1;
	return mRet;
}


};  //  namespace Maths

