
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include <stdlib.h>
#include "Maths/CNoise.h"
#include "Maths/CRandom.h"


namespace Maths {


void CNoise::Init( int nDimensions, unsigned int nSeed ){
	m_nDimensions = MIN( nDimensions, MAX_DIMENSIONS );
	CRandom r( nSeed );

	int i, j, k;
	for( i=0; i<256; i++ ){
		m_nMap[i] = i;
		for( j=0; j<m_nDimensions; j++){
			m_nBuffer[i][j] = (float)r.RandomD( -0.5, 0.5 );
		}
		Normalize( m_nBuffer[i], m_nDimensions );
	}

	while( --i ){
		j = r.RandomI(0, 255);
		SWAP( m_nMap[i], m_nMap[j], k );
	}
//	_fpreset();	// Bug in CRandom! Causes messed up floating point operations!
}

float CNoise::Noise(float *f){
	int n[MAX_DIMENSIONS];			// Indexes to pass to lattice function
	float r[MAX_DIMENSIONS];		// Remainders to pass to lattice function
	float w[MAX_DIMENSIONS];		// Cubic values to pass to interpolation function

	for(int i=0; i<m_nDimensions; i++){
		n[i] = Floor(f[i]);
		r[i] = f[i] - n[i];
		w[i] = Cubic(r[i]);
	}

	float fValue;

	switch( m_nDimensions ){
	case 1:
		fValue = Lerp(Lattice(n[0], r[0]),
					  Lattice(n[0]+1, r[0]-1),
					  w[0]);
		break;
	case 2:
		fValue = Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1]),
						   Lattice(n[0]+1, r[0]-1, n[1], r[1]),
						   w[0]),
					  Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1),
						   Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1),
						   w[0]),
					  w[1]);
		break;
	case 3:
		fValue = Lerp(Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2], r[2]),
								Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2], r[2]),
								w[0]),
						   Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2], r[2]),
								Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2], r[2]),
								w[0]),
						   w[1]),
					  Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2]+1, r[2]-1),
								Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2]+1, r[2]-1),
								w[0]),
						   Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2]+1, r[2]-1),
								Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2]+1, r[2]-1),
								w[0]),
						   w[1]),
					  w[2]);
		break;
	case 4:
		fValue = Lerp(Lerp(Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2], r[2], n[3], r[3]),
									 Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2], r[2], n[3], r[3]),
									 w[0]),
								Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2], r[2], n[3], r[3]),
									 Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2], r[2], n[3], r[3]),
									 w[0]),
								w[1]),
						   Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2]+1, r[2]-1, n[3], r[3]),
									 Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2]+1, r[2]-1, n[3], r[3]),
									 w[0]),
								Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2]+1, r[2]-1),
									 Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2]+1, r[2]-1, n[3], r[3]),
									 w[0]),
								w[1]),
						   w[2]),
					  Lerp(Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2], r[2], n[3]+1, r[3]-1),
									 Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2], r[2], n[3]+1, r[3]-1),
									 w[0]),
								Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2], r[2], n[3]+1, r[3]-1),
									 Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2], r[2], n[3]+1, r[3]-1),
									 w[0]),
								w[1]),
						   Lerp(Lerp(Lattice(n[0], r[0], n[1], r[1], n[2]+1, r[2]-1, n[3]+1, r[3]-1),
									 Lattice(n[0]+1, r[0]-1, n[1], r[1], n[2]+1, r[2]-1, n[3]+1, r[3]-1),
									 w[0]),
								Lerp(Lattice(n[0], r[0], n[1]+1, r[1]-1, n[2]+1, r[2]-1),
									 Lattice(n[0]+1, r[0]-1, n[1]+1, r[1]-1, n[2]+1, r[2]-1, n[3]+1, r[3]-1),
									 w[0]),
								w[1]),
						   w[2]),
					  w[3]);
		break;
	default:
		fValue = 0.0f;
	}
	return CLAMP(-0.99999f, 0.99999f, fValue);
}


};  //  namespace Maths


