
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CNoise
	\ingroup g_maths
	\author  Sean O'Neil
	\warning Conversion work in progress
	\date    2000

	This class implements the Perlin noise function. Initialize it with the number
	of dimensions (1 to 4) and a random seed. I got the source for the first 3
	dimensions from "Texturing & Modeling: A Procedural Approach". I added the
	extra dimension because it may be desirable to use 3 spatial dimensions and
	one time dimension. As with CRandom, it is good to encapsulate the noise
	buffers in a class because there may be several planetary bodies, each
	initialized with a different random seed.

*/


#ifndef TEDDY_MATHS_NOISE_H
#define TEDDY_MATHS_NOISE_H


#include "Maths/NoiseMath.h"
#include <string.h>


namespace Maths {


class CNoise {
public:
	CNoise(){}
	CNoise( int nDimensions, unsigned int nSeed ){
		Init( nDimensions, nSeed );
	}

	void  Init ( int nDimensions, unsigned int nSeed );
	float Noise( float *f );

protected:
	int           m_nDimensions;                   //!<  Number of dimensions used by this object
	unsigned char m_nMap[256];                     //!<  Randomized map of indexes into buffer
	float         m_nBuffer[256][MAX_DIMENSIONS];  //!< Random n-dimensional buffer

	float Lattice( int ix, float fx, int iy=0, float fy=0, int iz=0, float fz=0, int iw=0, float fw=0 ){
		int   n[4] = { ix, iy, iz, iw };
		float f[4] = { fx, fy, fz, fw };
		int   nIndex = 0;
		float fValue = 0;
		int   i;

		for( i=0; i<m_nDimensions; i++ ){
			nIndex = m_nMap[(nIndex + n[i]) & 0xFF];
		}
		for( i=0; i<m_nDimensions; i++ ){
			fValue += m_nBuffer[nIndex][i] * f[i];
		}
		return fValue;
	}

};


};  //  namespace Maths


#endif  //  TEDDY_MATHS_NOISE_H


