
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CRandom
	\ingroup g_maths
	\author  Sean O'Neil
	\date    2000

	This class implements a random number generator. It wraps the assembler
	routines used above to allow objects to keep their own seed values instead
	of using globals. This is very useful when a specific random seed is used to
	generate each planetary body.

*/


#ifndef TEDDY_MATHS_C_RANDOM_H
#define TEDDY_MATHS_C_RANDOM_H


#include "Maths/NoiseMath.h"
#include <string.h>


namespace Maths {


class CRandom {
public:
	CRandom(){}
	CRandom( unsigned int nSeed){
		Init(nSeed);
	}

	void Init( unsigned int nSeed ){
		srand( nSeed );
	}

	double Random(){
		return (double)rand()/(double)RAND_MAX;
	}
	double RandomD( double dMin, double dMax ){
		double dInterval = dMax - dMin;
		double d         = dInterval * Random();
		return dMin + MIN( d, dInterval );
	}

	unsigned int RandomI( unsigned int nMin, unsigned int nMax ){
		unsigned int nInterval = nMax - nMin;
		unsigned int i         = (unsigned int)((nInterval+1.0) * Random());
		return nMin + MIN( i, nInterval );
	}

protected:
	unsigned int m_nSeed[5];
};



};  //  namespace Maths


#endif  //  TEDDY_MATHS_NOISE_H


