
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Matrix
	\ingroup g_maths
	\author  Timo Suoranta
	\brief   4*4 matrix
	\warning Rotation matrices not optimized
	\bug     operators [] () not implemented
	\date    1999, 2000

	Matrix can not be explained.
*/


#ifndef TEDDY_MATHS_MATRIX_H
#define TEDDY_MATHS_MATRIX_H


namespace Maths {


class Vector;   //  #include "Vector.h"
class Vector4;  //  #include "Vector4.h"


class Matrix {
public:
	double A[4][4];

	Matrix();
	//Matrix( const Quaternion &q ){

	void    VectorToColumn  ( const Vector4 &v, const int col );
	void    VectorToColumn  ( const Vector  &v, const int col );
	void    VectorToRow     ( const Vector4 &v, const int row );
	void    VectorToRow     ( const Vector  &v, const int row );
	bool    Invert          ();
	Matrix &operator*=      ( const Matrix &m );
	Matrix  operator*       ( const Matrix &m ) const;
	Matrix &ConcatTranslate ( const double x, const double y, const double z );
	Matrix &ConcatRotate    ( const double r, const Vector &orig, const Vector &axis );
	Matrix &ConcatScale     ( const double x, const double y, const double z );
	Matrix &ConcatOrthogonal( double xmin, double xmax, double ymin, double ymax, double zmin, double zmax );
/*	Matrix  operator=       ( const Matrix &m );  */
	Matrix  Copy            ( const Matrix &m );
	Vector  Transform       ( const Vector &v );
};


};  //  namespace Maths


#endif  //  TEDDY_MATHS_MATRIX_H

