
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Maths/NoiseMath.h"


namespace Maths {


// Coefficients of basis matrix.
#define CR00     -0.5f
#define CR01      1.5f
#define CR02     -1.5f
#define CR03      0.5f
#define CR10      1.0f
#define CR11     -2.5f
#define CR12      2.0f
#define CR13     -0.5f
#define CR20     -0.5f
#define CR21      0.0f
#define CR22      0.5f
#define CR23      0.0f
#define CR30      0.0f
#define CR31      1.0f
#define CR32      0.0f
#define CR33      0.0f


float Spline( float x, int nKnots, float *pKnot ){
	int   nSpans = nKnots - 3;
	float c0, c1, c2, c3;

	if( nSpans < 1 ){
		return 0;
	}

	// Find the appropriate 4-point span of the spline.
	x = Clamp( 0, 1, x)  * nSpans;
	int n = (int)x;
	if( n >= nSpans ){
		n = nSpans;
	}
	x     -= n;
	pKnot += n;

    // Evaluate the span cubic at x using Horner's rule.
	c3 = CR00*pKnot[0] + CR01*pKnot[1] + CR02*pKnot[2] + CR03*pKnot[3];
	c2 = CR10*pKnot[0] + CR11*pKnot[1] + CR12*pKnot[2] + CR13*pKnot[3];
	c1 = CR20*pKnot[0] + CR21*pKnot[1] + CR22*pKnot[2] + CR23*pKnot[3];
	c0 = CR30*pKnot[0] + CR31*pKnot[1] + CR32*pKnot[2] + CR33*pKnot[3];
	return ((c3*x + c2)*x + c1) * x + c0;
}


};  //  namespace Maths

