
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Maths/Plane.h"
#include <iostream>
using namespace std;


namespace Maths {


/*!	
	   Ax + Bx + Cx + D = 0
	
	-( Ax + Bx + Cx ) = D
*/
Plane::Plane( const double A, const double B, const double C, const double D ){
	r = A;
	s = B;
	t = C;
	d = D;
}


Plane::Plane( const Vector &p1, const Vector &p2, const Vector &p3 ){
	*this = (p1-p2) * (p1-p3);
	this->Norm();
	d = -( r*p1.r + s*p1.s + t*p1.t );
}


/*!
	Distance - returns positive or negative or (near) zero
	if point is on side or another or in this plane.
*/
double Plane::distance( const Vector &p ) const {
	return( r*p.r + s*p.s + t*p.t + d );
}


//!  Negate plane, that is, flip
void Plane::Neg(){
	r = -r;
	s = -s;
	t = -t;
	d = -d;
	/*cout << "Plane negged "; debug(); cout << "\n";*/
}


//!  Almost equal. Compares each component, allow treshold difference
bool Plane::aEq( const Plane &p, double dir_t, double dist_t ) const {
	if( (*this | p) > dir_t ){
		if( fabs(d-p.d) < dist_t ){
			return true;
		}
	}
	return( false );
}


//!  Assign vector
Plane &Plane::operator=( const Vector &v ){
	r = v.r;
	s = v.s;
	t = v.t; 
	return( *this );
}


//!  Debugging information
void Plane::debug() const {
	cout << r << ", " << s << ", " << t << ", " << d;
}


};  //  namespace Maths


