
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Quaternion
	\ingroup g_maths
	\author  Timo Suoranta
	\brief   Quaternion
	\warning Tested very little.
	\date    2000

	Simple Quaternion class.

	This class is based on Black-Axe's code from FaqSys.
*/


#ifndef TEDDY_MATHS_QUATERNION_H
#define TEDDY_MATHS_QUATERNION_H


#include "trigmath.h"


namespace Maths {


class Matrix;  //  #include "Matrix.h"


class Quaternion {
public:
	double W;
	double X;
	double Y;
	double Z;

    Quaternion();
    Quaternion( const double w, const double x=0.0, const double y=0.0, const double z=0.0 );
  
    friend Quaternion  operator* ( const Quaternion &a, const Quaternion &b );
    const  Quaternion &operator*=( const Quaternion & );
    const  Quaternion &operator~ ();
    const  Quaternion &operator- ();
    const  Quaternion &Normalize ();
    const  Quaternion &FromAxis  ( const double angle, double x, double y, double z );
    void   ToMatrix              ( Matrix &m ) const;
    void   Slerp                 ( const Quaternion &a, const Quaternion &b, const double t );
    const  Quaternion &exp();
    const  Quaternion &log();
};


};  //  namespace Maths


#endif  //  TEDDY_MATHS_QUATERNION_H

