
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CVertex
	\ingroup g_models
	\author  Sean O'Neil
	\brief   ROAM vertex (tksuoran's SmoothVertex, with texture cooords..)
	\date    2000

	This class implements a simple vertex with a position, a normal, and a
	set of 2-dimensional texture coordinates. Note that it is derived from
	CListNode so it can be used with CLinkedList in ListTemplates.h. Note that
	it also has its new and delete operators overridden if USE_ALLOCATOR is
	defined so it will use CMemoryBlock for better performance.
*/


#ifndef TEDDY_MODELS_C_VERTEX_H
#define TEDDY_MODELS_C_VERTEX_H


#include "SysSupport/CMemoryBlock.h"
#include "DataStructures/CLinkedList.h"
#include "DataStructures/CListNode.h"
#include "Graphics/OpenGL.h"
#include "Maths/CVector.h"
#include <string.h>  //  memset
using namespace SysSupport;
using namespace DataStructures;
using namespace Maths;
//using namespace Graphics;


namespace Models {


#define CVertexList CLinkedList<CVertex>


class CVertex : public CListNode<CVertex> {
public:
	CVector m_vPosition;     //!<  The position of the vertex
	CVector m_vNormal;       //!<  The normal of the vertex (used for smooth shading)
	float   m_fTexCoord[2];  //!<  The texture coordinates of the vertex

public:
	CVertex(){
		memset((char *)this, 0, sizeof(CVertex));
	}

	CVertex( const CVector &v ){
		m_vPosition    = v;
		m_vNormal      = v;
		m_fTexCoord[0] = m_fTexCoord[1] = 0;
	}

	void Draw( GLenum nMode=GL_SMOOTH ){
		if( nMode == GL_SMOOTH){
			glNormal3fv( m_vNormal );
		}
		glTexCoord2fv( m_fTexCoord );
		glVertex3fv( m_vPosition );
	}

public:
	void *operator new( unsigned int n ){
		return m_mbVertex.GetElement();
	}
	void operator delete( void *p ){
		m_mbVertex.ReleaseElement( p );
	}

private:
	static CMemoryBlock m_mbVertex;
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_C_VERTEX_H

