
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/ElementArray.h"
#include "Graphics/OpenGL.h"


namespace Models {


/*static*/ const unsigned char ElementArray::HAVE_NORMAL  = 0x01;
/*static*/ const unsigned char ElementArray::HAVE_VERTEX  = 0x02;
/*static*/ const unsigned char ElementArray::HAVE_COLOR   = 0x04;
/*static*/ const unsigned char ElementArray::HAVE_TEXTURE = 0x08;


//!  Default ElementIndex constructor
ElementArray::ElementArray( GLsizei num_elements, GLsizei num_indeces, unsigned char enabled_elements ){
	element_size = 0;
	if( enabled_elements & HAVE_NORMAL ){
		normal_offset = element_size;
		element_size += 3 * sizeof( GLfloat );
	}
	if( enabled_elements & HAVE_VERTEX ){
		vertex_offset = element_size;
		element_size += 3 * sizeof( GLfloat );
	}
	if( enabled_elements & HAVE_COLOR ){
		color_offset = element_size;
		element_size += 4 * sizeof( unsigned char );
	}
	if( enabled_elements & HAVE_TEXTURE ){
		texture_offset = element_size;
		element_size += 2 * sizeof( GLfloat );
	}
	this->enabled_elements = enabled_elements;
	this->num_elements     = num_elements;
	this->num_indeces      = num_indeces;
	used_indeces  = 0;
	used_elements = 0;
	index         = new GLushort[num_indeces];
	array         = new char    [num_elements*element_size];
}


//!  Indexing operator
/*NormalVertexElement &ElementArray::operator[]( int i ){
	void                *address;
	NormalVertexElement *norm_vert_p;

	address     = (void*)(&array[i*element_size]);
	norm_vert_p = (NormalVertexElement*)(address);
	return *norm_vert_p;
}*/


//!  Draw
void ElementArray::drawArray(){
	if( enabled_elements & HAVE_NORMAL ){
		glEnableClientState( GL_NORMAL_ARRAY );
		glNormalPointer( GL_FLOAT, element_size, array+normal_offset );
	}
	if( enabled_elements & HAVE_VERTEX ){
		glEnableClientState( GL_VERTEX_ARRAY );
		glVertexPointer( 3, GL_FLOAT, element_size, array+vertex_offset );
	}

	glDrawElements( GL_TRIANGLES, used_indeces, GL_UNSIGNED_SHORT, index );

	if( enabled_elements & HAVE_NORMAL ){
		glDisableClientState( GL_NORMAL_ARRAY );
	}
	if( enabled_elements & HAVE_VERTEX ){
		glDisableClientState( GL_VERTEX_ARRAY );
	}
}


};  //  namespace Models

