
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   ElementArray
	\ingroup g_models
	\author  Timo Suoranta
	\brief   OpenGL interleaved array
	\bug     Destructors missing?
	\date    2000
*/


#ifndef TEDDY_MODELS_ELEMENT_ARRAY_H
#define TEDDY_MODELS_ELEMENT_ARRAY_H


#include "Graphics/OpenGL.h"


namespace Scenes {
	class Camera;
};
using namespace Scenes;


namespace Models {


class ElementArray {
public:
	ElementArray( GLsizei num_elements, GLsizei num_indeces, unsigned char enabled_elements );
	void drawArray();

	virtual void draw( Camera *c ) = 0;
//	char     *operator[]    ( int i );

	static const unsigned char HAVE_NORMAL;
	static const unsigned char HAVE_VERTEX;
	static const unsigned char HAVE_COLOR;
	static const unsigned char HAVE_TEXTURE;

protected:
	unsigned char  enabled_elements; 
	GLsizei        normal_offset;      //!<  Normal offset
	GLsizei        vertex_offset;      //!<  Vertex offset
	GLsizei        color_offset;       //!<  Color offset
	GLsizei        texture_offset;     //!<  Texture offset
	GLsizei        element_size;       //!<  Size in bytes of single element
	GLsizei        num_elements;       //!<  Reserved elements
	GLsizei        num_indeces;        //!<  Reserved indeces
	GLsizei        used_elements;      //!<  First free element
	GLsizei        used_indeces;       //!<  First free index
	GLushort      *index;              //!<  Index
	char          *array;		       //!<  Array
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_ELEMENT_ARRAY_H

