
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/Face.h"
#include "Models/Vertex.h"
#include "Graphics/OpenGL.h"

#include <iostream>
#include <algorithm>
using namespace std;


namespace Models {


//!  Virtual destructor - FIX (unimplemented)
/*virtual*/ Face::~Face(){
}


//!  Debugging information about the face
void Face::debug(){
	list<Vertex*>::iterator v_it;
	
	cout << "Face: ";
	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		cout << "("; (*v_it)->debug(); cout << ") - "; cout.flush();
		v_it++;
	}
}


//!  Default Face constructor, no initializations
Face::Face(){
}


//!  Insert a vertex to the face - pointer version
void Face::insert( Vertex *v ){
	vertices.push_front( v );
}


//!  Insert a vertex to the face - component version
void Face::insert( const double r, const double s, const double t ){
	vertices.push_front( new Vertex(r,s,t) );
}


//!  Insert a vertex to the face - pointer version, reverse order
void Face::append( Vertex *v ){
	vertices.push_back( v );
}


//!  Drawing a Face Element
void Face::draw() const {
	glBegin( GL_POLYGON );
	list<Vertex*>::const_iterator v_it = vertices.begin();
	while( v_it != vertices.end() ){
		(*v_it)->draw();
		v_it++;
	}
	glEnd();
}


//!  Query
bool Face::contains( const Vertex *v ) const {
	if(  find( vertices.begin(), vertices.end(), v ) 
		 != vertices.end()  ){
		return true;
	}else{
		return false;
	}
}


//!  Reverse order of vertices in Face.  Remember to update normal as well (not done by this)
void Face::reverse(){
	vertices.reverse();
}
/*
bool Face::operator=( const Face &f ){
	list<Vertex *>::iterator v_it;

	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		if( f.contains(*v_it) == false ){
			return false;
		}
	}

}
*/


};  //  namespace Models

