
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Face
	\ingroup g_models
	\author  Timo Suoranta
	\brief   Ordered collection of vertices to define a polygon
	\warning No constructor that would take FlatFace or SmoothFace reference
	\bug     Destructors missing?
	\date    1999, 2000

	Face is a polygon mesh. The polygon is defined by edge vertices.
	Care must be taken to insert or append (and maintain) vertices 
	in counterclockwise (IIRC) order for backfaceculling to work.

	Face baseclass itself does not contain normal information, and
	it therefor it can not be lit.
*/


#ifndef TEDDY_MODELS_FACE_H
#define TEDDY_MODELS_FACE_H


#include "Models/Vertex.h"
#include "Models/Element.h"
#include <list>
using namespace std;


namespace Models {


class Face : public Element {
public:
	Face();
    virtual ~Face();

	void insert  ( const double r, const double s, const double t );
	void insert  ( Vertex *v );
	void append  ( const double r, const double s, const double t );
	void append  ( Vertex *v );
	void reverse ();
	bool contains( const Vertex *v ) const;

	virtual void debug();
	virtual void draw () const;

protected:
	list<Vertex*> vertices;
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_FACE_H

