
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/FlatFace.h"
#include "Models/Vertex.h"
#include "Graphics/OpenGL.h"
#include <iostream>
using namespace std;


namespace Models {


//!  Debugging information
void FlatFace::debug(){
	Face::debug();
	cout << "Normal: ";
	normal.debug();
	cout << endl;
	cout.flush();
}


//!  Default FlatFace constructor, no initializations
FlatFace::FlatFace(){
}


//!  Calculate a normal for the FlatFace - requires at least three vertices
void FlatFace::makeNormal(){
	list<Vertex*>::iterator v_i = vertices.begin();

	if( vertices.size()<3 ){
		cout << "Not enough vertices for FlatFace " << vertices.size() << endl; cout.flush();
		normal = Vector::IdentityX;
		return;
	}

	Vector &a  = *(*v_i++);
	Vector &b  = *(*v_i++);
	Vector &c  = *(*v_i);
	normal     = (a-b)*(a-c);
	normal.Norm();

}


//!  Drawing FlatFace Element
void FlatFace::draw() const {
	glBegin( GL_POLYGON );
	glNormal3f( normal.r, normal.s, normal.t );
	list<Vertex*>::const_iterator v_it = vertices.begin();
	while( v_it != vertices.end() ){
		(*v_it)->draw();
		v_it++;
	}
	glEnd();
}


//!  Query
const Vector &FlatFace::getNormal() const {
	return normal;
}


};  //  namespace Models

