
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   FlatFace
	\ingroup g_models
	\author  Timo Suoranta
	\brief   Flat lighting enabled Face
	\warning No constructor that would take Face or SmoothFace reference
	\bug     Destructors missing?
	\date    1999, 2000

	Flatface is subclass of Face, extended with Face normal
	information. It can be used with FLAT shading lighting.
	For smooth shading, the SmoothFace should be used instead.
*/


#ifndef TEDDY_MODELS_FLAT_FACE_H
#define TEDDY_MODELS_FLAT_FACE_H


#include "Models/Face.h"
#include "Maths/Vector.h"


namespace Models {


class FlatFace : public Face {
public:
	FlatFace();
	virtual ~FlatFace(){}

	void          makeNormal();
	const Vector &getNormal() const;

	virtual void draw () const;
	virtual void debug();

protected:
	Vector normal;
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_FLAT_FACE_H

