
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/Icosahedron.h"
#include "Models/Triangle.h"
#include "Maths/Vector.h"
#include "Models/SmoothFace.h"
#include "Models/SmoothVertex.h"


namespace Models {


#define X .525731112119133606
#define Z .850650808352039932

#define NUM_VERTS   12
#define NUM_TRIGONS 20


static double vertices[NUM_VERTS][3] = {
	{ -X,  0,  Z },
	{  X,  0,  Z },
	{ -X,  0, -Z },
	{  X,  0, -Z },
	{  0,  Z,  X },
	{  0,  Z, -X },
	{  0, -Z,  X },
	{  0, -Z, -X },
	{  Z,  X,  0 },
	{ -Z,  X,  0 },
	{  Z, -X,  0 },
	{ -Z, -X,  0 }
};

static int indeces[NUM_TRIGONS][3] = {
	{  0,  4,  1 },
	{  0,  9,  4 },
	{  9,  5,  4 },
	{  4,  5,  8 },
	{  4,  8,  1 },
	{  8, 10,  1 },
	{  8,  3, 10 },
	{  5,  3,  8 },
	{  5,  2,  3 },
	{  2,  7,  3 },
	{  7, 10,  3 },
	{  7,  6, 10 },
	{  7, 11,  6 },
	{ 11,  0,  6 },
	{  0,  1,  6 },
	{  6,  1, 10 },
	{  9,  0, 11 },
	{  9, 11,  2 },
	{  9,  2,  5 },
	{  7,  2, 11 }
};


Icosahedron::Icosahedron( const double radius ):Mesh( "icosahedron" ){
	int     i;  //  which trigon - ith
	int     j;  //  vertex of current trigon
	double  r;
	double  s;
	double  t;
	Vector *v[3];

	this->radius = radius;
	this->setClipRadius( radius );

	for( i=0; i<NUM_TRIGONS; i++ ){
		for( j=0; j<3; j++ ){
			r = vertices[indeces[i][j]][0] * radius;
			s = vertices[indeces[i][j]][1] * radius;
			t = vertices[indeces[i][j]][2] * radius;
			v[j] = new Vector(r,s,t);
		}
		elements.insert( elements.end(), new Triangle( v[0], v[1], v[2] )  );
	}

}


};  //  namespace Models

