
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   ModelInstance
	\ingroup g_models
	\author  Timo Suoranta
	\brief   Instance of Model
	\date    1999, 2000
	
	ModelInstance is baseclass for Light and Camera
*/


#ifndef TEDDY_MODELS_MODEL_INSTANCE_H
#define TEDDY_MODELS_MODEL_INSTANCE_H


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif


#include "Graphics/Renderable.h"
#include "Graphics/RenderContext.h"
#include "Materials/Material.h"
#include "Maths/Matrix.h"
#include "Maths/Vector.h"
#include "Models/Mesh.h"
#include <string>
using namespace Graphics;
using namespace Materials;
using namespace std;


namespace Models {


class ModelInstance : public Renderable {
public:
	ModelInstance( const string &name );
	ModelInstance( const string &name, Mesh *mesh );
	ModelInstance( const string &name, Mesh *mesh, Vector *offset, Vector *rotation=&Vector::NullVector );

	const string   &getName      () const;
	virtual Mesh   *getMesh      () const;
	void            setMesh      ( Mesh *m );
	virtual void    translate    ( const double x, const double y, const double z );
	void            translate    ( const Vector &v );
	void            rotate       ( const double r, const Vector &orig, const Vector &axis );
	void            scale        ( const double k );

	void            qrot         ( const double angle, const double x, const double y, const double z );
	void            face         ( const Vector &target );

    void            foward       ( const double l );
    void            pitch        ( const double pitch );
    void            roll         ( const double roll );

	double          distanceTo   ( const Vector& target ) const;
	double          distanceSqr  ( const Vector& target ) const;

	Vector          getLook      () const;
	virtual Vector  getLoc       () const;
	virtual float   getClipRadius() const;                 //!<  Wrapper for mesh->getClipRadius();

	virtual Vector  transform    ( const Vector &v );
	virtual Vector  invTransform ( const Vector &v );

	virtual void    draw         ( RenderContext *rc );
	virtual void    update       ( RenderContext *rc );

	Matrix  transformations;

protected:
	string  name;
	Mesh   *mesh;
	bool    accessible;	
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_MODEL_INSTANCE_H

