
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/PolygonMesh.h"
#include "Models/SmoothVertex.h"
#include "Models/SmoothFace.h"
#include "Graphics/OpenGL.h"


namespace Models {


//!  Constructor
PolygonMesh::PolygonMesh( const string &name ):Mesh(name){
}


//!  Constructor
PolygonMesh::PolygonMesh( const string &name, Material *mat ):Mesh(name,mat){
}


//!  Average shared normals for vertices
void PolygonMesh::smoothVertices(){
	list<Vertex*>::iterator   v_it;
	list<Element*>::iterator  f_it;
	Vector                   *normal;

	//  Set normals to null vectors
	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		((SmoothVertex*)(*v_it))->setNormal( Vector(0,0,0) );
		v_it++;
	}

	//  Add face normals to vertex normals
	f_it = elements.begin();
	while( f_it != elements.end() ){
		((SmoothFace*)(*f_it))->smooth();
		f_it++;
	}

	//  Normalize normals
	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		normal = new Vector( 0.0, 0.0, 0.0 );
		((SmoothVertex*)(*v_it))->normNormal();
		v_it++;
	}

	/*
	-- for all vertices
	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		-- make normal
		Vector *normal = new Vector( 0.0, 0.0, 0.0 );

		-- find all 
		f_it = faces.begin();
		count = 0;
		while( f_it != faces.end() ){
			-- that contain the vertex
			if( (*f_it)->contains( (*v_it) ) ){
				-- add that to the normal
				*normal += ((FlatFace*)(*f_it))->getNormal();
				count++;
			}
			f_it++;
		}
		cout << count << ",";
		-- normalize normal
		normal->Norm();
		((SmoothVertex*)(*v_it))->setNormal( normal );
		v_it++;
	}
	*/
}


};  //  namespace Models

