
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/SmoothFace.h"
#include "Models/SmoothVertex.h"
#include "Graphics/OpenGL.h"


namespace Models {


//!  Default SmoothFace constructor, no initializations
SmoothFace::SmoothFace(){
}


//!  Draw SmoothFace Element
void SmoothFace::draw() const {
	//  Different from flatface. Here SmoothVertex::draw will set normal too
	// 	while FlatFace sets normal once here
	glBegin( GL_POLYGON );
	list<Vertex*>::const_iterator v_it = vertices.begin();
	while( v_it != vertices.end() ){
		(*v_it)->draw();
		v_it++;
	}
	glEnd();
}


//!  Calculated average normal for SmoothFace
void SmoothFace::smooth(){
	list<Vertex*>::iterator v_it;

	v_it = vertices.begin();
	while( v_it != vertices.end() ){
		((SmoothVertex*)(*v_it))->addNormal( this->normal );
		v_it++;
	}
}


};  //  namespace Models

