
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   SmoothFace
	\ingroup g_models
	\author  Timo Suoranta
	\brief   Smooth lighting enabled Face
	\warning No constructor that would take Face or FlatFace reference
	\bug     Edge treshold angle not yet supported
	\bug     Destructors missing?
	\date    1999, 2000

	SmoothFace is Face with normal for every vertex.

	Actually it uses SmoothVertex, and they are shared among
	connected SmoothFaces. This arrangement can only be used
	while there is no Edge treshold support.
*/


#ifndef TEDDY_MODELS_SMOOTH_FACE_H
#define TEDDY_MODELS_SMOOTH_FACE_H


#include "Models/FlatFace.h"


namespace Models {


class SmoothFace : public FlatFace {
public:
	SmoothFace();

	void smooth();

	virtual void draw() const;
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_SMOOTH_FACE_H

