
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/SmoothVertex.h"
#include "Graphics/OpenGL.h"


namespace Models {


//!  Default SmoothVertex constructor
SmoothVertex::SmoothVertex():Vertex(){
}


//!  Component SmoothVertex constructor
SmoothVertex::SmoothVertex( const double r, const double s, const double t):Vertex(r,s,t){
}

//!  Vector SmoothVertex constructor
SmoothVertex::SmoothVertex( const Vector &v):Vertex(v){
}



//!  Setting SmoothVertex normal directly
void SmoothVertex::setNormal( const Vector &normal ){
	this->normal = normal;
}


//!  Add to SmoothVertex normal
void SmoothVertex::addNormal( const Vector &add ){
	normal += add;
}


//!  Normalize SmoothVertex normal
void SmoothVertex::normNormal(){
	normal.Norm();
}


//!  Draw SmoothVertex Element, again without glBegin() like vertex::draw()
void SmoothVertex::draw() const {
	glNormal3f( normal.r, normal.s, normal.t );
	glVertex3f( r, s, t );
}


};  //  namespace Models

