
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/Sphere.h"
#include "Models/QuadStrip.h"
#include "Models/SmoothVertex.h"
#include "Maths/trigmath.h"


namespace Models {


Sphere::Sphere( const string &name, const double rad, const int stacks, const int slices ):Mesh("name"){
	double r;
	double x;
	double y;
	double z;
	int    i;
	int    j;
	int    k;

	this->radius = rad;
	this->stacks = stacks;
	this->slices = slices;
	this->setClipRadius( rad );

	//  glVertex3f( 0.0, 0.0, -radius );

	for( j=1; j<stacks-1; j++ ){
		QuadStrip *qs = new QuadStrip();
		for( i=0; i<slices+1; i++ ){
			for( k=0; k<2; k++ ){
				y = radius*cos( (j+k)*M_PI/stacks );
				r =	radius*sin( (j+k)*M_PI/stacks );
				x = r*cos( i*M_2_PI/slices );
				z = r*sin( i*M_2_PI/slices );
				SmoothVertex *sv = new SmoothVertex( x, y, -z );
				Vector normal = Vector( x, y, -z );
				normal.Norm();
				sv->setNormal( normal );
				qs->insert( sv );
			}
		}
		this->insert( qs );
	}

}


//void Sphere::drawElements() const {

	//  draw ring
	/*glBegin( GL_QUAD_STRIP );	
	for( i=0; i<slices+1; i++ ){
		for( k=0; k<2; k++ ){
			x = radius*(k+2)*cos( i*(PI*2)/slices );
			z = radius*(k+2)*sin( i*(PI*2)/slices );
			glVertex3f( x, 0.0, z );
		}
	}
	glEnd();*/

	//  draw slice edges
/*
	for( i=0; i<slices; i++ ){
		glBegin( GL_LINE_LOOP );	
		for( j=0; j<2*stacks; j++ ){
			y = radius*cos( j*(PI)/stacks );
			r =	radius*sin( j*(PI)/stacks );
			x = r*cos( i*(PI*2)/slices );
			z = r*sin( i*(PI*2)/slices );
			glVertex3f( x, z, y );
		}
		glEnd();
	}


	//  draw stack edges

	for( j=0; j<2*stacks; j++ ){
		glBegin( GL_LINE_LOOP );	
		for( i=0; i<slices; i++ ){
			y = radius*cos( j*(PI)/stacks );
			r =	radius*sin( j*(PI)/stacks );
			x = r*cos( i*(PI*2)/slices );
			z = r*sin( i*(PI*2)/slices );
			glVertex3f( x, z, y );
		}
		glEnd();
	}
*/

	//  glVertex3f( 0.0, 0.0, radius );
//}


};  //  namespace Models

