
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/Torus.h"
#include "Models/QuadStrip.h"
#include "Models/SmoothVertex.h"
#include "Maths/Vector.h"
#include "Maths/trigmath.h"


namespace Models {


Torus::Torus( const string &name, const double rt, const double rc, const int numt, const int numc ):Mesh(name){
	int    i;
	int    j;
	int    k;
	double s;
	double t;
	double x;
	double y;
	double z;

	for( i=0; i<numc; i++ ){
		QuadStrip *qs = new QuadStrip();
		for( j=0; j<=numt; j++ ){
			for( k=1; k>=0; k-- ){
				s = (i + k) % numc + 0.5;
				t = j % numt;

				x = (rt + rc * cos(s*M_2_PI/numc)) * cos(t*M_2_PI/numt);
				y = (rt + rc * cos(s*M_2_PI/numc)) * sin(t*M_2_PI/numt);
				z = rc * sin(s*M_2_PI/numc);
				SmoothVertex *sm = new SmoothVertex( x, y, z );

				x = cos(t*M_2_PI/numt) * cos(s*M_2_PI/numc);
				y = sin(t*M_2_PI/numt) * cos(s*M_2_PI/numc);
				z = sin(s*M_2_PI/numc);
				sm->setNormal( Vector( x, y, z ) );
				qs->insert( sm );
			}
		}
		this->insert( qs );
	}
/*
	for( i=0; i<slices+1; i++ ){
		QuadStrip *qs = new QuadStrip();
		for( j=0; j<stacks; j++ ){
			for( k=0; k<2; k++ ){
				base_x = cos( (i+k)*(PI*2)/slices );
				base_z = sin( (i+k)*(PI*2)/slices );
				y = minor_radius*cos( (j)*(PI*2)/stacks );
				r =	minor_radius*sin( (j)*(PI*2)/stacks );
				qs->insert( base_x*(major_radius+r), y, base_z*(major_radius+r) );
			}
		}
		this->insert( qs );
	}
*/
}

/*
void drawTorus(float rc, int numc, float rt, int numt)
{
    int i, j, k;
    double s, t;
    double x, y, z;
    double pi, twopi;

    pi = 3.14159265358979323846;
    twopi = 2 * pi;
 
    for (i = 0; i < numc; i++) {
	glBegin(GL_QUAD_STRIP);
        for (j = 0; j <= numt; j++) {
	    for (k = 1; k >= 0; k--) {
		s = (i + k) % numc + 0.5;
		t = j % numt;

		x = cos(t*twopi/numt) * cos(s*twopi/numc);
		y = sin(t*twopi/numt) * cos(s*twopi/numc);
		z = sin(s*twopi/numc);
		glNormal3f(x, y, z);

		x = (rt + rc * cos(s*twopi/numc)) * cos(t*twopi/numt);
		y = (rt + rc * cos(s*twopi/numc)) * sin(t*twopi/numt);
		z = rc * sin(s*twopi/numc);
		glVertex3f(x, y, z);
	    }
        }
	glEnd();
    }
}
*/


};  //  namespace Models

