
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Models/Triangle.h"
#include "Graphics/OpenGL.h"


namespace Models {


//!  Poor implementation for recursion depth, fix later
int Triangle::rec_depth = 0;


/*!
	Poor interface to set recursion depth
	Recursion depth should be component specific...
*/
void Triangle::setRecDepth( int r ){
	rec_depth = r;
}


//!  Triangle constructor, three vectors.. should be vertices?
Triangle::Triangle( Vector *a, Vector *b, Vector *c ){
	this->a = a;
	this->b = b;
	this->c = c;
}


//!  Draw Triangle Element. Entrypoint.
void Triangle::draw() const {
	subdivide( a, b, c, rec_depth );
}


//!  Recursive part of Draw Triangle Element. Includes glBegin( GL_POLYGON )
void Triangle::subdivide( Vector *a1, Vector *b1, Vector *c1, int counter ) const {

	if( counter==0 ){
		Vector na = Vector( a1->r, a1->s, a1->t );
		Vector nb = Vector( b1->r, b1->s, b1->t );
		Vector nc = Vector( c1->r, c1->s, c1->t );

		na.Norm();
		nb.Norm();
		nc.Norm();

		glBegin( GL_POLYGON );
		glNormal3f( na.r,  na.s, na.t );
		glVertex3f( a1->r, a1->s, a1->t );

		glNormal3f( nc.r,  nc.s,  nc.t );
		glVertex3f( c1->r, c1->s, c1->t );

		glNormal3f( nb.r,  nb.s,  nb.t );
		glVertex3f( b1->r, b1->s, b1->t );

		glEnd();
		return;
	}

	Vector a2 = *a1 + *b1; a2.Norm(); a2 *= a1->Len();
	Vector b2 = *b1 + *c1; b2.Norm(); b2 *= b1->Len();
	Vector c2 = *c1 + *a1; c2.Norm(); c2 *= c1->Len();

	subdivide(  a1, &a2, &c2, counter-1 );
	subdivide(  b1, &b2, &a2, counter-1 );
	subdivide(  c1, &c2, &b2, counter-1 );
	subdivide( &a2, &b2, &c2, counter-1 );
}


//!  Exact Triangle equality comparison
bool Triangle::operator==( const Triangle &t ) const {
	if( a == t.a &&
		b == t.b &&
		c == t.b    ){
		return true;
	}else{
		return false;
	}
}


};  //  namespace Models

