
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Triangle
	\ingroup g_models
	\author  Timo Suoranta
	\brief   Triangle is the primivite for subdivable surfaces 
	\bug     Recursion depth not settable properly
	\bug     Normal vector not settable
	\bug     Destructors missing?
	\date    1999, 2000

	Triangle is 
*/


#ifndef TEDDY_MODELS_TRIANGLE_H
#define TEDDY_MODELS_TRIANGLE_H


#include "Models/Element.h"
#include "Maths/Vector.h"
#include <list>
using namespace std;
using namespace Maths;


namespace Models {


class Triangle : public Element {
public:
	Triangle( Vector *a, Vector *b, Vector *c );

	virtual void draw     () const;
	virtual void subdivide( Vector *a1, Vector *b1, Vector *c1, int counter ) const;

	bool operator==( const Triangle &t ) const;

	static void setRecDepth( const int r );

protected:
	Vector *a;
	Vector *b;
	Vector *c;

	static int rec_depth;  // FIX
};


};  //  namespace Models


#endif  //  TEDDY_MODELS_TRIANGLE_H

