
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "PhysicalComponents/Button.h"
#include "PhysicalComponents/Frame.h"
#include "PhysicalComponents/GradientFill.h"
#include "PhysicalComponents/LayoutConstraint.h"
#include "PhysicalComponents/Style.h"


namespace PhysicalComponents {


//! Constructor
Button::Button( string label ){
	this->label = label;

	constraint  = new LayoutConstraint();
	//constraint->local_x_offset_pixels  = style->inner_x_space_pixels;
	//constraint->local_y_offset_pixels  = style->inner_y_space_pixels;
	constraint->min_x_fill_pixels        = label.size() * style->button_font->getWidth() + (style->inner_x_space_pixels)*2;
	constraint->min_y_fill_pixels        = style->button_font->getHeight() + (style->inner_y_space_pixels)*2;

	this->insert( new Frame() );
	this->insert(
		new GradientFill(
			Color( 0.8f, 0.8f, 0.8f, 1.0f ),
			Color( 0.7f, 0.7f, 0.7f, 1.0f ),
			Color( 0.5f, 0.5f, 0.5f, 1.0f ),
			Color( 0.6f, 0.6f, 0.6f, 1.0f )
		)
	);

	state    = up;
	ordering = post_self;
}


//!  Area interface
/*virtual*/ void Button::drawSelf(){
	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glEnable    ( GL_BLEND );
	glEnable    ( GL_TEXTURE_2D );
	glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	style->text_color.glApply();
	drawString  ( style->button_font, label, style->inner_x_space_pixels, style->inner_y_space_pixels );
	glPopAttrib ();
}


//!  MouseListener interface
/*virtual*/ void Button::mouseKey( int button, int state, int x, int y ){
	int dummy = 1;
}


//!  MouseListener interface
/*virtual*/ void Button::mouseMotion( int x, int y ){
}


};  //  namespace PhysicalComponents

