
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Console
	\ingroup g_physical_components
	\author  Timo Suoranta
	\brief   Text (fixed width font) console userinterface component
	\warning Current console class is more instance than general console
	\todo    Selection
	\date    2000
	
	Console component can be used for standard input and output in
	simple text form. 

	At the moment MouseListener is used for window dragging, but
	that will change into selection later, and dragging will be
	put into Frame Label.

	The basic input and output is working now, but there are still
	a number of cliches, so be aware.

	Console may or may not be connected to a parser. Currently
	I have plugged in Kamin Scheme interpreter as parser. If you
	enable it, be sure not to try to parse lines that are not
	complete, legal scheme statements. At the moment it is only
	small subset of scheme, and you can only send one line at time
	to it.
*/


#ifndef TEDDY_PHYSICAL_COMPONENTS_CONSOLE_H
#define TEDDY_PHYSICAL_COMPONENTS_CONSOLE_H


#include "Inputs/KeyListener.h"
#include "Kamin/Base.h"
#include "PhysicalComponents/Area.h"
#include <vector>
#include <string>
using namespace Inputs;
using namespace Kamin;
using namespace std;


namespace PhysicalComponents {


class Console : public Area, public KeyListener {
public:
	Console( const int w, const int h, Base *b );

	//  Console interface
	Console &setLine   ( const int l, const string &s );	
	Console &operator<<( const string &s );
	
	void cursorDown();
	void newLine   (){ insertLine(cy+1); cursorDown(); cx=0; };
	void insertLine( const int y );                                     //!<  Insert new line
	void killLine  ( const int y );                                     //!<  Kill line
	void killChar  ( const int x, const int y );                        //!<  Kill character
	
	void scroll    ( int x1, int y1, int x2, int y2, int dx, int dy );  //!<  Scroll area 

	void parse     ( string *commandline );

	//  Area interface
	virtual void drawSelf();  // const? Area::

	//  KeyListener interface
	virtual void focusActive( const bool active );
	virtual void keyDown    ( SDL_keysym key );
	virtual void keyUp      ( SDL_keysym key );

protected:
	virtual void setSize( const int width, const int height );

	vector<string*>  buffer_lines;      //!<  Buffer
	int              font_width;        //!<  For quick access.. FIX? Observer?
	int              font_height;       //!<  For quick access.. FIX? Observer?
	int              width_chars;       //!<  Number of visible chars per line
	int              height_chars;      //!<  Number of visible lines
	int              max_height_chars;  //!<  Number of lines in buffer
	int              topleft_x;         //!<  Current scroll x-position - first visible char
	int              topleft_y;         //!<  Current scroll y-position - first visible line
	int              cx;                //!<  Cursor X position
	int              cy;                //!<  Cursor Y position
	bool             cursor_visible;    //!<  Do we want visible cursor?
	Color            text_color;        //!<  Text color
	Base            *parser;
};


};  //  namespace PhysicalComponents


#endif  //  TEDDY_PHYSICAL_COMPONENTS_CONSOLE_H

