
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "PhysicalComponents/Moving.h"
#include "PhysicalComponents/LayoutConstraint.h"
#include "PhysicalComponents/Style.h"
#include "Inputs/FocusManager.h"


namespace PhysicalComponents {


//!  Constructor
Moving::Moving(){
	constraint = new LayoutConstraint();
	constraint->local_x_fill_pixels = style->glyph_x_pixels;
	constraint->local_y_fill_pixels = style->glyph_y_pixels;
	ordering = pre_self;
	FocusManager::registerArea( this );
}


//!  Destructor
Moving::~Moving(){
	//  FIX
}


//!  Drawing code
void Moving::drawSelf(){
	int width;
	int height;

	glPushAttrib( GL_ALL_ATTRIB_BITS );
	getSize( &width, &height );
	Color(1.0f,1.0f,1.0f).glApply();
	this->drawFillRect( 1, 1, width-1, height-1 );
	drawBiColRect( style->hilight_color, style->shadow_color,  0, -1, width, height-1 );
	glPopAttrib();

}


//!  MouseListener interface
/*virtual*/ void Moving::mouseKey( int button, int state, int x, int y ){
	if( state==SDL_PRESSED ){
		mouse_x[button] = x;
		mouse_y[button] = y;
	}
	mouse_b[button] = state;
}


//!  MouseListener interface
/*virtual*/ void Moving::mouseMotion( int x, int y ){
	for( int b=1; b<4; b++ ){
		if( mouse_b[b]==SDL_PRESSED ){
			int x_delta = x - mouse_x[b];
			int y_delta = y - mouse_y[b];
			mouse_x[b] = x;
			mouse_y[b] = y;
			parent->moveDelta( x_delta, y_delta );
		}
	}
}


};  //  namespace PhysicalComponents

