
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "PhysicalComponents/WindowFrame.h"
#include "PhysicalComponents/LayoutConstraint.h"
#include "PhysicalComponents/Style.h"
#include "Graphics/Color.h"
#include "PhysicalComponents/Button.h"
#include "PhysicalComponents/Label.h"
#include "PhysicalComponents/Sizing.h"
#include "PhysicalComponents/Moving.h"
#include <string>
using namespace Graphics;
using namespace Inputs;
using namespace std;


namespace PhysicalComponents {


//!  Default constructor
WindowFrame::WindowFrame( string name ){
	constraint = new LayoutConstraint();
	constraint->local_x_offset_pixels  =  -style->frame_x_pixels;
	constraint->local_y_offset_pixels  =  -style->frame_y_pixels;
	constraint->local_x_fill_pixels    = 2*style->frame_x_pixels;
	constraint->local_y_fill_pixels    = 2*style->frame_y_pixels;
	constraint->parent_x_fill_relative = 1.0;
	constraint->parent_y_fill_relative = 1.0;

	this->insert( new Sizing() );
	this->insert( new Moving() );

	ordering = pre_self;
}


//!  Destructor FIX
/*virtual*/ WindowFrame::~WindowFrame(){
}


//!  Drawing code
void WindowFrame::drawSelf(){
	int width;
	int height;
	//int i = style->frame_x_pixels;

/*
#define GL_CURRENT_BIT                    0x00000001
#define GL_POINT_BIT                      0x00000002
#define GL_LINE_BIT                       0x00000004
#define GL_POLYGON_BIT                    0x00000008
#define GL_POLYGON_STIPPLE_BIT            0x00000010
#define GL_PIXEL_MODE_BIT                 0x00000020
#define GL_LIGHTING_BIT                   0x00000040
#define GL_FOG_BIT                        0x00000080
#define GL_DEPTH_BUFFER_BIT               0x00000100
#define GL_ACCUM_BUFFER_BIT               0x00000200
#define GL_STENCIL_BUFFER_BIT             0x00000400
#define GL_VIEWPORT_BIT                   0x00000800
#define GL_TRANSFORM_BIT                  0x00001000
#define GL_ENABLE_BIT                     0x00002000
#define GL_COLOR_BUFFER_BIT               0x00004000
#define GL_HINT_BIT                       0x00008000
#define GL_EVAL_BIT                       0x00010000
#define GL_LIST_BIT                       0x00020000
#define GL_TEXTURE_BIT                    0x00040000
#define GL_SCISSOR_BIT                    0x00080000
#define GL_ALL_ATTRIB_BITS                0x000fffff
*/
//	glPushAttrib( GL_ALL_ATTRIB_BITS );

	getSize( &width, &height );

	drawBiColRect( style->shadow_color, style->hilight_color, style->frame_x_pixels-1, style->frame_y_pixels-2, width-style->frame_x_pixels+1, height-style->frame_y_pixels );
	drawBiColRect( style->hilight_color, style->shadow_color,  0, -1, width, height-1 );

	int x0 = style->frame_x_pixels - 1;
	int y0 = style->frame_y_pixels - 1;
	int x1 = width  - style->frame_x_pixels+1;
	int y1 = height - style->frame_y_pixels+1;
	int x2 = width  - 1;
	int y2 = height;

	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

	style->frame_color.glApply();

	glBegin( GL_QUADS );

/*	Color::CYAN.glApply();

	vertex2i( 0,     0 );
	vertex2i( width, 0 );
	vertex2i( width, height );
	vertex2i( 0,     height );*/

//	Color::RED.glApply();

	vertex2i( 1,  1  );
	vertex2i( x2, 1  );
	vertex2i( x2, y0 );
	vertex2i( 1,  y0 );

//	Color::BLUE.glApply();	

	vertex2i( 1,  y1   );
	vertex2i( x2, y1   );
	vertex2i( x2, y2-1 );
	vertex2i( 1,  y2-1 );

//	Color::YELLOW.glApply();

	vertex2i( 1,  y0 );
	vertex2i( x0, y0 );
	vertex2i( x0, y1 );
	vertex2i( 1,  y1 );

	vertex2i( x1, y0 );
	vertex2i( x2, y0 );
	vertex2i( x2, y1 );
	vertex2i( x1, y1 );

	glEnd();



//	glPopAttrib();

}


//!  Area Layout Interface - Move area
void WindowFrame::moveDelta( const int x_delta, const int y_delta ){
	parent->moveDelta( x_delta, y_delta );
}


//!  Area Layout interface - Resize area
void WindowFrame::sizeDelta( const int x_delta, const int y_delta ){
	parent->sizeDelta( x_delta, y_delta );
}


#if 0
//!  MouseListener interface
/*virtual*/ void WindowFrame::mouseKey( int button, int state, int x, int y ){
	if( state==GLUT_DOWN ){
		mouse_x[button] = x;
		mouse_y[button] = y;
	}
	mouse_b[button] = state;	
}


//!  MouseListener interface
/*virtual*/ void WindowFrame::mouseMotion( int x, int y ){
	for( int b=0; b<3; b++ ){
		if( mouse_b[b]==GLUT_DOWN ){
			int x_delta = x - mouse_x[b];
			int y_delta = y - mouse_y[b];
			mouse_x[b] = x;
			mouse_y[b] = y;
			parent->moveDelta( x_delta, y_delta );
		}
	}
}
#endif


};  //  namespace PhysicalComponents

