
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Physics/RigidBody.h"


namespace Physics {


//!  Constructor with MoodelInstance (defaults to NULL for those subclasses that want to set it by themselves)
RigidBody::RigidBody( ModelInstance *instance /*=NULL*/ ){
	tick_translation       = Vector( 0, 0, 0 );
	tick_translation_delta = Vector( 0, 0, 0 );
	tick_rotation          = Vector( 0, 0, 0 );
	tick_rotation_delta    = Vector( 0, 0, 0 );
	this->instance         = instance;
}


//!  Destructor
/*virtual*/ RigidBody::~RigidBody(){
	//  FIX
}


//!  Get bind ModelInstance
ModelInstance *RigidBody::getInstance(){
	return instance;
}


//!  Bind new ModelInstance
void RigidBody::setInstance( ModelInstance *instance ){
	this->instance = instance;
}


//!  Simulate one tick
void RigidBody::tick(){
	instance->translate( tick_translation.r, tick_translation.s, tick_translation.t );
	instance->rotate( tick_rotation.r, Vector::NullVector, Vector::IdentityX );
	instance->rotate( tick_rotation.s, Vector::NullVector, Vector::IdentityY );
	instance->rotate( tick_rotation.t, Vector::NullVector, Vector::IdentityZ );
	tick_translation += tick_translation_delta;
	tick_rotation    += tick_rotation_delta;
}


};  //  namespace Physics

