
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Physics/System.h"
#include "Graphics/OpenGL.h"


namespace Physics {


//!  Constructor
System::System(){
	sys_time = 0;
}


//!  Destructor
/*virtual*/ System::~System(){
	//  FIX
}


//!  Add new RigidBody to System
void System::addRigidBody( RigidBody *rb ){
	rigid_bodies.push_back( rb );
}


//  Task interface
/*virtual*/ void System::update(){
	//  Update rigid bodies
	list<RigidBody*>::iterator rb_it = rigid_bodies.begin();
	while( rb_it != rigid_bodies.end() ){
		(*rb_it)->tick();
		rb_it++;
	}

	//  Update system time;
	sys_time += SIMULATION_INTERVAL_MS;
}


//!  Return system time
float System::getTime(){
	return sys_time;
}


list<RigidBody*> System::getRigidBodies(){
	return rigid_bodies;
}


};  //  namespace Physics

