
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Scanner.h"
#include "Graphics/View.h"
#include "Materials/NoiseTexture.h"
#include "Materials/FractalTexture.h"
#include "Maths/trigmath.h"
#include "Maths/Vector.h"
#include "Models/ModelInstance.h"
#include "PhysicalComponents/GradientFill.h"
#include "PhysicalComponents/TextureFill.h"
#include "PhysicalComponents/LayoutConstraint.h"
#include "PhysicalComponents/Style.h"
#include "PhysicalComponents/WindowFrame.h"
#include "Physics/System.h"
#include "Physics/RigidBody.h"
#include "Scenes/Camera.h"
#include <stdio.h>
#include <string>
using namespace Graphics;
using namespace Models;
using namespace Physics;
using namespace PhysicalComponents;
using namespace std;


//!  Constructor
Scanner::Scanner( Camera *camera, System *system ):Area(){
	this->camera = camera;
	this->system = system;
	constraint = new LayoutConstraint();
	constraint->local_x_offset_pixels = SCREEN_X-280;
	constraint->local_y_offset_pixels =  20;
	constraint->local_x_fill_pixels   = 256;
	constraint->local_y_fill_pixels   = 256;


	this->insert(
		new PhysicalComponents::GradientFill(
			Color( 0.7f, 0.0f, 0.1f, 0.43f ),
			Color( 0.7f, 0.0f, 0.1f, 0.43f ),
			Color( 0.7f, 0.0f, 0.1f, 0.13f ),
			Color( 0.7f, 0.0f, 0.1f, 0.13f )
		)
	);

/*
	this->insert(
		new TextureFill(
//			new NoiseTexture( "Fractal Texture", 256, 256, 190800 )
			new FractalTexture( "Fractal Texture", 256, 256, 190800, 0.5, 2.0f )

		)
	);*/

	this->insert(
		new WindowFrame( "Scanner" )
	);

	range = 1;
	ordering = post_self;
}


void Scanner::cycle(){
	range = (range<32)? range*=2 : 1;
}


//!  Drawing code
void Scanner::drawSelf(){
	list<RigidBody*> rigid_bodies = system->getRigidBodies();
	Vector loc;
	int    x,  y,  z;
	int    x1, y1, y2;

	glPushAttrib( GL_ALL_ATTRIB_BITS );

	glColor4f( 1.0, 1.0, 1.0, 1.0 );

	list<RigidBody*>::iterator rb_it = rigid_bodies.begin();
	while( rb_it != rigid_bodies.end() ){
		loc = (*rb_it)->getInstance()->getLoc();
		loc = camera->invTransform( loc );

		Material *mat = (*rb_it)->getInstance()->getMesh()->getMaterial();
		if( mat!=NULL ){
			mat->ambient.glApply();
		}

		x = (int)( loc.r*range/64.0);
		y = (int)( loc.s*range/64.0);
		z = (int)(-loc.t*range/64.0);
		
		x1 = x;
		y1 = -z / 4;
		y2 = y1 - y / 2;
		
		if( (y2 > - 50) && (y2 < 50) &&
			(x1 > -100) && (x1 <100)    ){
		
			x1 += 100;
			y1 +=  50;
			y2 +=  50;
		
			glBegin( GL_LINES );
			vertex2i( x1  , y2   ); vertex2i( x1+3, y2   );
			vertex2i( x1  , y2+1 ); vertex2i( x1+4, y2+1 );
			vertex2i( x1  , y2+2 ); vertex2i( x1+4, y2+2 );
			vertex2i( x1  , y2+3 ); vertex2i( x1+3, y2+3 );
			vertex2i( x1  , y1   ); vertex2i( x1  , y2   );
			vertex2i( x1+1, y1   ); vertex2i( x1+1, y2   );
			glEnd();
		}
		rb_it++;
	}

	glPopAttrib();
}

