
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Camera
	\ingroup g_scenes
	\author  Timo Suoranta
	\brief   Camera
	\date    2000
*/


#ifndef TEDDY_SCENES_CAMERA_H
#define TEDDY_SCENES_CAMERA_H


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif


#include "Maths/Plane.h"
#include "Maths/Vector.h"
#include "Models/Mesh.h"
#include "Models/ModelInstance.h"
namespace PhysicalComponents {
	class Projection;
};
using namespace PhysicalComponents;
#include <string>
using namespace Graphics;
using namespace Models;
using namespace std;


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif


namespace Scenes {


class Scene;


class Camera : public ModelInstance {
public:

	Camera( const string &name, Mesh *skybox_m, Scene *scene );
	virtual ~Camera();

	void           setFov      ( const float fov );
	float          getFov      () const;
	void           projectScene( Projection *p );
	void           updatePlanes();
	bool           cull        ( ModelInstance *mi );
	ModelInstance *pickInstance( Projection *p, const int x, const int y );

protected:
	float   far_clip;       //!<  Far clip plane z
	float   near_clip;      //!<  Near clip plane z
	float   fov;            //!<  Field of vision
	Plane  *view_plane[6];  //!<  Camera view volume planes for clipping
	Scene  *scene;          //!<  Scene to draw
};


};  //  namespace Scenes


#endif  //  TEDDY_SCENES_CAMERA_H

