
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   Scene
	\ingroup g_scenes
	\author  Timo Suoranta
	\brief   Collections of Models, ModelInstances, Lights and Cameras
	\bug     Destructors missing?
	\todo    Improve light management
	\todo    Improved sorting, especially for transluent surface support
	\date    1999, 2000
*/


#ifndef TEDDY_SCENES_SCENE_H
#define TEDDY_SCENES_SCENE_H


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif

#include "Models/ModelInstance.h"
#include "Materials/Light.h"
#include "Maths/Vector.h"
#include "PhysicalComponents/Projection.h"
#include <string>
#include <list>
using namespace Models;
using namespace std;


namespace Scenes {


class Camera;


class Scene {
public:
	Scene( const string &name );

	void                  addInstance  ( ModelInstance *i );
	void                  addLight     ( Light         *l );
	void                  draw         ( Camera *c, Projection *p );
	ModelInstance        *pickInstance ( Camera *c, Projection *p );
	void                  update       ( Projection *p );
	void                  setSort      ( const bool state );
	void                  translate    ( const Vector &c_pos );
	list<ModelInstance*> &getInstances ();

	static int culled;

protected:
	list<ModelInstance*> instances;
	list<Light*>         lights;
	string               name;
	bool                 sort;
};


};  //  namespace Scenes


#endif  //  TEDDY_SCENES_SCENE_H

