
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "Scenes/SkyBox.h"
#include "Models/Line.h"
#include "Graphics/OpenGL.h"


namespace Models {


//!  Dirty hack to let users disable skybox - FIX
bool SkyBox::disable = false;


//!  Constructor. Loads textures. Should have Destructor as well...
SkyBox::SkyBox( int d ):Mesh("skybox"){
	/*
	
	Currently no textures used

	front = new Material( "SkyBox Front", new Texture("simple_sunset_FR.raw") );
	back  = new Material( "SkyBox Back",  new Texture("simple_sunset_BK.raw") );
	left  = new Material( "SkyBox Left",  new Texture("simple_sunset_LF.raw") );
	right = new Material( "SkyBox Right", new Texture("simple_sunset_RT.raw") );
	up    = new Material( "SkyBox Up",    new Texture("simple_sunset_UP.raw") );
	down  = new Material( "SkyBox DOwn",  new Texture("simple_sunset_DN.raw") );
	*/
};


//!  Skybox rendering code
void SkyBox::drawElements(){
	glPushAttrib( GL_ALL_ATTRIB_BITS );

	glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
	double a = 2.0;
	glColor4f( 0.5f, 0.5f, 0.55f, 1.0f );
	//for texture use only -> material
	//glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	//glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

#define tmin (0.0 + 0.5/256.0)  // this will remove (most of) visible edges 
#define tmax (1.0 - 0.5/256.0)  // in skybox

	//front->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmin, tmin );
	glVertex3f(  a,  a, -a );
	glTexCoord2f( tmax, tmin);
	glVertex3f( -a,  a, -a );
	glTexCoord2f( tmax, tmax );
	glVertex3f( -a, -a, -a );
	glTexCoord2f( tmin, tmax );
	glVertex3f(  a, -a, -a );
	glEnd();

	//back->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmin, tmin );
	glVertex3f( -a,  a, a );
	glTexCoord2f( tmax, tmin );
	glVertex3f(  a,  a, a );
	glTexCoord2f( tmax, tmax );
	glVertex3f(  a, -a, a );
	glTexCoord2f( tmin, tmax );
	glVertex3f( -a, -a, a );
	glEnd();

	//right->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmin, tmin );
	glVertex3f( -a,  a,  -a );
	glTexCoord2f( tmax, tmin );
	glVertex3f( -a,  a,   a );
	glTexCoord2f( tmax, tmax );
	glVertex3f( -a, -a,  a );
	glTexCoord2f( tmin, tmax );
	glVertex3f( -a, -a, -a );
	glEnd();

	//left->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmin, tmin );
	glVertex3f( a,  a,   a );
	glTexCoord2f( tmax, tmin );
	glVertex3f( a,  a,  -a );
	glTexCoord2f( tmax, tmax );
	glVertex3f( a, -a,  -a );
	glTexCoord2f( tmin, tmax );
	glVertex3f( a, -a,   a );
	glEnd();

	//up->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmax, tmin );
	glVertex3f( -a,  a,   a );
	glTexCoord2f( tmax, tmax );
	glVertex3f( -a,  a,  -a );
	glTexCoord2f( tmin, tmax );
	glVertex3f(  a,  a,  -a );
	glTexCoord2f( tmin, tmin );
	glVertex3f(  a,  a,   a );
	glEnd();

	//down->glApply();
	glBegin( GL_QUADS );
	glTexCoord2f( tmax, tmin );
	glVertex3f( -a,  -a,   a );
	glTexCoord2f( tmax, tmax );
	glVertex3f( -a,  -a,  -a );
	glTexCoord2f( tmin, tmax );
	glVertex3f(  a,  -a,  -a );
	glTexCoord2f( tmin, tmin );
	glVertex3f(  a,  -a,   a );
	glEnd();

	glPopAttrib();
}


};  //  namespace Models


