
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "SimulationTimer.h"
#include "Physics/System.h"
#include "SysSupport/Timer.h"
#include <iostream>
using namespace Physics;
using namespace std;


static Timer sync;
static float sys_time  = 0.0f;
static float frame_age = 0.0f;


/*static*/ Uint32 SimulationTimer::callb( Uint32 interval, void *param ){
	System *s = (System*)(param);

	frame_age = sync.Passed();
	sys_time += frame_age;

	s->update();

	//  Update timer
	sync.Update();

/*	cout << "SIMULATION " << sys_time << endl;
	cout.flush();*/

	return interval;
}


#if 0
#include "InputTask.h"
#include "Graphics/OpenGL.h"
#include "ui.h"


	char keys[256];
	RigidBody *camera;
	System    *system;
	int        flight_mode;

#define PITCH_CONST (5.0)
#define ROLL_CONST  (5.0)
#define PITCH_DAMP  (0.03)
#define ROLL_DAMP   (0.03)
#define ACCEL_CONST (0.08)
#define DECEL_CONST (0.08)
#define MAX_SPEED   (100.00)

#define UP       0
#define DOWN     1
#define FM_ELITE 0
#define FM_FREE	 1

double pitch   = 0.0f;
double roll    = 1.0f;
double speed   = 6.0f;
bool   wait_up = false;
bool   touch   = false;


//!  Constructor - clear key array
InputTask::InputTask( System *system, RigidBody *camera ):Task("User Interface Task"){
	this->system = system;
	this->camera = camera;
	for( int i=0; i<256; i++ ){
		keys[i] = UP;
	}
	flight_mode = FM_ELITE;
}


//!  Destructor - unimplemented
InputTask::~InputTask(){
	//  FIX
}


//!  Notify input task about key down message
void InputTask::keyDown( SDL_keysym key ){
	char buffer[100];

	touch = true;	
	sprintf( buffer, "Key down scancode < %d > keysym [ %d ] unicode ( %d )", key.scancode, key.sym, key.unicode );
	*console << buffer;
	console->newLine();
	
	keys[key.sym] = DOWN;	

	if( key.sym == SDLK_1 ){
		flight_mode = FM_ELITE;
		speed = 0.0;              //  FIX Think what to do...
	}
	if( key.sym == SDLK_2 ){
		flight_mode = FM_FREE;
		speed = 0.0;              //  FIX Think what to do...
	}

	//  -----  Control Pitch and Roll -----
	if( key.sym == 'x' ){
		pitch = PITCH_CONST;
	}
	if( key.sym == 's' ){
		pitch = -PITCH_CONST;
	}
	if( key.sym == ',' ){
		roll = ROLL_CONST;
	}
	if( key.sym == '.' ){
		roll = -ROLL_CONST;
	}
	
}


//!  Notify input task about key up message
void InputTask::keyUp( SDL_keysym key ){
	keys[key.sym] = UP;
	wait_up = false;

	//  -----  Control Pitch and Roll -----
	if( key.sym == 'x' ){
		if( keys['s'] == DOWN ){
			pitch = -PITCH_CONST;
		}
	}
	if( key.sym == 's' ){
		if( keys['x'] == DOWN ){
			pitch = PITCH_CONST;
		}
	}
	if( key.sym == ',' ){
		if( keys['.'] == DOWN ){
			roll = -ROLL_CONST;
		}
	}
	if( key.sym == '.' ){
		if( keys[','] == DOWN ){
			roll = ROLL_CONST;
		}
	}

}


//!  Task interface
/*virtual*/ void InputTask::run(){
	Uint32 age = system->getFrameAge();

	//  -----  Control Pitch and Roll -----
	if( touch ){
		if( keys['x'] == UP &&
			keys['s'] == UP    ){
			if( pitch > PITCH_CONST/100 ){
				pitch -= PITCH_DAMP*age;
				if( pitch < PITCH_CONST/100 ){
					pitch = 0.0;
				}
			}else if( pitch < -PITCH_CONST/100 ){
				pitch += PITCH_DAMP*age;
				if( pitch > -PITCH_CONST/100 ){
					pitch = 0.0;
				}
			}
		}
		if( keys[','] == UP &&
			keys['.'] == UP    ){
			if( roll > ROLL_CONST/100 ){
				roll -= ROLL_DAMP*age;
				if( roll < ROLL_CONST/100 ){
					roll = 0.0;
				}
			}else if( roll < -ROLL_CONST/100 ){
				roll += ROLL_DAMP*age;
				if( roll > -ROLL_CONST/100 ){
					roll = 0.0;
				}
			}
		}
	}

	camera->getInstance()->pitch( pitch * age / 100.0 );
	camera->getInstance()->roll ( roll  * age / 100.0 );
	
	//  ----- Control Speed -----
	double old_speed = speed;
	if( keys[SDLK_SPACE] == DOWN && wait_up==false ){  //  Faster
		if( speed < MAX_SPEED ){
			speed += ACCEL_CONST * age;
			if( old_speed < 0 && speed > 0 ){
				speed   = 0;
				wait_up = true;
			}else if( speed > MAX_SPEED ){
				speed = MAX_SPEED;
			}
		}
	}
	if( keys[SDLK_TAB] == DOWN && wait_up==false ){  //  Slower
		if( speed > -MAX_SPEED ){
			speed -= DECEL_CONST * age;
			if( old_speed > 0 && speed < 0 ){
				speed   = 0;
				wait_up = true;
			} else if( speed< -MAX_SPEED ){
				speed = -MAX_SPEED;
			}
		}
	}
	if( keys[SDLK_ESCAPE] == DOWN ){
		speed = 0.0;
	}

	double scale = 1.0;
	if( keys['j'] == DOWN ){
		scale *= 100.0;
	}
	if( keys['u'] == DOWN ){
		scale *= 100000.0;
	}

	//  Fully controlled 'Elite' flight mode - go where nose points...

	if( flight_mode == FM_ELITE ){
		camera->tick_translation = camera->getInstance()->getLook() * -speed*scale;
	}else{
		camera->tick_translation_delta = camera->getInstance()->getLook() * -speed*scale;
	}
}


#endif


