
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta, Sean O' Neil

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\file
	\class   CMemoryBlock
	\ingroup g_sys_support
	\author  Sean O'Neil
	\brief   Memory Management class
	\date    2000
*/


#ifndef TEDDY_SYS_SUPPORT_C_MEMORY_BLOCK_H
#define TEDDY_SYS_SUPPORT_C_MEMORY_BLOCK_H


#include <string.h>


namespace SysSupport {


//#define RELEASE(a){ if(a) { a->Release(); a = NULL; } }


class CMemoryBlock {
public:
	CMemoryBlock(){
		m_nElements     = 0;
		m_nElementSize  = 0;
		m_pBuffer       = NULL;
		m_pFreeStack    = NULL;
		m_nFreePosition = 0;
	}

	CMemoryBlock( int nElements, int nElementSize ){
		m_pBuffer    = NULL;
		m_pFreeStack = NULL;
		Init( nElements, nElementSize );
	}

	~CMemoryBlock(){
		Cleanup();
	}

	void Cleanup(){
		if( m_pFreeStack ){
			delete (char*)(m_pFreeStack);
			m_pFreeStack = NULL;
		}

		if( m_pBuffer ){
			delete (char*)(m_pBuffer);
			m_pBuffer = NULL;
		}
	}

	void Init( int nElements, int nElementSize ){
		Cleanup();
		m_nElements    = nElements;
		m_nElementSize = nElementSize;
		m_pBuffer      = new char[nElements * nElementSize];
		//		TRACE("CMemoryBlock::Init() allocating %d bytes.\n", nElements * nElementSize);
		m_pFreeStack   = new void *[nElements];
		for( m_nFreePosition = 0; m_nFreePosition < nElements; m_nFreePosition++ ){
			m_pFreeStack[m_nFreePosition] = (void *)( (int)m_pBuffer + m_nFreePosition * nElementSize );
		}
	}

	int GetElementSize     (){ return m_nElementSize;  }
	int GetFreeElementCount(){ return m_nFreePosition; }

	void *GetElement(){
		void *pBlock = NULL;
		if( m_nFreePosition ){
			pBlock = m_pFreeStack[--m_nFreePosition];
		}
		return pBlock;
	}

	void ReleaseElement( void *pBlock ){
		m_pFreeStack[m_nFreePosition++] = pBlock;
	}

protected:
	int    m_nElements;
	int    m_nElementSize;
	void  *m_pBuffer;
	void **m_pFreeStack;
	int    m_nFreePosition;
};


};  //  namespace SysSupport


#endif  //  TEDDY_SYS_SUPPORT_C_MEMORY_BLOCK_H

