
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#include "SysSupport/FileScan.h"


#if defined( WIN32 )  //  PLATFORM SELECTION - WIN32


#include <windows.h>


FileScan::FileScan( const char *mask ){
	WIN32_FIND_DATA fData;
	HANDLE          search;

	search = FindFirstFile( mask, &fData );

	if( search == INVALID_HANDLE_VALUE ){
		return;
	}

	do{
		files.push_back(string(fData.cFileName));
	}while( FindNextFile( search, &fData ) );
}


#elif 1  // PLATFORM SELECTION - ASSUME POSIX

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <glob.h>


// FIXME: this could use some better error handling...

FileScan::FileScan( const char *pattern ) : files() {
	glob_t g;
	int r = glob(
				 pattern,
				 GLOB_MARK
#ifdef GLOB_TILDE
				 |GLOB_TILDE
#endif
				 ,NULL,
				 &g
				);
	if( r==GLOB_NOMATCH ){
		return;
	}
	if( r!=0 ){
		fprintf(
			stderr,
			"glob(3) failed for \"%s\": %s\n",
			pattern,
			strerror(errno)
		);
		exit( EXIT_FAILURE );
	}
	for( int i=0; (long)(i)<(long)(g.gl_pathc); i++ ){
		files.push_back( string(g.gl_pathv[i]) );
	}
	globfree( &g );
}


#else


#error "Target has no implementation for FileScan class"


#endif  // PLATFORM_SELECTION

