
/*!
	\file
	\brief Initialization code for the polymessage MIDI playback app.
	
	THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR
	A PARTICULAR PURPOSE.
	
	Copyright (C) 1993 - 1997 Microsoft Corporation. All Rights Reserved.
*/


#pragma warning(disable:4756)
#include "MIDIPlyr.H"


static BOOL init           ();
static void deinit         ();
static void AttemptFileOpen( char *fname );
static BOOL PrerollAndWait ();


PSEQ gpSeq;
UINT guDevice = 0;


int mmain( int argc, char **argv ){
	int a;
/*	if( argc!=2 ){
		printf( "Usage: %1 file\n" );
		exit( 1 );
	}*/
	midi_init();                    // init();
	midi_open_file( "2001.mid" );   // AttemptFileOpen( argv[1] );
	midi_play();                    // Command( IDM_PLAY );
	a = fgetc( stdin );
	midi_stop();                    // seqStop( gpSeq );
	midi_cleanup();                 // deinit();
	return( 0 );
}


void midi_init(){
	init();
}

void midi_open_file( char *fname ){
	AttemptFileOpen( fname );
}


void midi_cleanup(){
	deinit();
}


void midi_play(){
	if( gpSeq->uState != SEQ_S_OPENED ){
		//DPF( 1, "IDM_PLAY: State %u", gpSeq->uState );
	}
				        	    
	if( PrerollAndWait() ){
		seqStart( gpSeq );
	}
}


void midi_stop(){
	seqStop( gpSeq );
}


void midi_pause_stop(){
	seqPause( gpSeq );
}


void midi_pause_play(){
	seqRestart( gpSeq );
}


/*
*/
BOOL init(){
    if ((gpSeq = (PSEQ)LocalAlloc(LPTR, sizeof(SEQ))) == NULL){
        return FALSE;
	}

    gpSeq->cBuffer  = C_MIDI_BUFFERS;
    gpSeq->cbBuffer = CB_MIDI_BUFFERS;

    if( seqAllocBuffers(gpSeq) != MMSYSERR_NOERROR ){
        return FALSE;
	}
	return TRUE;
}


static void deinit(){
    if( gpSeq != NULL ){
        seqFreeBuffers( gpSeq );
        LocalFree(( HLOCAL)gpSeq );
        gpSeq = NULL;
    }
}


/*!
	Stop and close the current file.
	Open the new file.
	Preroll the new file.
*/
static void AttemptFileOpen( char *fname ){
    MMRESULT mmrc;
    PSTR     pStrFile = "";
    
    /* Stop, close, etc. if we're still playing
    */
    //DPF(1, "AttemptFileOpen: Calling seqStop(); state = %u", gpSeq->uState);
    
    mmrc = seqStop( gpSeq );
	if( mmrc != MMSYSERR_NOERROR ){
		//Error(hWnd, IDS_STOPFAILED, mmrc);
		return;
	}

    //DPF(1, "Calling seqCloseFile(); state = %u", gpSeq->uState);
    seqCloseFile( gpSeq );

    //DPF(1, "Calling seqOpenFile(); state = %u", gpSeq->uState);
    /* Open new file
    */

    gpSeq->pstrFile = fname;
    mmrc = seqOpenFile( gpSeq );
    if( mmrc != MMSYSERR_NOERROR ){
//        Error(hWnd, IDS_OPENFAILED, mmrc);
        return;
    }

    pStrFile = fname;
}


/*!
	Prerolls the sequencer using the current device ID and file.
	Just call preroll and loop processing messages until done.
*/
static BOOL PrerollAndWait(){
    PREROLL  preroll;
    MMRESULT mmrc;
    
    preroll.tkBase   = 0;
    preroll.tkEnd    = gpSeq->tkLength;
    gpSeq->uDeviceID = guDevice;

    if( MMSYSERR_NOERROR != (mmrc = seqPreroll(gpSeq, &preroll)) ){
        // Error(hWnd, IDS_PREROLLFAILED, mmrc);
        return FALSE;
	}

	return TRUE;
}

