
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
 
 
\page p_program_flow Program Flow


At startup, a single instance of UI class is created. It creates
a number of instances of some classes. Some important instances are:

<ul>

<li>InputTask *input_task

This is a task, which does input processing. At the moment
input processing in Teddy is a real mess, so don't be alarmed
if you get confused. Will be cleared up later.

<li>Input *input

This class is theoretically responsible to get input from
system to Teddy. In practice, it does nothing, and reading
input is done in UI by using SDL calls. Should be fixed later.

<li>View *view

View is a single OpenGL window in host environment running
Teddy. Inside this View Teddy can manage multiple virtual
windows called Areas. Areas can be both three- and two-
dimensional. More precisely, Projection is subclass of
Area such that it can contain custom projection matrix,
while Area is plain two-dimensional ares.

<li>TestLayer *layer

Views' contents are organized into Layers. Layer is simple
subclass of Area such that it always covers the whole View.
Teddy layers remotely work like PhotoShop layers, except there
is nothing cool about Teddy layers yet..

Layers themselves are collections of Areas. Drawing a layer
is simple drawing Areas that below to that Layer. Similarly
View is drawn by drawing layers in it.

Any Area can listen to input messages, if they implement
input listener. You have to have a specific subclass of
Area that does this.

When teddy receives input message from host system, it is
processed by FocusManager. FocusManager maintains focus
for each input device. Focus is Area (implementing listener
for the specific input device) that will receive inputs. At
the moment there is fixed policy that focus can be changed
by clicking on top of Area. Will be fixed later to more
flexible and configurable policy.

TestLayer is subclass of Layer, also imlementing some input
listeners, so that input messages can be sent to it. It is
the root sink for all input in the test application that
receives all input if no other Area is interested.

<li>System *system

This is the physics system simulator. In order to have
something happening by itself, you have to create RigidBody
(better name maybe?) instance and tell your System instance
about it. The physics system periodically updates all RigidBodies
it knows about.

<li>Scene *scene

Scene is collection of Lights and ModelInstances, and is
what is drawn. Physics System and Scene are separeated,
thus you can have objects that are not simulated with
physics engine, but are drawn, and also objects that are
simulated with physics system, but not drawn. Drawback of
this scheme is that you have to add new instances usually
to both collections - that does not happen automatically
when you create an instance.

<li>Camera *camera

Camera is a tool which can be used to draw a scene. Scene
alone can not draw itself, as it requires an instance of
Camera to perform view volume culling.

At the moment camera also has skybox and starfield hacked
in. They are not very well designed, so please don't worry
too much about those.

Still, even Camera and Scene are not enough to render the
Scene. There needs to be some Area where to do the rendering.

<li>FrontCamera *front_camera

Although name suggest, FrontCamera class is not subclass of
Camera. It is subclass of Projection. In practice, it is a
3D Projection Area, such that it can render any Camera &
Scene.

<li>Sight *sight

Sight is example of derived Area. It is plain simple one.
It uses LayoutConstraint such that it will always be positioned
to the center of the parent Area wherever Sight instance is
added.

<li>Console *console

Console is work-in-progress console like thingy. It has basic
output, input and editing features, and some bugs, but no
advanced editing features like selection. To be improved.

<li>Hud *hud

Hud is another derived Area that displays some textual information.
It is much simpler than Console though.

<li>Scanner *scanner

Yet another primitive derived Area which displays rigidbodies in
system in radarlike array.

<li>Base *interpreter

Currently I have Kamin Scheme interpreter, which is very much
work-in-progress. Use at your own risk.

</ul>


*/

