
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


#if defined( WIN32 )
#pragma warning(disable:4786)
#endif

#if defined( WIN32 )
extern "C" {
#include "arch/win32/MIDIPLYR.H"
}
#endif

#include "FrontCamera.h"
#include "Hud.h"
#include "Sight.h"
#include "TestLayer.h"
#include "ui.h"

#include "Inputs/Input.h"
#include "Kamin/Scheme.h"
#include "Maths/Random.h"
#include "PhysicalComponents/Console.h"
#include "Physics/System.h"
#include "Graphics/OpenGL.h"
#include "Graphics/GlFeature.h"
#include "Graphics/View.h"
#include "ThreadMaster.h"
#include <iostream>
#include <list>
#include <stdio.h>
using namespace Graphics;
using namespace Inputs;
using namespace Maths;
using namespace Models;
using namespace PhysicalComponents;
using namespace std;


Console *console;  //  filescope, extern, for easy access from other files.. fix?


/*!
	Constructor the the testing user interface.
	Will be replaced later.
*/
UI::UI( bool fullscreen ){
	view        = new View     ( TITLE_STRING, fullscreen );   //!<  View must be the first thing to be created before other OpenGL related things
	system      = new System   ();                             //!<  System will hold physics simulation bodies
	interpreter = new Scheme   ();
	input       = new Input    ( view );
	scene       = new Scene    ( "Test scene" );               //!<  Scene will hold renderable objects - camera is window to scene
	camera      = new Camera   ( "Front View", NULL, scene );
	crb         = new RigidBody( camera );                     //!<  RigidBody to carry camera
	layer       = new TestLayer( this, view );
	console     = new Console  ( 80, 20, interpreter );
	instance    = NULL;
	mouse_mode  = MM_CONTROL_CAMERA;
	rec_depth   = 0;

	FocusManager::setFocus( layer, layer );
	displayHelp();
	camera->translate( 0, SPACE*5, SPACE*20 );
	view  ->insert( layer );

#ifdef USE_WINDOW_SYSTEM
	layer ->insert( console );
#endif
	layer ->place ();
	view->display();

#if defined( WIN32 )
	midi_init();
	atexit( midi_cleanup );
	midi_open_file( "2001.mid" );
	midi_play();
	atexit( midi_stop );
#endif

	//  Set defaults and fix up few things
	GlFeature::depth_test.masterDisable();  GlFeature::depth_test.apply( true );
	GlFeature::cull_face .masterEnable();   GlFeature::cull_face .apply( true );
	GlFeature::lighting  .masterDisable();  GlFeature::lighting  .apply( true );

	GlFeature::blend     .masterEnable();  GlFeature::blend     .apply( true );
	view->setBlendFunc( GL_ONE, GL_ONE );
	// glColor4f( 0.3f, 0.3f, 1.0f, 1.0f );  nice blue grid

	//  ----- Set default material properties if they are not being used -----
	glColor4f( 0.5f, 0.6f, 0.7f, 1.0f );
	Color ambient   = Color::GRAY_25;
	Color diffuse   = Color::GRAY_50;
	Color specular  = Color::GRAY_50;
	Color emission  = Color::BLACK;
	float shininess = 10.0f;
	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient .rgba );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse .rgba );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular.rgba );
	glMaterialfv( GL_FRONT, GL_EMISSION,  emission.rgba );
	glMaterialf ( GL_FRONT, GL_SHININESS, shininess );

	view->setPolygonMode ( GL_FILL );
	view->setShademodel  ( GL_SMOOTH );
	scene->setSort       ( false );
	system->addRigidBody ( crb );
	glLightModeli( GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE );
	glLightModeli( GL_LIGHT_MODEL_TWO_SIDE,     GL_TRUE );
	glCullFace( GL_BACK );
	glFrontFace( GL_CW );

	SDL_EnableUNICODE( 1 );
	SDL_EnableKeyRepeat( SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL );

	//  Create physical components for user interface
	front_camera = new FrontCamera( camera );
	hud          = new Hud        ( camera, system );
	scanner      = new Scanner    ( camera, system );
	sight        = new Sight      ();

	//layer->insert( scanner );
	//front_camera->insert( sight );
	layer->insert( front_camera );
	layer->insert( hud );
	layer->place();
	addGrid();
    view->display();

	loadLWO();
	addLights( 1, false );
    view->display();

	scene->update( front_camera );
	instanceCycle();

	ThreadMaster *tm = new ThreadMaster( view, system );
	input->inputLoop();
}

