
/*
    TEDDY - General graphics application library
    Copyright (C) 1999, 2000  Timo Suoranta

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Timo Suoranta
    tksuoran@cc.helsinki.fi
*/


/*!
	\class   UI
	\ingroup g_testing_environment
	\author  Timo Suoranta
	\brief   UserInterface
	\warning This is the temprorary testing environment only.
	\date    1999, 2000
	
	This class contains the application logic for the testing environment.
*/


#ifndef TEDDY_TEST_UI_H
#define TEDDY_TEST_UI_H


#include "Graphics/View.h"
#include "Models/Mesh.h"
#include "Models/ModelInstance.h"
#include "Inputs/Input.h"
#include "Physics/RigidBody.h"
#include "Physics/System.h"
#include "PhysicalComponents/Console.h"
#include "PhysicalComponents/Layer.h"
#include "Scenes/Scene.h"
#include "Scenes/Camera.h"

#include "TestLayer.h"
#include "FrontCamera.h"
#include "Sight.h"
#include "Hud.h"
#include "Scanner.h"

#include <list>
using namespace Graphics;
using namespace Inputs;
using namespace Models;
using namespace PhysicalComponents;
using namespace std;


#define TITLE_STRING              "glElite Version 1.33"
#define MM_CONTROL_CAMERA         0
#define MM_TRANSLATE_INSTANCE     1
#define MM_ROTATE_SCALE_INSTANCE  2
#define SPACE                     150



class UI {
public:
	UI( bool fullscreen );

	void cameraRotate       ( const int x, const int y );  // mouse move delta
	void cameraTranslate    ( const int x, const int y );  // mouse move delta
	void instanceRotate     ( const int x, const int y );  // mouse move delta
	void instanceScale      ( const int x, const int y );  // mouse move delta
	void instanceTranslateXZ( const int x, const int y );  // mouse move delta
	void instanceTranslateYZ( const int x, const int y );  // mouse move delta
	
	//  Action logical components FIX (no registering, no mapping...)
	void mouseMotion         ( const int b, const int x_delta, const int y_delta );
	void selectInstance      ( const int x, const int y );
	void displayHelp         ();  
	void displayExtensions   ();
	void instanceFace        ();
	void instanceCycle       ();
	void scannerCycle        ();
	void polygonModeLine     ();
	void polygonModeFill     ();
	void cullFaceEnable      ();
	void cullFaceDisable     ();
	void depthTestEnable     ();
	void depthTestDisable    ();
	void lightingOn          ();
	void lightingOff         ();
	void fovNormal           ();
	void fovWide             ();
	void antialiseOn         ();
	void antialiseOff        ();
	void chooseMouseMode     ();
	void quit                ();
	
protected:
	System        *system;       //!<  The System
	Input         *input;
	View          *view;
	Scene         *scene;
	Camera        *camera;
	TestLayer     *layer;
	FrontCamera   *front_camera;
	Sight         *sight;
	Hud           *hud;
	Scanner       *scanner;
	ModelInstance *instance;
	Base          *interpreter;
	RigidBody     *crb;
	
	int  mouse_mode;
	int  mouse_x[3];
	int  mouse_y[3];
	int  mouse_b[3];
	int  rec_depth;

	//  Routines to add things to scene
	void loadLWO  ();
	void addGrid  ();
	void addROAM  ();
	void addLights( int num=1, const bool animate=false );

	//  FIX
	void inputLoop();
};


extern Console *console;


#endif  //  TEDDY_TEST_UI_H


