
#include "ui.h"


#define TRANSLATE_SCALE           0.0001
#define distance_delta            0.01f
#define SCALE                     3.00f


//!  Exit program
void UI::quit(){
	exit( 0 );
}


void UI::chooseMouseMode(){
	mouse_mode++;
	if( mouse_mode>2 ){
		mouse_mode = 0;
	}
	switch( mouse_mode ){
	case MM_CONTROL_CAMERA:
		*console << "Drag Left Mouse Buttom down: Rotate camera";console->newLine();
		*console << "Drag Right Mouse Button down: Move forward/backward";console->newLine();
		break;
	case MM_TRANSLATE_INSTANCE:
		*console << "Drag Left Mouse Buttom down: Translate object X, Z";console->newLine();
		*console << "Drag Right Mouse Button down: Translate object X, Y";console->newLine();
		break;
	case MM_ROTATE_SCALE_INSTANCE:
		*console << "Drag Left Mouse Buttom down: Rotate object X, Y";console->newLine();
		*console << "Drag Right Mouse Button down: Rotate object Z, Scale object";console->newLine();
		break;
	default:
		break;
	}
}


//!  Interface: Displace help message in console
void UI::displayHelp(){
	*console << (char *)glGetString( GL_VENDOR   ); console->newLine();
	*console << (char *)glGetString( GL_RENDERER ); console->newLine();
	*console << (char *)glGetString( GL_VERSION  ); console->newLine(); console->newLine();
	*console << TITLE_STRING; console->newLine();
	*console << "by Timo Suoranta and Sean O'Neil"; console->newLine();
	*console << "--------------------------------"; console->newLine();

	*console << "(set add10 ((lambda (x) (lambda  (y) (+ x y) ) ) 10 ))"; console->newLine();
	*console << "Starting up..."; console->newLine();
}


//!  Display OpenGL extensions
void UI::displayExtensions(){
	char *ext_c = (char *)glGetString( GL_EXTENSIONS );
	char *ext = new char[strlen(ext_c)];
	strcpy( ext, ext_c );
	char *i;

	char *start = i = ext;
	while( *i != '\0' ){
		if( *i==' ' ){
			*i = '\0';
			*console << start;
			console->newLine();
			start = ++i;
			continue;
		}
		i++;
	}
}
		

//!  mouse motion
void UI::mouseMotion( const int b, const int x_delta, const int y_delta ){
	switch( mouse_mode ){
	case MM_CONTROL_CAMERA: 
		switch( b ){
		case 1: cameraRotate   ( x_delta, y_delta ); break;
		case 2: cameraTranslate( x_delta, y_delta ); break;
		case 3: cameraTranslate( x_delta, y_delta ); break;
		default: break;
		}
		break;
	case MM_TRANSLATE_INSTANCE:
		switch( b ){
		case 1: instanceTranslateXZ( x_delta, y_delta ); break;
		case 2: instanceTranslateYZ( x_delta, y_delta ); break;
		case 3: instanceTranslateYZ( x_delta, y_delta ); break;
		default: break;
		}
		break;
	case MM_ROTATE_SCALE_INSTANCE:
		switch( b ){
		case 1: instanceRotate( x_delta, y_delta ); break;
		case 2: instanceScale ( x_delta, y_delta ); break;
		case 3: instanceScale ( x_delta, y_delta ); break;
		default: break;
		}
		break;
	default:
		break;
	}
}


//!  translate camera
void UI::cameraRotate( const int x_delta, const int y_delta ){
	camera->roll( x_delta*0.4 );
	camera->pitch( y_delta*-0.4 );
}


//!  rotate camera
void UI::cameraTranslate( const int x_delta, const int y_delta ){
	camera->roll( x_delta*0.4 );
	camera->foward( y_delta*SCALE );
}


//!  translate instance on XZ plane
void UI::instanceTranslateXZ( const int x_delta, const int y_delta ){
	if( instance==NULL ){
		return;
	}

	Vector delta  = Vector( -x_delta, 0.0, -y_delta );
	Vector result = camera->transform( delta );
	
	instance->translate( result );
}


//!  translate instance on YZ plane
void UI::instanceTranslateYZ( const int x_delta, const int y_delta ){
	if( instance==NULL ){
		return;
	}
	Vector delta  = Vector( -x_delta, y_delta, 0.0 );
	Vector result = camera->transform( delta );
	instance->translate( result );
}


//!  rotate instance
void UI::instanceRotate( const int x_delta, const int y_delta ){
	if( instance==NULL ){
		return;
	}
	instance->rotate(  (x_delta*0.3)/180.0*3.1516, Vector(0,0,0), Vector(0.0,0.0,1.0) );
	instance->rotate( -(y_delta*0.3)/180.0*3.1516, Vector(0,0,0), Vector(0.0,1.0,0.0) );
}


//!  scale instance
void UI::instanceScale( const int x_delta, const int y_delta ){
	if( instance==NULL ){
		return;
	}
	
	double dist = -0.3*y_delta;
	if( dist<0 ){
		dist = 1/sqrt(1-(dist/10));
	}else{
		dist = sqrt(1+(dist/10));
	}
	
	instance->rotate( (x_delta*0.3)/180.0*3.1516, Vector(0,0,0), Vector(0.0,1.0,0.0) );
	instance->scale( dist );
}


//!  Mouse buttonpress event
void UI::selectInstance( const int x, const int y ){
	static char sel_line[80];
	ModelInstance *pick = camera->pickInstance( front_camera, x, y );
	if( pick!=NULL ){
		instance = pick;
		hud->setTarget( instance );
		double d = instance->distanceTo( camera->getLoc() );
		sprintf(
			sel_line,
			"Target: %s Distance: %.3f",
			(pick->getName()).c_str(),
			d
		);
		*console << sel_line;
		console->newLine();
	}
}


void UI::instanceFace(){
	*console << "Instance Face";
	console->newLine();
	if( instance!=NULL ){
		camera->face( instance->getLoc() );
	}
}


void UI::instanceCycle(){
	*console << "Instance Cycle";
	console->newLine();
	list<ModelInstance*> &instances     = scene->getInstances();
	list<ModelInstance*>::iterator i_it = instances.begin();
	while( i_it != instances.end() ){
		if( (*i_it) == instance ){
			i_it++;
			if( i_it != instances.end() ){
				instance = *i_it;
			}else{
				instance = *(instances.begin());
			}
			break;			
		}
		i_it++;
	}

	hud->setTarget( instance );
}


void UI::scannerCycle(){
	*console << "Scanner Cycle Range";
	console->newLine();
	scanner->cycle();
}
