
#include "ui.h"
#include "Imports/LWMesh.h"
#include "Materials/Light.h"
#include "Models/CSphere.h"
#include "Models/Grid.h"
#include "Models/Icosahedron.h"
#include "Models/Mesh.h"
#include "Models/Sphere.h"
#include "Models/Torus.h"
#include "Models/Triangle.h"
#include "SysSupport/FileScan.h"
using namespace Imports;
using namespace Models;
using namespace SysSupport;


//!  Test ROAM object
void UI::addROAM(){
	Material *mat;

	mat = new Material( "", true, GL_SMOOTH );
	mat->setShininess( 1.4f );
	mat->setAmbient  ( Color( 31.0f/255.0f, 94.0f/255.0f,125.0f/255.0f) );
	mat->setDiffuse  ( Color( 31.0f/255.0f, 94.0f/255.0f,125.0f/255.0f) );
	mat->setSpecular ( Color(0.90f,0.90f,1.00f) );

	CSphere *roam = new CSphere();
	roam->Init( 19082000, 100.0f, 2.0f, 0.1f );
	roam->setMaterial( mat );
	ModelInstance *mi_roam = new ModelInstance(
		"Planet (ROAM)",
		roam,
		new Vector( 0.0, 0.0, 0.0 )
	);
	scene->addInstance( mi_roam );
}


//!  Add preset light to scene
void UI::addLights( int num, const bool animate ){
	Light     *light;
	RigidBody *lrb;

	light = new Light( "Light 1" );
		light->setAmbient ( Color(0.43f,0.43f,0.43f) );
		light->setDiffuse ( Color(0.83f,0.83f,0.89f) );
		light->setSpecular( Color(1.00f,1.00f,1.00f) );
		light->setPosition( Vector( 0.0, -1000.0, 0.0 ) );
		light->enable();
		scene->addLight( light );
		lrb = new RigidBody( light );
		if( animate ){
			lrb->tick_translation = Vector( 2.0, 0.0, 0.0 );
		}
		system->addRigidBody( lrb );

	if( num>1 ){
		light = new Light( "Light 2" );
		light->setAmbient ( Color(0.03f,0.03f,0.03f) );
		light->setDiffuse ( Color(0.23f,0.23f,0.27f) );
		light->setSpecular( Color(0.50f,0.50f,0.50f) );
		light->setPosition( Vector( 0.0, 1000000.0, 0.0 ) );
		light->enable();
		scene->addLight( light );
		lrb = new RigidBody( light );
		if( animate ){
			lrb->tick_translation = Vector( 0.0, -20.0, 0.0 );
		}
		system->addRigidBody( lrb );
	}

	if( num>2 ){
		light = new Light( "Light 3" );
		light->setAmbient ( Color(0.03f,0.03f,0.03f) );
		light->setDiffuse ( Color(0.42f,0.43f,0.33f) );
		light->setSpecular( Color(0.47f,0.33f,0.33f) );
		light->setPosition( Vector( 0.0, 0.0, -1000.0 ) );
		light->enable();
		scene->addLight( light );
		lrb = new RigidBody( light );
		if( animate ){
			lrb->tick_translation = Vector( -2.0, -1.0, 10.0 );
		}
		system->addRigidBody( lrb );
	}

	if( num>3 ){
		light = new Light( "Light 4" );
		light->setAmbient ( Color(0.00f,0.00f,0.00f) );
		light->setDiffuse ( Color(0.30f,0.30f,0.30f) );
		light->setSpecular( Color(0.30f,0.30f,0.30f) );
		light->setPosition( Vector( -240000.0, -1000.0, -100.0 ) );
		light->enable();
		scene->addLight( light );
		lrb = new RigidBody( light );
		if( animate ){
			lrb->tick_translation = Vector( 40.0, 20.0, 1.0 );
		}
		system->addRigidBody( lrb );
	}
}
		

/*!
	LightWave Object scanner and loader
	Messy - FileScan class currently works differently on
	different platforms..
*/
void UI::loadLWO(){
	char       buf[256];
	double     cursor = -400.0;
	RigidBody *rb;
	int        i;

	FileScan lwos( "*lwo" );
	sprintf( buf, "Found %d lightwave objects", lwos.get_files().size() );
	*console << "Scanning Lightwave files in current directory..."; console->newLine();
	*console << buf; console->newLine();
	view->display();

	list<string>::iterator f_it = lwos.get_files().begin();
	char *fname;

	//  -----  Read In Files -----
	int     num           = lwos.get_files().size();
	int     per_line      = (int)(sqrt( num ));
	int     lines         = num/per_line+1;
	double  line_z[1000];
	double  line_x[1000];
	Mesh   *lwom[1000];
	int     count         = 0;
	int     cur_line      = 0;
	int     items_on_line = 0;

	//  Initialize each line to width 0
	for( i=0; i<lines; i++ ){
		line_z[i] = 0;
		line_x[i] = 0;
	}
	for( i=0; i<num; i++ ){
		lwom[i] = NULL;
	}

	double total_z = 0;

	//  Read in all files and slice them to lines
	while( f_it != lwos.get_files().end() ){
		fname = new char[ (*f_it).size()+5 ];
		sprintf( fname, "%s",  (*f_it).c_str() );
		lwom[count] = new LWMesh( fname );
		double ob_size = lwom[count]->getClipRadius();
		count++;
		items_on_line++;

		//  Line full enough?
		if( items_on_line > per_line ){
			line_z[cur_line] += SPACE;  //  Add some space
			total_z += line_z[cur_line];
			cur_line++;
			items_on_line = 0;
		}

		//  Make sure line is wide enough
		line_x[cur_line] += ob_size + SPACE;  //  Add some space
		if( line_z[cur_line] < ob_size ){
			line_z[cur_line] = ob_size;
		}
		*console << "Processed file " << fname; view->display();
		console->newLine();
		f_it++;
	}
	total_z += line_z[cur_line];


	//  Place all lines so that:
	//   - Each line has own Z, centered on X
	//   - All lines together are centered in Z
	cur_line        = 0;
	items_on_line   = 0;
	double z_offset = -total_z / 2.0;
	double x_offset = -line_x[cur_line] / 2.0;
	z_offset += line_z[cur_line]/2;
	for( i=0; i<num; i++ ){
		if( lwom[i] == NULL ){
			cerr << "NULL Mesh" << endl;
			cerr.flush();
			continue;
		}
		ModelInstance *mi = new ModelInstance(
			"",
			lwom[i],
			new Vector( x_offset, SPACE, z_offset )
		);
		rb = new RigidBody( mi );

		switch( i %3 ){
		case 0: rb->tick_rotation = Vector( 0.007, 0.005, 0.003 ); break;
		case 1: rb->tick_rotation = Vector(-0.003,-0.007,-0.005 ); break;
		case 2: rb->tick_rotation = Vector( 0.005, 0.003, 0.007 ); break;
		default: break;
		}

		scene->addInstance( mi );
		system->addRigidBody( rb );

		x_offset += lwom[i]->getClipRadius() + SPACE;
		items_on_line++;
		if( items_on_line > per_line ){
			z_offset += line_z[cur_line]/2;
			cur_line++;
			z_offset += line_z[cur_line]/2;
			items_on_line = 0;
			x_offset = -line_x[cur_line] / 2.0;
		}

	}

	*console << "Done."; console->newLine();
	view->display();
}


//!  Add simple grid to scene
void UI::addGrid(){
	Material *m = new Material;
	m->disableFlags( USE_GL_LIGHTING );
	m->setDiffuse( Color(0.2f,0.4f,0.3f) );
	ModelInstance *mi = new ModelInstance(
		"Grid",
		new Grid( 10, 10, 500, 500 ),
		new Vector( 0.0, 0.0f, 0.0 )
	);
	mi->getMesh()->setMaterial( m );
	scene->addInstance( mi );
	system->addRigidBody( new RigidBody(mi) );
}

