//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//


#include "bezier.h"
#include "beziertessaltion.h"
#include "scene.h"
#include <math.h>
#include <stdlib.h>

CBezierPatch::CBezierPatch(CBezierSpline *spline,DWORD *indices)
{
	DWORD i,j;
	for (i=0;i<4;i++) {
		for (j=0;j<4;j++) {
			m_indices[i][j] = *indices;
			indices++;
		}
	}
	m_spline = spline;

}

CBezierPatch::~CBezierPatch(){
}



bool CBezierPatch::Render(CScene *scene ,LPDIRECT3DVERTEXBUFFER7 vb,LPWORD &indices,DWORD &indicesCount)
{
	float minZ;
	DWORD teslLevel;
	

	if (!EvalSurface(vb,scene,indices,indicesCount))
		return false;

	return true;
}



DWORD	CBezierPatch::GetTessalationOrder(float minZ) 
{
	return 10;
}






bool	CBezierPatch::EvalSurface(LPDIRECT3DVERTEXBUFFER7 vb,CScene *scene,LPWORD &indices,DWORD &indicesCount)
{
	
	D3DTLVERTEX *vtx;
	DWORD i,j,k,t;
	DWORD teslLevel = m_spline->GetTeslLevel();
	Vector4	*projCtrlPts = m_spline->GetProjCtrlPts();
	Vector4	*worldCtrlPts = m_spline->GetViewCtrlPts();
	Vector3 scaling = scene->GetScreenScaling();
	Vector3 trans = scene->GetScreenTranslation();
	CBezierTessalation::TessLevel *tessLevel = m_spline->m_tesl->GetTessalationLevel(teslLevel);
	float *coeffs = tessLevel->m_teslValues;
	DWORD coeffsSkip = tessLevel->m_teslValuesSkip;
	DWORD lightState = scene->GetLightingState();

	float x,y,z,w;
	float nxk,nyk,nzk;
	float nxt,nyt,nzt;
	float wx,wy,wz;

	vb->Lock(DDLOCK_WAIT,(LPVOID *)&vtx,NULL);
	


	for (i=0;i<teslLevel;i++) {
		for (j=0;j<teslLevel;j++) {
			x = y = z = w = 0.0f;
			nxk = nyk = nzk = 0.0f;
			nxt = nyt = nzt = 0.0f;
			wx = wy = wz = 0.0f;
			for (k=0;k<4;k++) {
				float kCoeff = coeffs[k + i*8];
				for (t=0;t<4;t++) {
					float tCoeff = coeffs[t + j*8];
					float ktCoeff = kCoeff * tCoeff;
					Vector4 *pt = &projCtrlPts[m_indices[k][t]];
					x += pt->x*ktCoeff;
					y += pt->y*ktCoeff;
					z += pt->z*ktCoeff;
					w += pt->w*ktCoeff;

					if (lightState) {
						Vector4 *ptw = &worldCtrlPts[m_indices[k][t]];
						
						wx += ptw->x*ktCoeff;
						wy += ptw->y*ktCoeff;
						wz += ptw->z*ktCoeff;

						nxk += ptw->x*tCoeff*coeffs[k + 4 + i*8];
						nyk += ptw->y*tCoeff*coeffs[k + 4 + i*8];
						nzk += ptw->z*tCoeff*coeffs[k + 4 + i*8];

						nxt += ptw->x*kCoeff*coeffs[t+4 + j*8];
						nyt += ptw->y*kCoeff*coeffs[t+4 + j*8];
						nzt += ptw->z*kCoeff*coeffs[t+4 + j*8];
					}

		
				}
			}
			
			vtx->rhw = w = 1.0f/w;
			vtx->sx = x*w*scaling.x + trans.x;
			vtx->sy = y*w*scaling.y + trans.y;
			vtx->sz = z*w*scaling.z + trans.z;

			
			
			if (lightState) {
				Vector3 norm;
				Vector3 color;
				Vector3 dir;
				float dot;

				color.x = 0.0;
				color.y = 0.0;
				color.z = 0.0;


				norm.x = nyk*nzt - nzk*nyt;
				norm.y = -(nxk*nzt - nzk*nxt);
				norm.z = nxk*nyt - nyk*nxt;

				float size = 1.0f/sqrt(norm.x*norm.x + norm.y*norm.y + norm.z*norm.z);
				float dir_size;

				for (k = 1,t = 0;t< 8; t++,k <<= 1) {
					if (lightState & k) {
						const CScene::Light& light = scene->GetLight(t);
						if (light.type == CScene::Light::DIRECTIONAL_LIGHT) {
							dir = light.norm;
							dir_size = 1.0f;
						} else {
							dir.x = -light.pos.x + wx;
							dir.y = -light.pos.y + wy;
							dir.z = -light.pos.z + wz;
							dir_size = 1.0f/sqrt(dir.x*dir.x + dir.y*dir.y + dir.z*dir.z);
						}
						dot = dir.x*norm.x + dir.y*norm.y + dir.z*norm.z;
						
						if (dot > 0.0f) {
							dot *= size * dir_size;
							color.x += dot*light.color.x;
							color.y += dot*light.color.y;
							color.z += dot*light.color.z;
						}
					}
				}

				int r = (int)(color.x * 255.0f);
				int g = (int)(color.y * 255.0f);
				int b = (int)(color.z * 255.0f);

				if (r > 255) r = 255; else if (r < 0) r = 0;
				if (g > 255) g = 255; else if (g < 0) g = 0;
				if (b > 255) b = 255; else if (b < 0) b = 0;

				vtx->color = 0xff000000 + (r << 16) + (g << 8) + b;
			} else {
				vtx->color = 0xffffffff;
			}
			vtx->specular = 0x0;
			vtx->tu = 0;
			vtx->tv = 0;
			vtx++;
		}
	}


	indices = tessLevel->m_indices;
	indicesCount = tessLevel->m_indicesCount;
	vb->Unlock();
	return true;
}
