//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#include "bezierSpline.h"
#include "bezier.h"
#include "scene.h"
#include <fstream.h>
#include "util.h"
#include <stdlib.h>
#include <math.h>

CBezierPatch *CBezierSpline::CreatePatch(DWORD *patchIndices) 
{
	return new CBezierPatch(this,patchIndices);
}



void CBezierSpline::SpecialEffect() {

/*
	int i;
	static float x = 0.0;
	for (i=0;i<m_ctrlPtsCnt;i++) {
		m_objCtrlPts[i].x = m_origObjCtrlPts[i].x + sin(x+m_origObjCtrlPts[i].z);
	}
	x += 0.05;
*/

}


void CBezierSpline::PrepearSpline(CScene *scene) 
{
	DWORD i,j;
	

	Matrix obj2proj;
	Matrix world2proj;

	scene->GetWorld2ProjMatrix(world2proj);
	MatrixMultiply(obj2proj,world2proj,m_obj2world);

	for (i=0;i<m_ctrlPtsCnt;i++) {
		Vector3 &vSrc = m_objCtrlPts[i];
		m_projCtrlPts[i].x = (vSrc.x*obj2proj._11 + vSrc.y*obj2proj._21 + vSrc.z* obj2proj._31 + obj2proj._41);
		m_projCtrlPts[i].y = (vSrc.x*obj2proj._12 + vSrc.y*obj2proj._22 + vSrc.z* obj2proj._32 + obj2proj._42);
		m_projCtrlPts[i].z = (vSrc.x*obj2proj._13 + vSrc.y*obj2proj._23 + vSrc.z* obj2proj._33 + obj2proj._43);
		m_projCtrlPts[i].w = (vSrc.x*obj2proj._14 + vSrc.y*obj2proj._24 + vSrc.z* obj2proj._34 + obj2proj._44);

		m_viewCtrlPts[i].x = (vSrc.x*m_obj2world._11 + vSrc.y*m_obj2world._21 + vSrc.z* m_obj2world._31 + m_obj2world._41);
		m_viewCtrlPts[i].y = (vSrc.x*m_obj2world._12 + vSrc.y*m_obj2world._22 + vSrc.z* m_obj2world._32 + m_obj2world._42);
		m_viewCtrlPts[i].z = (vSrc.x*m_obj2world._13 + vSrc.y*m_obj2world._23 + vSrc.z* m_obj2world._33 + m_obj2world._43);
	
	}

}

CBezierSpline::CBezierSpline(CBezierTessalation *tesl) {
	m_ctrlPtsCnt = 0;
	m_objCtrlPts = NULL;
	m_origObjCtrlPts = NULL;
	m_projCtrlPts = NULL;
	m_patches = NULL;
	m_patchCnt = NULL;
	m_tesl = tesl;
	m_viewCtrlPts = NULL;
}

CBezierSpline::~CBezierSpline() {
}


void	CBezierSpline::SetObj2WorldMatrix(const Matrix mat) {
	m_obj2world = mat;
}

BOOL	CBezierSpline::EvalPatch(	DWORD patch,
									LPDIRECT3DVERTEXBUFFER7 vb,
									CScene *scene,
									LPWORD &indices, 
									DWORD &indicesCnt) {
	if (patch > m_patchCnt)
		return false;

	if (patch == 0)
		PrepearSpline(scene);
	return m_patches[patch]->Render(scene,vb,indices,indicesCnt);
}

void	CBezierSpline::CreateSplineFromFile(const char *filename) {
	DWORD i,j;
	DWORD indices[16];
	ifstream f;
	f.open(filename);

	f >> m_ctrlPtsCnt;
	m_objCtrlPts = (Vector3 *)alignedAlloc(sizeof(Vector3)*m_ctrlPtsCnt);
	m_origObjCtrlPts = (Vector3 *)alignedAlloc(sizeof(Vector3)*m_ctrlPtsCnt);
	m_viewCtrlPts = (Vector4 *)alignedAlloc(sizeof(Vector4)*m_ctrlPtsCnt);
	m_projCtrlPts = (Vector4 *)alignedAlloc(sizeof(Vector4)*m_ctrlPtsCnt);

	for (i=0;i<m_ctrlPtsCnt;i++) {
		f >> m_objCtrlPts[i].x;
		f >> m_objCtrlPts[i].y;
		f >> m_objCtrlPts[i].z;
		m_origObjCtrlPts[i] = m_objCtrlPts[i];
	}

	f >> m_patchCnt;
	m_patches = (CBezierPatch **)new DWORD[m_patchCnt];
	for (i=0;i<m_patchCnt;i++) {
		for (j=0;j<16;j++) {
			f >> indices[j];
		}
		m_patches[i] = CreatePatch(indices);
	}

}


