//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#ifndef _BEZIER_SPLINE_H
#define _BEZIER_SPLINE_H

#include <d3d.h>

class CBezierPatch;
class CBezierTessalation;
class CScene;
typedef D3DVECTOR Vector3;
typedef D3DMATRIX Matrix;


struct Vector4 {
	float w,x,y,z;

	friend Vector4 operator - (Vector4& a,Vector4& b) {
		Vector4 t;
		t.x = a.x - b.x;
		t.y = a.y - b.y;
		t.z = a.z - b.z;
		t.w = a.w - b.w;
		return t;
	}
};


class CBezierSpline {
private:
	Vector3				*m_objCtrlPts;
	Vector3				*m_origObjCtrlPts;
	Vector4				*m_viewCtrlPts;
	Vector4				*m_projCtrlPts;
	CBezierPatch		**m_patches;
	CBezierTessalation	*m_tesl;
	DWORD				m_ctrlPtsCnt;
	DWORD				m_patchCnt;
	DWORD				m_teslLevel;
	Matrix				m_obj2world;


protected:
	virtual void SpecialEffect();
	virtual CBezierPatch *CreatePatch(DWORD *patchIndices);
	virtual void PrepearSpline(CScene *);

public:
	CBezierSpline(CBezierTessalation *tesl);
	virtual ~CBezierSpline();

	DWORD	GetPatchCount() {return m_patchCnt;}

	CBezierTessalation	*GetBezierTessalation(){return m_tesl;}

	void	SetObj2WorldMatrix(const Matrix mat);
	
	BOOL	EvalPatch(	DWORD patch,
						LPDIRECT3DVERTEXBUFFER7 vb,
						CScene *scene,
						LPWORD &indices, 
						DWORD &indicesCnt);
	
	virtual void	CreateSplineFromFile(const char *filename);

	Vector4	*GetProjCtrlPts() {return m_projCtrlPts;}
	Vector4	*GetViewCtrlPts() {return m_viewCtrlPts;}
	DWORD	 GetCtrlPtsCount(){return m_ctrlPtsCnt;}
	void	 SetTeslLevel(DWORD level) {m_teslLevel = level;}
	DWORD	 GetTeslLevel() {return m_teslLevel;}

	friend class CBezierPatch;

};

#endif

