//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#include "beziertessaltion.h"
#include "util.h"


CBezierTessalation::CBezierTessalation() 
{
	ZeroMemory(m_tessLevels,sizeof(m_tessLevels));
}

CBezierTessalation::~CBezierTessalation() 
{
	DWORD i;
	for (i=0;i<MAX_TESSELATION_LEVEL;i++) {
		if (m_tessLevels[i])
			DestroyTessalationLevel(m_tessLevels[i]);
	}
}


void CBezierTessalation::GenerateTessalationLevel(TessLevel *level)
{
	DWORD i,j;
	DWORD teslLevel = level->m_level;
	DWORD triCnt = (teslLevel - 1)*(teslLevel - 1)*2;
	level->m_indicesCount = 3*triCnt;
	
	alignedFree(level->m_indices);
	alignedFree(level->m_teslValues);

	LPWORD idx = level->m_indices = (LPWORD)alignedAlloc(sizeof(WORD)*level->m_indicesCount);

	for (i=1;i<teslLevel;i++) {
		for (j=1;j<teslLevel;j++) {
			idx[1] = (i-1)*teslLevel + j;
			idx[0] = idx[1] - 1;
			idx[5] = idx[1] + teslLevel;
			idx[4] = idx[1];
			idx[2] = idx[5] - 1;
			idx[3] = idx[2];
			idx += 6;
		}
	}

	DWORD teslSkip = level->m_teslValuesSkip = (level->m_level + 3) & 0xffffffc;
	level->m_teslValues = (float *)alignedAlloc(sizeof(float)*teslSkip*8);

	float *coeffs = level->m_teslValues;

	coeffs[0] = 1.0f;
	coeffs[1] = 0.0f;
	coeffs[2] = 0.0f;
	coeffs[3] = 0.0f;
	coeffs[4] = -3.0f;
	coeffs[5] = 3.0f;
	coeffs[6] = 0.0f;
	coeffs[7] = 0.0f;

	teslLevel--;

	coeffs[0 + teslLevel*8] = 0.0f;
	coeffs[1 + teslLevel*8] = 0.0f;
	coeffs[2 + teslLevel*8] = 0.0f;
	coeffs[3 + teslLevel*8] = 1.0f;
	coeffs[4 + teslLevel*8] = 0.0f;
	coeffs[5 + teslLevel*8] = 0.0f;
	coeffs[6 + teslLevel*8] = -3.0f;
	coeffs[7 + teslLevel*8] = 3.0f;

	float skip = 1.0f/(float)(teslLevel);
	float t = skip;

	coeffs += 8;

	for (i=1;i<teslLevel;i++) {
		float one_minus_t = 1.0f - t;
		float factor = t/one_minus_t;
		float last;
		last = coeffs[0] = one_minus_t*one_minus_t*one_minus_t;
		last *= factor;
		coeffs[1] = last * 3.0f;
		last *= factor;
		coeffs[2] = last * 3.0f;
		coeffs[3] = last * factor;


		coeffs[4] = -3*one_minus_t*one_minus_t;
		coeffs[5] = 3*one_minus_t*one_minus_t - 6*t*one_minus_t;
		coeffs[6] = 6*t*one_minus_t - 3*t*t;
		coeffs[7] = 3*t*t;

		coeffs += 8;
		t += skip;
	}		

}

void CBezierTessalation::DestroyTessalationLevel(TessLevel *level) {
	alignedFree(level->m_indices);
	alignedFree(level->m_teslValues);
}

CBezierTessalation::TessLevel *CBezierTessalation::GetTessalationLevel(DWORD level) 
{
	if (level > MAX_TESSELATION_LEVEL)
		return NULL;
	if (m_tessLevels[level])
		return m_tessLevels[level];

	m_tessLevels[level] = new TessLevel;
	ZeroMemory(m_tessLevels[level],sizeof(TessLevel));
	m_tessLevels[level]->m_level = level;
	GenerateTessalationLevel(m_tessLevels[level]);
	return m_tessLevels[level];
}

