//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#define STRICT
#define D3D_OVERLOADS
#include <math.h>
#include "D3DApp.h"
#include "PIIIBezier.h"
#include "scene.h"
#include "util.h"
#include "resource.h"

class CMyD3DApplication : public CD3DApplication
{
	static HRESULT ConfirmDevice( DDCAPS* pddDriverCaps,
                                  D3DDEVICEDESC7* pd3dDeviceDesc );

	CBezierTessalation tesl;
	CPIIIBezierTessalation piii_tesl;
	CPIIIBezierTessalation2 piii_tesl2;
	CPIIIBezierTessalation3 piii_tesl3;
	CScene	m_scene;
	CBezierSpline *m_spl[5];
	DWORD	m_current;

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove( FLOAT fTimeKey );
    HRESULT FinalCleanup();
    LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

public:
    CMyD3DApplication();
	void	RenderCurrentSpline();
	void	SetTeslLevel(int newLevel) {for (int i=0;i<5;i++) m_spl[i]->SetTeslLevel(newLevel);}
    virtual ~CMyD3DApplication();
};





INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
{
    CMyD3DApplication d3dApp;

    if( FAILED( d3dApp.Create( hInst, strCmdLine ) ) )
        return 0;

    return d3dApp.Run();
}




CMyD3DApplication::CMyD3DApplication()
                  :CD3DApplication()
{
    m_strWindowTitle  = "Bezier surfaces";
    m_bAppUseZBuffer  = TRUE;
    m_bShowStats      = TRUE;
    m_fnConfirmDevice = ConfirmDevice;

	m_spl[0] = new CBezierSpline(&tesl);
	m_spl[1] = new CPIIIBezierSpline(&piii_tesl);
	m_spl[2] = new CPIIIBezierSpline2(&piii_tesl2);
	m_spl[3] = new CPIIIBezierSpline3(&piii_tesl3);
	m_spl[4] = new CPIIIBezierSpline4(&piii_tesl3);
}


CMyD3DApplication::~CMyD3DApplication() 
{
	int i;
	for (i=0;i<5;i++) delete m_spl[i];
}



HRESULT CMyD3DApplication::OneTimeSceneInit()
{
	D3DMATRIX m;
	m_current = 0;
	int i;
	for (i=0;i<5;i++) {
		m_spl[i]->CreateSplineFromFile("teapot.bpt");
		m_spl[i]->SetTeslLevel(6);
	}
	SetUnitMatrix(m);
	m._34 = 0.1;
	m_scene.SetProjectionMatrix(m);
	SetRotateXMatrix(m,-3.1415927/3.0);
	SetUnitMatrix(m);
	m_scene.SetViewMatrix(m);

	m_scene.SetScreenScalingAndTranslation(D3DVECTOR(45.0f,45.0f,0.05f),D3DVECTOR(150.0f,150.0f,0.5f));


	CScene::Light light;
	
	light.color = D3DVECTOR(1.0,0.0,0);
	light.pos = D3DVECTOR(0,10,-10);
	light.type = CScene::Light::POINT_LIGHT;
	m_scene.SetLightData(0,light);

	
	light.color = D3DVECTOR(0.0,0.0,1.0);
	light.pos = D3DVECTOR(-10,10,-10);
	light.type = CScene::Light::POINT_LIGHT;
	m_scene.SetLightData(1,light);

	light.color = D3DVECTOR(0.0,1.0,1);
	light.pos = D3DVECTOR(10,100,-10);
	light.type = CScene::Light::POINT_LIGHT;

	m_scene.SetLightData(2,light);

	m_scene.SetLightingState(0x7);


    return S_OK;
}




HRESULT CMyD3DApplication::FrameMove( FLOAT fTimeKey )
{
	D3DMATRIX m,m1,m2;
	static float x = 0.0f;
	
	x += 0.01f;

	SetRotateYMatrix(m,x);
	SetRotateXMatrix(m1,5*x);
	MatrixMultiply(m,m1,m);
	m_spl[m_current]->SetObj2WorldMatrix(m);


    return S_OK;
}




HRESULT CMyD3DApplication::Render()
{
    // Clear the viewport
    m_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                         0x000000ff, 1.0f, 0L );

    // Begin the scene 
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
		
		RenderCurrentSpline();

        // End the scene.
        m_pd3dDevice->EndScene();
    }


    return S_OK;
}




HRESULT CMyD3DApplication::InitDeviceObjects()
{
	m_pd3dDevice->SetRenderState(D3DRENDERSTATE_ZENABLE,TRUE);
	m_pd3dDevice->SetRenderState(D3DRENDERSTATE_CULLMODE,D3DCULL_NONE);
	m_pd3dDevice->SetRenderState(D3DRENDERSTATE_CLIPPING,FALSE);
    return S_OK;
}




HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    
    return S_OK;
}




HRESULT CMyD3DApplication::FinalCleanup()
{
    return S_OK;
}




HRESULT CMyD3DApplication::ConfirmDevice( DDCAPS* pddDriverCaps,
                                          D3DDEVICEDESC7* pd3dDeviceDesc )
{
        return S_OK;
}




void	CMyD3DApplication::RenderCurrentSpline() 
{
	DWORD cnt = m_spl[m_current]->GetPatchCount();
	LPWORD indices;
	DWORD indicesCount;
	D3DVERTEXBUFFERDESC vbdesc;
	LPDIRECT3D7 pD3D;
	LPDIRECT3DVERTEXBUFFER7 vb;

	ZeroMemory( &vbdesc, sizeof(D3DVERTEXBUFFERDESC) );
	vbdesc.dwSize        = sizeof(D3DVERTEXBUFFERDESC);
	vbdesc.dwCaps        = D3DVBCAPS_WRITEONLY | D3DVBCAPS_DONOTCLIP;
	vbdesc.dwFVF         = D3DFVF_TLVERTEX;
	vbdesc.dwNumVertices = 10000;
	m_pd3dDevice->GetDirect3D(&pD3D);
	for (DWORD i=0;i<cnt;i++) {
		pD3D->CreateVertexBuffer( &vbdesc, &vb, 0L );
		m_spl[m_current]->EvalPatch(i,vb,&m_scene,indices,indicesCount);
		m_pd3dDevice->DrawIndexedPrimitiveVB(D3DPT_TRIANGLELIST,vb,0,1000,indices,indicesCount,D3DDP_WAIT);
		vb->Release();
	}
	pD3D->Release();

}

LRESULT CMyD3DApplication::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam ) {
	if (uMsg == WM_COMMAND) {
		switch (wParam) {
		case  ID_MODE_X87 :   
			m_current = 0;
			break;
		case  ID_MODE_SOA :                   
			m_current = 2;
			break;
		case  ID_MODE_SOANORMALINTERPOLATION  :
			m_current = 3;
			break;
		case  ID_MODE_SOACOLORINTERPOLATION   :
			m_current = 4;
			break;
		case  ID_MODE_AOS :
			m_current = 1;
			break;
		case  ID_LEVEL_2  :                   
			SetTeslLevel(2);
			break;
		case  ID_LEVEL_4  :                    
			SetTeslLevel(4);
			break;
		case  ID_LEVEL_6  :                    
			SetTeslLevel(6);
			break;
		case  ID_LEVEL_10 :                   
			SetTeslLevel(10);
			break;
		case  ID_LEVEL_18 :                    
			SetTeslLevel(18);
			break;
		case  ID_LEVEL_24 :                    
			SetTeslLevel(24);
			break;
		case  ID_LIGHTS_NONE :    
			m_scene.SetLightingState(0);
			break;
		case  ID_LIGHTS_1    :                
			m_scene.SetLightingState(1);
			break;
		case  ID_LIGHTS_2    :                
			m_scene.SetLightingState(3);
			break;
		case  ID_LIGHTS_3    :                 
			m_scene.SetLightingState(7);
			break;
		}
	}
	return CD3DApplication::MsgProc(hWnd,uMsg,wParam,lParam);

}

