#ifndef _PIPE_DEF_H__
#define _PIPE_DEF_H__

#include <fvec.h>


/*************************************************************/
/*                  Scalar Pipe Definitions                  */
/*************************************************************/


struct SVector {
	float x,y,z;
	SVector (float ax=0,float ay=0,float az=0) : x(ax),y(ay),z(az) {}
};

struct SColor {
	float r,g,b;
	SColor(float ar = 0,float ag = 0,float ab = 0) : r(ar),g(ag),b(ab){}
};


#define POINT_LIGHT 0
#define DIR_LIGHT 1


struct SLight {
	SVector dir;
	SVector pos;
	SColor	col;
	DWORD   type;
};

struct SVertex {
	float x,y,z;
	float nx,ny,nz;
	float tu,tv;

	SVertex(const SVector& pos,const SVector &norm, float u, float v) :
		x(pos.x),y(pos.y),z(pos.z),
		nx(norm.x),ny(norm.y),nz(norm.z),
		tu(u),tv(v) {}
};



/*************************************************************/
/*                  SIMD Pipe Definitions                    */
/*************************************************************/


struct STexture {
	float u,v;
};

struct SSimdVector {
	F32vec4 x,y,z;
};

struct SSimdVertex {
	F32vec4 x,y,z;
	F32vec4 nx,ny,nz;
	STexture	tex[4];		// texture data saved as is (not SOA).
};


struct SSimdColor {
	F32vec4 r,g,b;
};


struct SSimdLight {
	SSimdVector dir;
	SSimdVector pos;
	SSimdColor	col;
	DWORD   type;
	DWORD	stub[3];		// Align to 16 bytes.
};



struct SSimdMatrix {
	F32vec4	_11,_12,_13,_14;
	F32vec4	_21,_22,_23,_24;
	F32vec4	_31,_32,_33,_34;
	F32vec4	_41,_42,_43,_44;
};


#endif

