//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#include "scene.h"
#include "util.h"

void CScene::SetViewPoint(D3DVECTOR from,D3DVECTOR at, D3DVECTOR up) 
{
	::SetViewMatrix( m_world2view,from,at, up);
	m_matrixChanged = true;
}

void CScene::SetViewMatrix(D3DMATRIX &mat) 
{
	m_world2view = mat;
	m_matrixChanged = true;
}

void CScene::SetLightData(DWORD lightNum,const Light& light) 
{
	if (lightNum > 7)
		return;
	m_lights[lightNum] = light;
}

void CScene::SetProjectionMatrix(float fFOV, float fAspect, float fNearPlane, float fFarPlane)
{
	::SetProjectionMatrix(m_proj,fFOV,fAspect,fNearPlane,fFarPlane);
	m_matrixChanged = true;
}

void CScene::SetProjectionMatrix(D3DMATRIX &mat)
{
	m_proj = mat;
	m_matrixChanged = true;
}

void CScene::SetLightingState(BYTE state)
{
	m_lightingState = state;
}

void CScene::GetWorld2ViewMatrix(D3DMATRIX& mat)
{
	mat = m_world2view;
	m_matrixChanged = true;
}

void CScene::GetWorld2ProjMatrix(D3DMATRIX& mat)
{
	D3DMATRIX m;
	MatrixMultiply(m,m_proj,m_world2view);
	mat = m;
}


void CScene::SetScreenScalingAndTranslation(const D3DVECTOR scaling,const D3DVECTOR trans) {
	m_scaling = scaling;
	m_translation = trans;
}
