#include "common.h"
#include "scene.h"
#include <stdlib.h>
#include <stdio.h>


DWORD		g_hBackMat;
LPDIRECT3DMATERIAL2 g_lpmatBackground;



CScene::CScene() : m_Moon(11),m_Earth(19),m_Sun(29),m_Venus(21),m_Mars(19) {
	m_Comp = IdentityMatrix();
	m_Recalc = FALSE;
	m_nLights = 0;
	mode = 1;
}

void CScene::SetViewMatrix(D3DMATRIX &mat) {
	m_View = mat;
	m_Recalc = TRUE;
}


void CScene::SetProjMatrix(D3DMATRIX &mat) {
	m_Proj = mat;
	m_Recalc = TRUE;
}


const D3DMATRIX& CScene::GetCompositeMatrix() {
	if (m_Recalc) {
		//m_iView = MatrixInverse(m_View);
		m_Comp = MatrixMult(m_Proj,m_View);
		//m_Comp = MatrixTranspose(m_Comp);
		m_Recalc = FALSE;
	}
	return m_Comp;
}



HRESULT CScene::Fini (void)
{
	FiniViewport ();

	return DD_OK;
} 


 
 

HRESULT CScene::Render (void)
{
	static float moon_dist = 0.16f;
	static float moon_lastorbitpos = 0.0f;
	static float moon_lastselfpos = 0.00f;

	static float earth_lastorbitpos = 0.0f;
	static float earth_lastselfpos = 0.00f;
	static float earth_dist = 0.45f;

	static float venus_lastorbitpos = 0.0f;
	static float venus_lastselfpos = 0.00f;
	static float venus_dist = 0.7f;

	static float mars_lastorbitpos = 0.0f;
	static float mars_lastselfpos = 0.00f;
	static float mars_dist = 0.85f;

	static float sun_lastselfpos = 0.00f;

	static DWORD count = 0,fpsCount = 0,verts = 0,pipe_time = 1;
	static int FrameCounter = 0;
	static float FrameRate = 0.0f;
	
	LPDIRECT3DDEVICE2	lpDev;
	LPDIRECT3DVIEWPORT2 lpView; 	
    D3DRECT				d3dRect;
	D3DCLIPSTATUS		status;
	HRESULT				hResult;
	DWORD				newCount;
	float Hdiv2,Wdiv2;
	DWORD time_a,time_b;

	// moon
	D3DMATRIX MoonMatrix = IdentityMatrix();
	D3DMATRIX EarthMatrix = IdentityMatrix();
	D3DMATRIX SunMatrix = IdentityMatrix();
	D3DMATRIX VenusMatrix = IdentityMatrix();
	D3DMATRIX MarsMatrix = IdentityMatrix();

	EarthMatrix = MatrixMult(ScaleMatrix(0.2,0.2,0.2),EarthMatrix);
	EarthMatrix = MatrixMult(RotateYMatrix(earth_lastselfpos += 0.1),EarthMatrix);
	EarthMatrix = MatrixMult(TranslateMatrix(earth_dist,0,0),EarthMatrix);
	EarthMatrix = MatrixMult(RotateYMatrix(earth_lastorbitpos += 0.005),EarthMatrix);
	EarthMatrix = MatrixMult(RotateXMatrix(-pi/4),EarthMatrix);

	VenusMatrix = MatrixMult(ScaleMatrix(0.25,0.25,0.25),VenusMatrix);
	VenusMatrix = MatrixMult(RotateXMatrix(venus_lastselfpos += 0.05),VenusMatrix);
	VenusMatrix = MatrixMult(TranslateMatrix(0,venus_dist,0),VenusMatrix);
	VenusMatrix = MatrixMult(RotateZMatrix(venus_lastorbitpos += 0.0048),VenusMatrix);
	VenusMatrix = MatrixMult(RotateZMatrix(1),VenusMatrix);
	
	MarsMatrix = MatrixMult(ScaleMatrix(0.20,0.20,0.20),MarsMatrix);
	MarsMatrix = MatrixMult(RotateXMatrix(mars_lastselfpos += 0.1),MarsMatrix);
	MarsMatrix = MatrixMult(RotateYMatrix(pi/2),MarsMatrix);
	MarsMatrix = MatrixMult(TranslateMatrix(mars_dist,0,0),MarsMatrix);
	MarsMatrix = MatrixMult(RotateYMatrix(mars_lastorbitpos += 0.0052),MarsMatrix);

	
	MoonMatrix = MatrixMult(ScaleMatrix(0.08,0.08,0.08),MoonMatrix);
	MoonMatrix = MatrixMult(RotateYMatrix(moon_lastselfpos += 0.1),MoonMatrix);
	MoonMatrix = MatrixMult(TranslateMatrix(moon_dist,0,0),MoonMatrix);
	MoonMatrix = MatrixMult(RotateYMatrix(moon_lastorbitpos += 0.05),MoonMatrix);
	MoonMatrix = MatrixMult(TranslateMatrix(earth_dist,0,0),MoonMatrix);
	MoonMatrix = MatrixMult(RotateYMatrix(earth_lastorbitpos),MoonMatrix);
	MoonMatrix = MatrixMult(RotateXMatrix(-pi/4),MoonMatrix);
	
	SunMatrix = MatrixMult(ScaleMatrix(0.30,0.30,0.30),SunMatrix);
	SunMatrix = MatrixMult(RotateYMatrix(sun_lastselfpos += 0.01),SunMatrix);
	SunMatrix = MatrixMult(RotateZMatrix(-pi/4),SunMatrix);

	m_Moon.SetWorldMatrix(MoonMatrix);
	m_Earth.SetWorldMatrix(EarthMatrix);
	m_Sun.SetWorldMatrix(SunMatrix);
	m_Venus.SetWorldMatrix(VenusMatrix);
	m_Mars.SetWorldMatrix(MarsMatrix);


	lpDev	= lpd3dWindow->lpd3dDevice;
	lpView  = lpd3dWindow->lpd3dViewport;


    
	d3dRect.lX1 = 0;
	d3dRect.lX2 = WIDTH;
	d3dRect.lY1 = 0;
	d3dRect.lY2 = HEIGHT;


	hResult = lpView->Clear (1UL, &d3dRect,
							 D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER);
	if (FAILED (hResult))
	{
		return hResult;
	}


	hResult = lpDev->BeginScene ();

	if (FAILED (hResult))
	{
		return hResult;
	}
    hResult = lpDev->SetRenderState (D3DRENDERSTATE_ZENABLE, 1);
	if (FAILED (hResult))
	{
		return hResult;
	}

	FrameCounter++;
	static char Buff[256];

	if ((newCount = GetTickCount()) - fpsCount > 1000) {
		count++;
		FrameRate = (float)(FrameCounter*1000)/(newCount-fpsCount);
		sprintf(Buff,"%s - %8.3f fps - %5d KVerts/Sec",(mode == 0) ? "Legacy pipeline" : "SIMD pipeline",FrameRate,verts/pipe_time);
		fpsCount = newCount;
		FrameCounter = 1;
		pipe_time = 1;
		verts = 0;
	}


	time_a = GetTickCount();
	if (!mode) {
		m_Moon.BasicPipeline(this);
		m_Earth.BasicPipeline(this);
		m_Sun.BasicPipeline(this);
		m_Venus.BasicPipeline(this);
		m_Mars.BasicPipeline(this);
	} else {
		m_Moon.SimdPipeline(this);
		m_Earth.SimdPipeline(this);
		m_Sun.SimdPipeline(this);
		m_Venus.SimdPipeline(this);
		m_Mars.SimdPipeline(this);
	}
	time_b = GetTickCount();
	pipe_time += time_b - time_a;
	verts += m_Moon.GetVertexCount();
	verts += m_Earth.GetVertexCount();
	verts += m_Sun.GetVertexCount();
	verts += m_Venus.GetVertexCount();
	verts += m_Mars.GetVertexCount();

	
	
	HDC hdc = GetDC(lpd3dWindow->hWnd);
	SetTextColor(hdc,(mode) ? 0x00ffffff : 0x0000ffff);
	SetBkMode(hdc,TRANSPARENT);
	TextOut(hdc,0,0,Buff,strlen(Buff));
	ReleaseDC(lpd3dWindow->hWnd,hdc);

    hResult = lpDev->SetRenderState (D3DRENDERSTATE_TEXTUREHANDLE, NULL);
	m_Moon.Render(lpDev);
	m_Earth.Render(lpDev);
	m_Mars.Render(lpDev);
	m_Venus.Render(lpDev);

    hResult = lpDev->SetRenderState (D3DRENDERSTATE_TEXTUREHANDLE, m_SunTex.GetHandle());
	m_Sun.Render(lpDev);

	lpDev->EndScene();
	if (FAILED (hResult))
	{
		return hResult;
	}
    return DD_OK;
} 


  

HRESULT CScene::Restore (void)
{
	return DD_OK;
} 




HRESULT CScene::InitViewport (void)
{
	LPDIRECT3D2			lpD3D;
	LPDIRECT3DDEVICE2   lpDev;
	LPDIRECT3DVIEWPORT2 lpViewport;
	HRESULT				hResult;
    D3DMATERIAL			matBackground, mat;
    D3DMATERIALHANDLE	hBackMat, hMat;


	lpD3D  = lpd3dWindow->lpD3D;
	lpDev  = lpd3dWindow->lpd3dDevice;
	lpViewport = lpd3dWindow->lpd3dViewport;

   
    hResult = lpD3D->CreateMaterial (&g_lpmatBackground, NULL);
	if (FAILED (hResult))
	{
		return hResult;
    }

    memset (&matBackground, 0, sizeof(D3DMATERIAL));
    matBackground.dwSize = sizeof(D3DMATERIAL);
    matBackground.diffuse.r = (D3DVALUE)0.1;
    matBackground.diffuse.g = (D3DVALUE)0.1;
    matBackground.diffuse.b = (D3DVALUE)0.3;
    matBackground.ambient.r = (D3DVALUE)1.0;
    matBackground.ambient.g = (D3DVALUE)1.0;
    matBackground.ambient.b = (D3DVALUE)1.0;
//    matBackground.hTexture = m_BackTex.GetHandle();
	matBackground.dwRampSize = 1;
    
	hResult = g_lpmatBackground->SetMaterial (&matBackground);
	if (FAILED (hResult))
	{
		return hResult;
	}
    
	hResult = g_lpmatBackground->GetHandle (lpDev, &hBackMat);
	if (FAILED (hResult))
	{
		return hResult;
	}

    hResult = lpViewport->SetBackground (hBackMat);
	if (FAILED (hResult))
	{
		return hResult;
	}

	//g_hBackMat = hBackMat;

    return DD_OK;
} 



HRESULT CScene::FiniViewport (void)
{
	HRESULT hResult;
	LPDIRECT3DVIEWPORT2 lpViewport;

	// Check Initialization
	if (! lpd3dWindow)
	{
		// Error,
		return DDERR_GENERIC;
	}

	lpViewport = lpd3dWindow->lpd3dViewport;

	// Cleanup Background
	if (g_lpmatBackground)
	{
		hResult = g_lpmatBackground->Release ();
		if (FAILED (hResult))
		{
		}

		g_lpmatBackground = NULL;
		g_hBackMat = NULL;
	}

	// Success
	return DD_OK;
} 




HRESULT CScene::Init (CWindow *lpd3dWin)
{
	HRESULT hResult;
	D3DMATRIX view = IdentityMatrix();
	D3DMATRIX proj = IdentityMatrix();
	SLight    light;

	if (! lpd3dWin)
	{
		return DDERR_INVALIDPARAMS;
	}

	lpd3dWindow = lpd3dWin;

	view  = IdentityMatrix();
	view = MatrixMult(TranslateMatrix(0,0.5,0.5),view);
	view = MatrixMult(RotateXMatrix(-pi/4 ),view);
	view = MatrixMult(ScaleMatrix(1.9,1.9,0.5),view);
	SetViewMatrix(view);
	proj._34 = 1;
	proj = MatrixMult(ScaleMatrix(min(WIDTH/2,HEIGHT/2),min(WIDTH/2,HEIGHT/2),1),proj);
	proj = MatrixMult(TranslateMatrix(WIDTH/2,HEIGHT/2,0),proj);
	SetProjMatrix(proj);

	m_SunTex.Load(lpd3dWin->lpd3dDevice,"PIII.bmp");

	m_Sun.SetAmbientColor(SColor(0.9,0.9,0.9));

	m_Moon.SetMaterialColor(SColor(1,1,1));
	m_Earth.SetMaterialColor(SColor(1,1,1));
	m_Mars.SetMaterialColor(SColor(1,0.5,0.5));
	m_Sun.SetMaterialColor(SColor(0.25,0.25,0.25));
	m_Venus.SetMaterialColor(SColor(0.8,0.8,1));

	light.pos = SVector(0,0,0);
	light.col = SColor(1,1,1);
	light.type = POINT_LIGHT;

	AddLight(light);

	light.dir = SVector(0,1,1);
	light.col = SColor(0.1,0.1,0.4);
	light.type = DIR_LIGHT;

	AddLight(light);

	


	// Initialize Viewport
	hResult = InitViewport ();
	if (FAILED (hResult))
		return hResult;

    return DD_OK;
} // End D3DScene::Init



HRESULT	CScene::AddLight(SLight &light) {
	if (m_nLights - 1 == MAX_LIGHTS)
		return TRUE;
	m_Lights[m_nLights++] = light;
	return DD_OK;
}

DWORD CScene::GetLightCount()
{
	return m_nLights;
}

SLight CScene::GetLight(int i)
{
	return m_Lights[i];
}


CScene::~CScene() {}
