//
//   Copyright (c) 1999-2000 Intel Corporation.
//
// Permition is granted to use, copy, distribute and prepare derivative works 
// of this library for any purpose and without fee, provided, that the above 
// copyright notice and this statement appear in all copies.  
// Intel makes no representations about the suitability of this software for 
// any purpose, and specifically disclaims all warranties. 
// See LEGAL.TXT for all the legal information.
//

#ifndef _SCENE_H_
#define _SCENE_H_

#include "d3d.h"

class CScene {
public:
	struct Light {
//		union {
			D3DVECTOR	pos,norm;
//			D3DVECTOR	norm;
//		};
		D3DVECTOR color;
		enum LightType {
			DIRECTIONAL_LIGHT,
			POINT_LIGHT
		} type;
	};

private:
	D3DMATRIX m_world2view;
	D3DMATRIX m_world2proj;
	D3DMATRIX m_proj;
	D3DVECTOR m_lightPosition;
	D3DVECTOR m_scaling;
	D3DVECTOR m_translation;
	D3DVECTOR m_ambientColor;
	Light	  m_lights[8];
	BYTE	  m_lightingState;
	bool	  m_matrixChanged;
public:
	void SetViewPoint(const D3DVECTOR from,const D3DVECTOR at, const D3DVECTOR up);
	void SetViewMatrix(D3DMATRIX &mat);
	void SetLightData(DWORD lightNum,const Light& light);
	void SetProjectionMatrix(float fFOV, float fAspect, float fNearPlane, float fFarPlane);
	void SetProjectionMatrix(D3DMATRIX& mat);
	void SetLightingState(BYTE state);

	void GetWorld2ViewMatrix(D3DMATRIX& mat);
	void GetWorld2ProjMatrix(D3DMATRIX& mat);
	bool isMatrixChanged() {return 	m_matrixChanged;}
	void SetScreenScalingAndTranslation(const D3DVECTOR scaling,const D3DVECTOR trans);
	D3DVECTOR GetScreenScaling() {return m_scaling;}
	D3DVECTOR GetScreenTranslation() {return m_translation;}
	
	const Light	&GetLight(DWORD num) {return m_lights[num];}

	BYTE GetLightingState() {return m_lightingState;}

};



#endif _SCENE_H_
