#include "common.h"
#include "scene.h"

extern HINSTANCE hInst;

void CWindow::Init() {
	WNDCLASS wc;

	if (InitFinished)
		return;

	hInst = GetModuleHandle(NULL);

    wc.style            = CS_DBLCLKS; 
    wc.lpfnWndProc      = (WNDPROC)CWindow::SolarSystemProc;	
    wc.cbClsExtra       = 0; 
    wc.cbWndExtra       = 0;						
    wc.hInstance        = hInst; 
    wc.hIcon            = LoadIcon (hInst, MAKEINTRESOURCE (IDI_ICON1));
    wc.hCursor          = LoadCursor ((HINSTANCE)NULL, IDC_ARROW); 
    wc.hbrBackground    = (HBRUSH)GetStockObject (WHITE_BRUSH);
    wc.lpszMenuName     = MAKEINTRESOURCE(IDR_MENU1);
    wc.lpszClassName    = CLASS_NAME; 

    if (! RegisterClass (&wc))
    {
        exit(1);
    }


	hWnd = CreateWindow(CLASS_NAME,
						"Solar System Demo",
						WS_OVERLAPPEDWINDOW,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						WIDTH,
						HEIGHT,
						NULL,
						NULL,
						hInst,
						NULL);

	if (hWnd == NULL)
		exit(1);


	InitDirect3D();

	lpScene = new CScene;
	lpScene->Init(this);

	ShowWindow(hWnd,SW_SHOW);
	UpdateWindow(hWnd);

	InitFinished = TRUE;

}



void CWindow::Fini() {
	delete lpScene;
	FiniDirect3D();
	DestroyWindow(hWnd);
}





void CWindow::InitDirect3D() {
    DDSURFACEDESC		ddsd;
    LPDIRECTDRAWCLIPPER lpddcClipper = NULL;    
	D3DVIEWPORT2	d3dViewport;
	int i;
	HRESULT hr;

	
	
	hr = DirectDrawCreate(NULL,&lpDD,NULL);
	VALIDATE(hr);
	hr = lpDD->QueryInterface(IID_IDirectDraw2,(LPVOID *)&lpDD2);
	VALIDATE(hr);
	hr = lpDD2->QueryInterface(IID_IDirect3D2,(LPVOID *)&lpD3D);
	VALIDATE(hr);

	hr = lpDD2->SetCooperativeLevel(hWnd,DDSCL_NORMAL);
	VALIDATE(hr);



    ddsd.dwSize         = sizeof(ddsd);
    ddsd.dwFlags        = DDSD_CAPS;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    hr = lpDD2->CreateSurface (&ddsd, &lpddsPrimary, NULL);
	VALIDATE(hr);

    hr = lpDD2->CreateClipper (0L, &lpddcClipper, 0);
	VALIDATE(hr);    
	
    hr = lpddcClipper->SetHWnd (0L, hWnd);
	VALIDATE(hr);    

    hr = lpddsPrimary->SetClipper (lpddcClipper);
	VALIDATE(hr);    

    RELEASE(lpddcClipper)


    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof (DDSURFACEDESC);
    hr = lpddsPrimary->GetSurfaceDesc (&ddsd);
	VALIDATE(hr);

    ddsd.dwSize         = sizeof(ddsd);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT /*| DDSD_PIXELFORMAT*/;
    ddsd.dwWidth        = WIDTH;
    ddsd.dwHeight       = HEIGHT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_3DDEVICE | DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;


	hr = lpDD2->CreateSurface (&ddsd, &lpddsRender, NULL);
	VALIDATE(hr);

    ZeroMemory (&ddsd, sizeof(ddsd));
    ddsd.dwSize            = sizeof(ddsd);
    ddsd.dwFlags           = DDSD_CAPS   |
                             DDSD_WIDTH  |
                             DDSD_HEIGHT |
                             DDSD_ZBUFFERBITDEPTH;
    ddsd.ddsCaps.dwCaps    = DDSCAPS_ZBUFFER | DDSCAPS_VIDEOMEMORY;
    ddsd.dwWidth           = WIDTH;
    ddsd.dwHeight          = HEIGHT;
    ddsd.dwZBufferBitDepth = 16;

    hr = lpDD2->CreateSurface (&ddsd, &lpddsZBuff, NULL);
	VALIDATE(hr);

	hr = lpddsRender->AddAttachedSurface (lpddsZBuff);
	VALIDATE(hr);


	hr = lpD3D->CreateDevice (IID_IDirect3DHALDevice,
				 			  lpddsRender, 
							  &lpd3dDevice);
	if (hr)
		hr = lpD3D->CreateDevice (IID_IDirect3DMMXDevice,
					 			  lpddsRender, 
								  &lpd3dDevice);

	if (hr)
		hr = lpD3D->CreateDevice (IID_IDirect3DRGBDevice,
					 			  lpddsRender, 
								  &lpd3dDevice);

	VALIDATE(hr);


    hr = lpD3D->CreateViewport (&lpd3dViewport, NULL);
	VALIDATE(hr);

    hr = lpd3dDevice->AddViewport (lpd3dViewport);
	VALIDATE(hr);

	

    ZeroMemory (&d3dViewport, sizeof(d3dViewport));
    d3dViewport.dwSize		= sizeof(d3dViewport);  
    d3dViewport.dwX			= 0UL;
    d3dViewport.dwY			= 0UL;
    d3dViewport.dwWidth		= WIDTH;
    d3dViewport.dwHeight	= HEIGHT;
	d3dViewport.dvClipX = 0;
	d3dViewport.dvClipY = 0;
	d3dViewport.dvClipWidth = WIDTH;
	d3dViewport.dvClipHeight = HEIGHT;
	d3dViewport.dvMinZ = 0.0f;
	d3dViewport.dvMaxZ = 1.0f;

    hr = lpd3dViewport->SetViewport2 (&d3dViewport);
	VALIDATE(hr);

    hr = lpd3dDevice->SetCurrentViewport (lpd3dViewport);
	VALIDATE(hr);

}

void CWindow::FiniDirect3D() {
	HRESULT hr;

    hr = lpd3dDevice->DeleteViewport (lpd3dViewport);
	VALIDATE(hr);
	RELEASE(lpd3dViewport);
	RELEASE(lpd3dDevice);

	hr = lpddsRender->DeleteAttachedSurface (0,lpddsZBuff);
	VALIDATE(hr);
	RELEASE(lpddsZBuff);

	RELEASE(lpddsRender);
	RELEASE(lpddsPrimary);
	RELEASE(lpDD2);
	RELEASE(lpD3D);
	RELEASE(lpDD);
}



void CWindow::OnIdle() {
	if (isInit()) {
		if (Animate)
			lpScene->Render();
		DrawScene();
	}

}






LRESULT CALLBACK CWindow::SolarSystemProc (
	    HWND    hWindow,
	    UINT    uiMsg,
	    WPARAM  wParam,
		LPARAM  lParam) {

	switch (uiMsg) {
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
    case WM_COMMAND:
		Wnd.OnCommand(wParam);
		break;
    case WM_PAINT:
    {
        HDC         hdc;
        PAINTSTRUCT ps;

        hdc = BeginPaint (hWindow, &ps); 
		Wnd.OnPaint(hdc);
        EndPaint (hWindow, &ps);
    }
		break;
	default:
		return DefWindowProc(hWindow,uiMsg,wParam,lParam);
	}
	return 0;
}

void CWindow::OnCommand(WPARAM wParam)
{
	if (!isInit())
		return;
	switch (wParam) {
	case ID_SINGLESTEP:
		lpScene->Render();
		Animate = FALSE;
		break;
	case ID_GO:
		Animate = TRUE;
		break;
	case ID_EXIT:
		PostQuitMessage(0);
		break;
	case ID_ABOUT:
		DialogBox(hInst,MAKEINTRESOURCE(IDD_DIALOG1),hWnd,(DLGPROC)DlgAboutProc);
		break;
	case ID_PIPELINE_CHANGEPIPELINE:
		lpScene->ChangeMode();
		break;
	case ID_PIPELINE_SIMDPIPELINE:
		lpScene->SetMode(1);
		break;
	case ID_PIPELINE_LEGACYPIPELINE:
		lpScene->SetMode(0);
		break;

	default:
		return;
	}
	return;
}

void CWindow::OnPaint(HDC hdc)
{
	DrawScene();
}

void CWindow::DrawScene()
{
	if (!isInit())
		return;

	RECT rect = {0,0,WIDTH,HEIGHT};
	RECT dstRect;
	GetWindowRect(hWnd,&dstRect);
	lpddsPrimary->Blt (&dstRect,	// Dest Rect
	                   lpddsRender,   // Src Surface
	                   &rect,     // Src Rect
	                   DDBLT_WAIT,    // Wait
					   NULL);         // Blt Special Effects (none)
}
